@@ -1,10 +1,12 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package oss
 
 import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"github.com/aliyun/alibaba-cloud-sdk-go/sdk/endpoints"
 	"io/ioutil"
 	"log"
 	"net/http"
@@ -16,6 +18,8 @@ import (
 	"strings"
 	"time"
 
+	"github.com/aliyun/alibaba-cloud-sdk-go/sdk/endpoints"
+
 	"github.com/aliyun/alibaba-cloud-sdk-go/sdk"
 	"github.com/aliyun/alibaba-cloud-sdk-go/sdk/auth/credentials"
 	"github.com/aliyun/alibaba-cloud-sdk-go/sdk/requests"
@@ -36,10 +40,10 @@ import (
 // Deprecated in favor of flattening assume_role_* options
 func deprecatedAssumeRoleSchema() *schema.Schema {
 	return &schema.Schema{
-		Type:       schema.TypeSet,
-		Optional:   true,
-		MaxItems:   1,
-		Deprecated: "use assume_role_* options instead",
+		Type:     schema.TypeSet,
+		Optional: true,
+		MaxItems: 1,
+		//Deprecated: "use assume_role_* options instead",
 		Elem: &schema.Resource{
 			Schema: map[string]*schema.Schema{
 				"role_arn": {
@@ -379,12 +383,13 @@ func (b *Backend) configure(ctx context.Context) error {
 	if endpoint == "" {
 		endpointsResponse, err := b.getOSSEndpointByRegion(accessKey, secretKey, securityToken, region)
 		if err != nil {
-			return err
-		}
-		for _, endpointItem := range endpointsResponse.Endpoints.Endpoint {
-			if endpointItem.Type == "openAPI" {
-				endpoint = endpointItem.Endpoint
-				break
+			log.Printf("[WARN] getting oss endpoint failed and using oss-%s.aliyuncs.com instead. Error: %#v.", region, err)
+		} else {
+			for _, endpointItem := range endpointsResponse.Endpoints.Endpoint {
+				if endpointItem.Type == "openAPI" {
+					endpoint = endpointItem.Endpoint
+					break
+				}
 			}
 		}
 		if endpoint == "" {
