@@ -11,7 +11,7 @@ import (
 	tfe "github.com/hashicorp/go-tfe"
 	"github.com/zclconf/go-cty/cty"
 
-	"github.com/hashicorp/terraform/internal/backend"
+	"github.com/hashicorp/terraform/internal/backend/backendrun"
 	"github.com/hashicorp/terraform/internal/command/arguments"
 	"github.com/hashicorp/terraform/internal/command/clistate"
 	"github.com/hashicorp/terraform/internal/command/views"
@@ -90,7 +90,7 @@ func TestRemoteStoredVariableValue(t *testing.T) {
 		"HCL computation": {
 			// This (stored expressions containing computation) is not a case
 			// we intentionally supported, but it became possible for remote
-			// operations in Terraform 0.12 (due to Terraform Cloud/Enterprise
+			// operations in Terraform 0.12 (due to HCP Terraform and Terraform Enterprise
 			// just writing the HCL verbatim into generated `.tfvars` files).
 			// We support it here for consistency, and we continue to support
 			// it in both places for backward-compatibility. In practice,
@@ -197,7 +197,7 @@ func TestRemoteContextWithVars(t *testing.T) {
 			streams, _ := terminal.StreamsForTesting(t)
 			view := views.NewStateLocker(arguments.ViewHuman, views.NewView(streams))
 
-			op := &backend.Operation{
+			op := &backendrun.Operation{
 				ConfigDir:    configDir,
 				ConfigLoader: configLoader,
 				StateLocker:  clistate.NewLocker(0, view),
@@ -253,12 +253,12 @@ func TestRemoteVariablesDoNotOverride(t *testing.T) {
 	varValue3 := "value3"
 
 	tests := map[string]struct {
-		localVariables    map[string]backend.UnparsedVariableValue
+		localVariables    map[string]backendrun.UnparsedVariableValue
 		remoteVariables   []*tfe.VariableCreateOptions
 		expectedVariables terraform.InputValues
 	}{
 		"no local variables": {
-			map[string]backend.UnparsedVariableValue{},
+			map[string]backendrun.UnparsedVariableValue{},
 			[]*tfe.VariableCreateOptions{
 				{
 					Key:      &varName1,
@@ -307,7 +307,7 @@ func TestRemoteVariablesDoNotOverride(t *testing.T) {
 			},
 		},
 		"single conflicting local variable": {
-			map[string]backend.UnparsedVariableValue{
+			map[string]backendrun.UnparsedVariableValue{
 				varName3: testUnparsedVariableValue{source: terraform.ValueFromNamedFile, value: cty.StringVal(varValue3)},
 			},
 			[]*tfe.VariableCreateOptions{
@@ -356,7 +356,7 @@ func TestRemoteVariablesDoNotOverride(t *testing.T) {
 			},
 		},
 		"no conflicting local variable": {
-			map[string]backend.UnparsedVariableValue{
+			map[string]backendrun.UnparsedVariableValue{
 				varName3: testUnparsedVariableValue{source: terraform.ValueFromNamedFile, value: cty.StringVal(varValue3)},
 			},
 			[]*tfe.VariableCreateOptions{
@@ -420,7 +420,7 @@ func TestRemoteVariablesDoNotOverride(t *testing.T) {
 			streams, _ := terminal.StreamsForTesting(t)
 			view := views.NewStateLocker(arguments.ViewHuman, views.NewView(streams))
 
-			op := &backend.Operation{
+			op := &backendrun.Operation{
 				ConfigDir:    configDir,
 				ConfigLoader: configLoader,
 				StateLocker:  clistate.NewLocker(0, view),
