@@ -22,7 +22,7 @@ import (
 	tfe "github.com/hashicorp/go-tfe"
 	version "github.com/hashicorp/go-version"
 
-	"github.com/hashicorp/terraform/internal/backend"
+	"github.com/hashicorp/terraform/internal/backend/backendrun"
 	"github.com/hashicorp/terraform/internal/cloud/cloudplan"
 	"github.com/hashicorp/terraform/internal/command/jsonformat"
 	"github.com/hashicorp/terraform/internal/configs"
@@ -33,7 +33,7 @@ import (
 
 var planConfigurationVersionsPollInterval = 500 * time.Millisecond
 
-func (b *Cloud) opPlan(stopCtx, cancelCtx context.Context, op *backend.Operation, w *tfe.Workspace) (*tfe.Run, error) {
+func (b *Cloud) opPlan(stopCtx, cancelCtx context.Context, op *backendrun.Operation, w *tfe.Workspace) (*tfe.Run, error) {
 	log.Printf("[INFO] cloud: starting Plan operation")
 
 	var diags tfdiags.Diagnostics
@@ -48,12 +48,22 @@ func (b *Cloud) opPlan(stopCtx, cancelCtx context.Context, op *backend.Operation
 		return nil, diags.Err()
 	}
 
+	if w.VCSRepo != nil && op.PlanOutPath != "" {
+		diags = diags.Append(tfdiags.Sourceless(
+			tfdiags.Error,
+			"Saved plans not allowed for workspaces with a VCS connection",
+			"A workspace that is connected to a VCS requires the VCS-driven workflow "+
+				"to ensure that the VCS remains the single source of truth.",
+		))
+		return nil, diags.Err()
+	}
+
 	if b.ContextOpts != nil && b.ContextOpts.Parallelism != defaultParallelism {
 		diags = diags.Append(tfdiags.Sourceless(
 			tfdiags.Error,
 			"Custom parallelism values are currently not supported",
-			`Terraform Cloud does not support setting a custom parallelism `+
-				`value at this time.`,
+			fmt.Sprintf("%s does not support setting a custom parallelism ", b.appName)+
+				"value at this time.",
 		))
 	}
 
@@ -61,8 +71,8 @@ func (b *Cloud) opPlan(stopCtx, cancelCtx context.Context, op *backend.Operation
 		diags = diags.Append(tfdiags.Sourceless(
 			tfdiags.Error,
 			"Displaying a saved plan is currently not supported",
-			`Terraform Cloud currently requires configuration to be present and `+
-				`does not accept an existing saved plan as an argument at this time.`,
+			fmt.Sprintf("%s currently requires configuration to be present and ", b.appName)+
+				"does not accept an existing saved plan as an argument at this time.",
 		))
 	}
 
@@ -108,18 +118,18 @@ func (b *Cloud) opPlan(stopCtx, cancelCtx context.Context, op *backend.Operation
 	return run, nil
 }
 
-func (b *Cloud) plan(stopCtx, cancelCtx context.Context, op *backend.Operation, w *tfe.Workspace) (*tfe.Run, error) {
+func (b *Cloud) plan(stopCtx, cancelCtx context.Context, op *backendrun.Operation, w *tfe.Workspace) (*tfe.Run, error) {
 	if b.CLI != nil {
-		header := planDefaultHeader
-		if op.Type == backend.OperationTypeApply || op.Type == backend.OperationTypeRefresh {
-			header = applyDefaultHeader
+		header := fmt.Sprintf(planDefaultHeader, b.appName)
+		if op.Type == backendrun.OperationTypeApply || op.Type == backendrun.OperationTypeRefresh {
+			header = fmt.Sprintf(applyDefaultHeader, b.appName)
 		}
 		b.CLI.Output(b.Colorize().Color(strings.TrimSpace(header) + "\n"))
 	}
 
 	// Plan-only means they ran terraform plan without -out.
 	provisional := op.PlanOutPath != ""
-	planOnly := op.Type == backend.OperationTypePlan && !provisional
+	planOnly := op.Type == backendrun.OperationTypePlan && !provisional
 
 	configOptions := tfe.ConfigurationVersionCreateOptions{
 		AutoQueueRuns: tfe.Bool(false),
@@ -129,7 +139,7 @@ func (b *Cloud) plan(stopCtx, cancelCtx context.Context, op *backend.Operation,
 
 	cv, err := b.client.ConfigurationVersions.Create(stopCtx, w.ID, configOptions)
 	if err != nil {
-		return nil, generalError("Failed to create configuration version", err)
+		return nil, b.generalError("Failed to create configuration version", err)
 	}
 
 	var configDir string
@@ -137,7 +147,7 @@ func (b *Cloud) plan(stopCtx, cancelCtx context.Context, op *backend.Operation,
 		// De-normalize the configuration directory path.
 		configDir, err = filepath.Abs(op.ConfigDir)
 		if err != nil {
-			return nil, generalError(
+			return nil, b.generalError(
 				"Failed to get absolute path of the configuration directory: %v", err)
 		}
 
@@ -175,21 +185,21 @@ in order to capture the filesystem context the remote workspace expects:
 		// be executed when we are destroying and doesn't need the config.
 		configDir, err = ioutil.TempDir("", "tf")
 		if err != nil {
-			return nil, generalError("Failed to create temporary directory", err)
+			return nil, b.generalError("Failed to create temporary directory", err)
 		}
 		defer os.RemoveAll(configDir)
 
 		// Make sure the configured working directory exists.
 		err = os.MkdirAll(filepath.Join(configDir, w.WorkingDirectory), 0700)
 		if err != nil {
-			return nil, generalError(
+			return nil, b.generalError(
 				"Failed to create temporary working directory", err)
 		}
 	}
 
 	err = b.client.ConfigurationVersions.Upload(stopCtx, cv.UploadURL, configDir)
 	if err != nil {
-		return nil, generalError("Failed to upload configuration files", err)
+		return nil, b.generalError("Failed to upload configuration files", err)
 	}
 
 	uploaded := false
@@ -202,7 +212,7 @@ in order to capture the filesystem context the remote workspace expects:
 		case <-time.After(planConfigurationVersionsPollInterval):
 			cv, err = b.client.ConfigurationVersions.Read(stopCtx, cv.ID)
 			if err != nil {
-				return nil, generalError("Failed to retrieve configuration version", err)
+				return nil, b.generalError("Failed to retrieve configuration version", err)
 			}
 
 			if cv.Status == tfe.ConfigurationUploaded {
@@ -212,7 +222,7 @@ in order to capture the filesystem context the remote workspace expects:
 	}
 
 	if !uploaded {
-		return nil, generalError(
+		return nil, b.generalError(
 			"Failed to upload configuration files", errors.New("operation timed out"))
 	}
 
@@ -235,9 +245,9 @@ in order to capture the filesystem context the remote workspace expects:
 		// Shouldn't get here because we should update this for each new
 		// plan mode we add, mapping it to the corresponding RunCreateOptions
 		// field.
-		return nil, generalError(
+		return nil, b.generalError(
 			"Invalid plan mode",
-			fmt.Errorf("Terraform Cloud doesn't support %s", op.PlanMode),
+			fmt.Errorf("%s doesn't support %s", b.appName, op.PlanMode),
 		)
 	}
 
@@ -281,7 +291,7 @@ in order to capture the filesystem context the remote workspace expects:
 
 	r, err := b.client.Runs.Create(stopCtx, runOptions)
 	if err != nil {
-		return r, generalError("Failed to create run", err)
+		return r, b.generalError("Failed to create run", err)
 	}
 
 	// When the lock timeout is set, if the run is still pending and
@@ -323,7 +333,7 @@ in order to capture the filesystem context the remote workspace expects:
 
 	if b.CLI != nil {
 		b.CLI.Output(b.Colorize().Color(strings.TrimSpace(fmt.Sprintf(
-			runHeader, b.Hostname, b.organization, op.Workspace, r.ID)) + "\n"))
+			runHeader, b.Hostname, b.Organization, op.Workspace, r.ID)) + "\n"))
 	}
 
 	// Render any warnings that were raised during run creation
@@ -357,7 +367,7 @@ in order to capture the filesystem context the remote workspace expects:
 	// Retrieve the run to get its current status.
 	r, err = b.client.Runs.Read(stopCtx, r.ID)
 	if err != nil {
-		return r, generalError("Failed to retrieve run", err)
+		return r, b.generalError("Failed to retrieve run", err)
 	}
 
 	// If the run is canceled or errored, we still continue to the
@@ -410,15 +420,15 @@ func (b *Cloud) AssertImportCompatible(config *configs.Config) error {
 		// Second, check the agent version is high enough.
 		agentEnv, isSet := os.LookupEnv("TFC_AGENT_VERSION")
 		if !isSet {
-			return fmt.Errorf("Error reading TFC agent version. To proceed, please remove any import blocks from your config. Please report the following error to the Terraform team: TFC_AGENT_VERSION not present.")
+			return fmt.Errorf("Error reading Terraform Cloud agent version. To proceed, please remove any import blocks from your config. Please report the following error to the Terraform team: TFC_AGENT_VERSION not present.")
 		}
 		currentAgentVersion, err := version.NewVersion(agentEnv)
 		if err != nil {
-			return fmt.Errorf("Error parsing TFC agent version. To proceed, please remove any import blocks from your config. Please report the following error to the Terraform team: %s", err)
+			return fmt.Errorf("Error parsing Terraform Cloud agent version. To proceed, please remove any import blocks from your config. Please report the following error to the Terraform team: %s", err)
 		}
 		desiredAgentVersion, _ := version.NewVersion("1.10")
 		if currentAgentVersion.LessThan(desiredAgentVersion) {
-			return fmt.Errorf("Import blocks are not supported in this version of the Terraform Cloud Agent. You are using agent version %s, but this feature requires version %s. Please remove any import blocks from your config or upgrade your agent.", currentAgentVersion, desiredAgentVersion)
+			return fmt.Errorf("Import blocks are not supported in this version of the HCP Terraform Agent. You are using agent version %s, but this feature requires version %s. Please remove any import blocks from your config or upgrade your agent.", currentAgentVersion, desiredAgentVersion)
 		}
 	}
 	return nil
@@ -426,7 +436,7 @@ func (b *Cloud) AssertImportCompatible(config *configs.Config) error {
 
 // renderPlanLogs reads the streamed plan JSON logs and calls the JSON Plan renderer (jsonformat.RenderPlan) to
 // render the plan output. The plan output is fetched from the redacted output endpoint.
-func (b *Cloud) renderPlanLogs(ctx context.Context, op *backend.Operation, run *tfe.Run) error {
+func (b *Cloud) renderPlanLogs(ctx context.Context, op *backendrun.Operation, run *tfe.Run) error {
 	logs, err := b.client.Plans.Logs(ctx, run.Plan.ID)
 	if err != nil {
 		return err
@@ -443,7 +453,7 @@ func (b *Cloud) renderPlanLogs(ctx context.Context, op *backend.Operation, run *
 				l, isPrefix, err = reader.ReadLine()
 				if err != nil {
 					if err != io.EOF {
-						return generalError("Failed to read logs", err)
+						return b.generalError("Failed to read logs", err)
 					}
 					next = false
 				}
@@ -511,13 +521,13 @@ func (b *Cloud) renderPlanLogs(ctx context.Context, op *backend.Operation, run *
 		return err
 	}
 	if renderSRO || shouldGenerateConfig {
-		jsonBytes, err := readRedactedPlan(ctx, b.client.BaseURL(), b.token, run.Plan.ID)
+		jsonBytes, err := readRedactedPlan(ctx, b.client.BaseURL(), b.Token, run.Plan.ID)
 		if err != nil {
-			return generalError("Failed to read JSON plan", err)
+			return b.generalError("Failed to read JSON plan", err)
 		}
 		redactedPlan, err = decodeRedactedPlan(jsonBytes)
 		if err != nil {
-			return generalError("Failed to decode JSON plan", err)
+			return b.generalError("Failed to decode JSON plan", err)
 		}
 	}
 
@@ -621,15 +631,20 @@ func shouldGenerateConfig(out string, run *tfe.Run) bool {
 }
 
 const planDefaultHeader = `
-[reset][yellow]Running plan in Terraform Cloud. Output will stream here. Pressing Ctrl-C
+[reset][yellow]Running plan in %s. Output will stream here. Pressing Ctrl-C
 will stop streaming the logs, but will not stop the plan running remotely.[reset]
 
 Preparing the remote plan...
 `
 
 const runHeader = `
-[reset][yellow]To view this run in a browser, visit:
-https://%s/app/%s/%s/runs/%s[reset]
+[reset][yellow]To view this run in a browser, visit:[reset]
+[reset][yellow]https://%s/app/%s/%s/runs/%s[reset]
+`
+
+const runHeaderErr = `
+To view this run in the browser, visit:
+https://%s/app/%s/%s/runs/%s
 `
 
 // The newline in this error is to make it look good in the CLI!
