@@ -6,7 +6,7 @@ package cloud
 import (
 	"github.com/hashicorp/hcl/v2/hclwrite"
 
-	"github.com/hashicorp/terraform/internal/backend"
+	"github.com/hashicorp/terraform/internal/backend/backendrun"
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/terraform"
 	"github.com/hashicorp/terraform/internal/tfdiags"
@@ -23,9 +23,9 @@ func allowedSourceType(source terraform.ValueSourceType) bool {
 // in order to allow callers to short circuit cloud runs that contain variable
 // declaration or parsing errors. The only exception is that missing required values are not
 // considered errors because they may be defined within the cloud workspace.
-func ParseCloudRunVariables(vv map[string]backend.UnparsedVariableValue, decls map[string]*configs.Variable) (map[string]string, tfdiags.Diagnostics) {
-	declared, diags := backend.ParseDeclaredVariableValues(vv, decls)
-	_, undedeclaredDiags := backend.ParseUndeclaredVariableValues(vv, decls)
+func ParseCloudRunVariables(vv map[string]backendrun.UnparsedVariableValue, decls map[string]*configs.Variable) (map[string]string, tfdiags.Diagnostics) {
+	declared, diags := backendrun.ParseDeclaredVariableValues(vv, decls)
+	_, undedeclaredDiags := backendrun.ParseUndeclaredVariableValues(vv, decls)
 	diags = diags.Append(undedeclaredDiags)
 
 	ret := make(map[string]string, len(declared))
@@ -52,7 +52,7 @@ func ParseCloudRunVariables(vv map[string]backend.UnparsedVariableValue, decls m
 // inside child modules as well as the main configuration and it is a lot of
 // effort to track down exactly where variable definitions exist. We just accept
 // all values.
-func ParseCloudRunTestVariables(globals map[string]backend.UnparsedVariableValue) (map[string]string, tfdiags.Diagnostics) {
+func ParseCloudRunTestVariables(globals map[string]backendrun.UnparsedVariableValue) (map[string]string, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 	ret := make(map[string]string, len(globals))
 
