@@ -7,19 +7,22 @@ import (
 	"bufio"
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
+	"net/url"
 	"path/filepath"
 	"strings"
 	"time"
 
 	"github.com/hashicorp/go-tfe"
 	tfaddr "github.com/hashicorp/terraform-registry-address"
+	svchost "github.com/hashicorp/terraform-svchost"
 	"github.com/hashicorp/terraform-svchost/disco"
 	"github.com/zclconf/go-cty/cty"
 
-	"github.com/hashicorp/terraform/internal/backend"
+	"github.com/hashicorp/terraform/internal/backend/backendrun"
 	"github.com/hashicorp/terraform/internal/command/format"
 	"github.com/hashicorp/terraform/internal/command/jsonformat"
 	"github.com/hashicorp/terraform/internal/command/views"
@@ -58,7 +61,7 @@ type TestSuiteRunner struct {
 
 	// GlobalVariables are the variables provided by the TF_VAR_* environment
 	// variables and -var and -var-file flags.
-	GlobalVariables map[string]backend.UnparsedVariableValue
+	GlobalVariables map[string]backendrun.UnparsedVariableValue
 
 	// Stopped and Cancelled track whether the user requested the testing
 	// process to be interrupted. Stopped is a nice graceful exit, we'll still
@@ -93,6 +96,10 @@ type TestSuiteRunner struct {
 	View    views.Test
 	Streams *terminal.Streams
 
+	// appName is the name of the instance this test suite runner is configured
+	// against. Can be "HCP Terraform" or "Terraform Enterprise"
+	appName string
+
 	// clientOverride allows tests to specify the client instead of letting the
 	// system initialise one itself.
 	clientOverride *tfe.Client
@@ -102,6 +109,10 @@ func (runner *TestSuiteRunner) Stop() {
 	runner.Stopped = true
 }
 
+func (runner *TestSuiteRunner) IsStopped() bool {
+	return runner.Stopped
+}
+
 func (runner *TestSuiteRunner) Cancel() {
 	runner.Cancelled = true
 }
@@ -142,7 +153,7 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 		diags = diags.Append(tfdiags.AttributeValue(
 			tfdiags.Error,
 			"Module source points to the public registry",
-			"Terraform Cloud can only execute tests for modules held within private registries.",
+			"HCP Terraform and Terraform Enterprise can only execute tests for modules held within private registries.",
 			cty.Path{cty.GetAttrStep{Name: "source"}}))
 		return moduletest.Error, diags
 	}
@@ -163,7 +174,7 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 
 	configurationVersion, err := client.ConfigurationVersions.CreateForRegistryModule(runner.StoppedCtx, id)
 	if err != nil {
-		diags = diags.Append(generalError("Failed to create configuration version", err))
+		diags = diags.Append(runner.generalError("Failed to create configuration version", err))
 		return moduletest.Error, diags
 	}
 
@@ -172,7 +183,7 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 	}
 
 	if err := client.ConfigurationVersions.Upload(runner.StoppedCtx, configurationVersion.UploadURL, configDirectory); err != nil {
-		diags = diags.Append(generalError("Failed to upload configuration version", err))
+		diags = diags.Append(runner.generalError("Failed to upload configuration version", err))
 		return moduletest.Error, diags
 	}
 
@@ -185,9 +196,9 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 	// the test run tidies up any state properly. This means, we'll send the
 	// cancellation signals and then still wait for and process the logs.
 	//
-	// This also means that all calls to TFC will use context.Background()
+	// This also means that all calls to HCP Terraform will use context.Background()
 	// instead of the stopped or cancelled context as we want them to finish and
-	// the run to be cancelled by TFC properly.
+	// the run to be cancelled by HCP Terraform properly.
 
 	opts := tfe.TestRunCreateOptions{
 		Filters:       runner.Filters,
@@ -209,7 +220,7 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 
 	run, err := client.TestRuns.Create(context.Background(), opts)
 	if err != nil {
-		diags = diags.Append(generalError("Failed to create test run", err))
+		diags = diags.Append(runner.generalError("Failed to create test run", err))
 		return moduletest.Error, diags
 	}
 
@@ -228,7 +239,7 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 		for i := 0; !completed; i++ {
 			run, err := client.TestRuns.Read(context.Background(), id, run.ID)
 			if err != nil {
-				diags = diags.Append(generalError("Failed to retrieve test run", err))
+				diags = diags.Append(runner.generalError("Failed to retrieve test run", err))
 				return // exit early
 			}
 
@@ -270,7 +281,7 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 	// Refresh the run now we know it is finished.
 	run, err = client.TestRuns.Read(context.Background(), id, run.ID)
 	if err != nil {
-		diags = diags.Append(generalError("Failed to retrieve completed test run", err))
+		diags = diags.Append(runner.generalError("Failed to retrieve completed test run", err))
 		return moduletest.Error, diags
 	}
 
@@ -302,6 +313,24 @@ func (runner *TestSuiteRunner) Test() (moduletest.Status, tfdiags.Diagnostics) {
 	}
 }
 
+// discover the TFC/E API service URL
+func discoverTfeURL(hostname svchost.Hostname, services *disco.Disco) (*url.URL, error) {
+	host, err := services.Discover(hostname)
+	if err != nil {
+		var serviceDiscoErr *disco.ErrServiceDiscoveryNetworkRequest
+
+		switch {
+		case errors.As(err, &serviceDiscoErr):
+			err = fmt.Errorf("a network issue prevented cloud configuration; %w", err)
+			return nil, err
+		default:
+			return nil, err
+		}
+	}
+
+	return host.ServiceURL(tfeServiceID)
+}
+
 func (runner *TestSuiteRunner) client(addr tfaddr.Module, id tfe.RegistryModuleID) (*tfe.Client, *tfe.RegistryModule, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
@@ -309,7 +338,7 @@ func (runner *TestSuiteRunner) client(addr tfaddr.Module, id tfe.RegistryModuleI
 	if runner.clientOverride != nil {
 		client = runner.clientOverride
 	} else {
-		service, err := discover(addr.Package.Host, runner.Services)
+		service, err := discoverTfeURL(addr.Package.Host, runner.Services)
 		if err != nil {
 			diags = diags.Append(tfdiags.AttributeValue(
 				tfdiags.Error,
@@ -365,10 +394,10 @@ func (runner *TestSuiteRunner) client(addr tfaddr.Module, id tfe.RegistryModuleI
 		if client, err = tfe.NewClient(cfg); err != nil {
 			diags = diags.Append(tfdiags.Sourceless(
 				tfdiags.Error,
-				"Failed to create the Terraform Cloud/Enterprise client",
+				"Failed to create the HCP Terraform or Terraform Enterprise client",
 				fmt.Sprintf(
 					`Encountered an unexpected error while creating the `+
-						`Terraform Cloud/Enterprise client: %s.`, err,
+						`HCP Terraform or Terraform Enterprise client: %s.`, err,
 				),
 			))
 			return nil, nil, diags
@@ -392,6 +421,11 @@ func (runner *TestSuiteRunner) client(addr tfaddr.Module, id tfe.RegistryModuleI
 	// Enable retries for server errors.
 	client.RetryServerErrors(true)
 
+	runner.appName = client.AppName()
+	if isValidAppName(runner.appName) {
+		runner.appName = "HCP Terraform"
+	}
+
 	// Aaaaand I'm done.
 	return client, module, diags
 }
@@ -404,13 +438,13 @@ func (runner *TestSuiteRunner) wait(ctx context.Context, client *tfe.Client, run
 			diags = diags.Append(tfdiags.Sourceless(
 				tfdiags.Error,
 				"Could not cancel the test run",
-				fmt.Sprintf("Terraform could not cancel the test run, you will have to navigate to the Terraform Cloud console and cancel the test run manually.\n\nThe error message received when cancelling the test run was %s", err)))
+				fmt.Sprintf("Terraform could not cancel the test run, you will have to navigate to the %s console and cancel the test run manually.\n\nThe error message received when cancelling the test run was %s", client.AppName(), err)))
 			return
 		}
 
 		// At this point we've requested a force cancel, and we know that
 		// Terraform locally is just going to quit after some amount of time so
-		// we'll just wait for that to happen or for TFC to finish, whichever
+		// we'll just wait for that to happen or for HCP Terraform to finish, whichever
 		// happens first.
 		<-ctx.Done()
 	}
@@ -420,11 +454,11 @@ func (runner *TestSuiteRunner) wait(ctx context.Context, client *tfe.Client, run
 			diags = diags.Append(tfdiags.Sourceless(
 				tfdiags.Error,
 				"Could not stop the test run",
-				fmt.Sprintf("Terraform could not stop the test run, you will have to navigate to the Terraform Cloud console and cancel the test run manually.\n\nThe error message received when stopping the test run was %s", err)))
+				fmt.Sprintf("Terraform could not stop the test run, you will have to navigate to the %s console and cancel the test run manually.\n\nThe error message received when stopping the test run was %s", client.AppName(), err)))
 			return
 		}
 
-		// We've request a cancel, we're happy to just wait for TFC to cancel
+		// We've request a cancel, we're happy to just wait for HCP Terraform to cancel
 		// the run appropriately.
 		select {
 		case <-runner.CancelledCtx.Done():
@@ -453,7 +487,7 @@ func (runner *TestSuiteRunner) renderLogs(client *tfe.Client, run *tfe.TestRun,
 
 	logs, err := client.TestRuns.Logs(context.Background(), moduleId, run.ID)
 	if err != nil {
-		diags = diags.Append(generalError("Failed to retrieve logs", err))
+		diags = diags.Append(runner.generalError("Failed to retrieve logs", err))
 		return diags
 	}
 
@@ -467,7 +501,7 @@ func (runner *TestSuiteRunner) renderLogs(client *tfe.Client, run *tfe.TestRun,
 			l, isPrefix, err = reader.ReadLine()
 			if err != nil {
 				if err != io.EOF {
-					diags = diags.Append(generalError("Failed to read logs", err))
+					diags = diags.Append(runner.generalError("Failed to read logs", err))
 					return diags
 				}
 				next = false
@@ -571,3 +605,35 @@ func (runner *TestSuiteRunner) renderLogs(client *tfe.Client, run *tfe.TestRun,
 
 	return diags
 }
+
+func (runner *TestSuiteRunner) generalError(msg string, err error) error {
+	var diags tfdiags.Diagnostics
+
+	if urlErr, ok := err.(*url.Error); ok {
+		err = urlErr.Err
+	}
+
+	switch err {
+	case context.Canceled:
+		return err
+	case tfe.ErrResourceNotFound:
+		diags = diags.Append(tfdiags.Sourceless(
+			tfdiags.Error,
+			fmt.Sprintf("%s: %v", msg, err),
+			fmt.Sprintf("For security, %s return '404 Not Found' responses for resources\n", runner.appName)+
+				"for resources that a user doesn't have access to, in addition to resources that\n"+
+				"do not exist. If the resource does exist, please check the permissions of the provided token.",
+		))
+		return diags.Err()
+	default:
+		diags = diags.Append(tfdiags.Sourceless(
+			tfdiags.Error,
+			fmt.Sprintf("%s: %v", msg, err),
+			fmt.Sprintf(`%s returned an unexpected error. Sometimes `, runner.appName)+
+				`this is caused by network connection problems, in which case you could retry `+
+				`the command. If the issue persists please open a support ticket to get help `+
+				`resolving the problem.`,
+		))
+		return diags.Err()
+	}
+}
