@@ -47,6 +47,14 @@ func ParseRefresh(args []string) (*Refresh, tfdiags.Diagnostics) {
 		))
 	}
 
+	if refresh.State.StatePath != "" {
+		diags = diags.Append(tfdiags.Sourceless(
+			tfdiags.Warning,
+			"Deprecated flag: -state",
+			"Use `path` attribute within the `local` backend instead: https://developer.hashicorp.com/terraform/language/v1.10.x/settings/backends/local#path",
+		))
+	}
+
 	args = cmdFlags.Args()
 	if len(args) > 0 {
 		diags = diags.Append(tfdiags.Sourceless(
