@@ -1,4 +1,7 @@
 #!/bin/bash
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
+
 
 # For normal use this package can just be tested with "go test" as standard,
 # but this script is an alternative to allow the tests to be run somewhere
@@ -8,7 +11,7 @@
 # archive that can be extracted on a Windows system to run the e2e tests there:
 #    $ GOOS=windows GOARCH=amd64 ./make-archive.sh
 #
-# This will produce a zip file build/terraform-s2stest_windows_amd64.zip which
+# This will produce a zip file build/terraform-e2etest_windows_amd64.zip which
 # can be shipped off to a Windows amd64 system, extracted to some directory,
 # and then executed as follows:
 #    set TF_ACC=1
@@ -31,13 +34,20 @@ GOEXE="$(go env GOEXE)"
 OUTDIR="build/${GOOS}_${GOARCH}"
 OUTFILE="terraform-e2etest_${GOOS}_${GOARCH}.zip"
 
+LDFLAGS="-X github.com/hashicorp/terraform/internal/command/e2etest.terraformBin=./terraform$GOEXE"
+# Caller may pass in the environment variable GO_LDFLAGS with additional
+# flags we'll use when building.
+if [ -n "${GO_LDFLAGS+set}" ]; then
+    LDFLAGS="${GO_LDFLAGS} ${LDFLAGS}"
+fi
+
 mkdir -p "$OUTDIR"
 
 # We need the test fixtures available when we run the tests.
 cp -r testdata "$OUTDIR/testdata"
 
 # Build the test program
-go test -o "$OUTDIR/e2etest$GOEXE" -c -ldflags "-X github.com/hashicorp/terraform/internal/command/e2etest.terraformBin=./terraform$GOEXE" github.com/hashicorp/terraform/internal/command/e2etest
+go test -o "$OUTDIR/e2etest$GOEXE" -c -ldflags "$LDFLAGS" github.com/hashicorp/terraform/internal/command/e2etest
 
 # Now bundle it all together for easy shipping!
 cd "$OUTDIR"
