@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package e2etest
 
 import (
@@ -12,8 +15,7 @@ func TestTerraformProviderRead(t *testing.T) {
 
 	t.Parallel()
 	fixturePath := filepath.Join("testdata", "terraform-provider")
-	tf := e2e.NewBinary(terraformBin, fixturePath)
-	defer tf.Close()
+	tf := e2e.NewBinary(t, terraformBin, fixturePath)
 
 	//// INIT
 	_, stderr, err := tf.Run("init")
