@@ -117,17 +117,17 @@ func TestMarshalPlannedOutputs(t *testing.T) {
 	after, _ := plans.NewDynamicValue(cty.StringVal("after"), cty.DynamicPseudoType)
 
 	tests := []struct {
-		Changes *plans.Changes
+		Changes *plans.ChangesSrc
 		Want    map[string]output
 		Err     bool
 	}{
 		{
-			&plans.Changes{},
+			&plans.ChangesSrc{},
 			nil,
 			false,
 		},
 		{
-			&plans.Changes{
+			&plans.ChangesSrc{
 				Outputs: []*plans.OutputChangeSrc{
 					{
 						Addr: addrs.OutputValue{Name: "bar"}.Absolute(addrs.RootModuleInstance),
@@ -149,7 +149,7 @@ func TestMarshalPlannedOutputs(t *testing.T) {
 			false,
 		},
 		{ // Delete action
-			&plans.Changes{
+			&plans.ChangesSrc{
 				Outputs: []*plans.OutputChangeSrc{
 					{
 						Addr: addrs.OutputValue{Name: "bar"}.Absolute(addrs.RootModuleInstance),
@@ -270,7 +270,7 @@ func TestMarshalPlanResources(t *testing.T) {
 			if err != nil {
 				t.Fatal(err)
 			}
-			testChange := &plans.Changes{
+			testChange := &plans.ChangesSrc{
 				Resources: []*plans.ResourceInstanceChangeSrc{
 					{
 						Addr: addrs.Resource{
@@ -316,7 +316,7 @@ func TestMarshalPlanValuesNoopDeposed(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	testChange := &plans.Changes{
+	testChange := &plans.ChangesSrc{
 		Resources: []*plans.ResourceInstanceChangeSrc{
 			{
 				Addr: addrs.Resource{
