@@ -217,6 +217,35 @@ func (m *Meta) providerDevOverrideRuntimeWarnings() tfdiags.Diagnostics {
 	}
 }
 
+// providerDevOverrideRuntimeWarningsRemoteExecution returns a diagnostics that contains at
+// least one warning if and only if there is at least one provider development
+// override in effect. If not, the result is always empty. The result never
+// contains error diagnostics.
+//
+// Certain commands when executing remotely can use this to include a warning that any provider overrides
+// are only locally configured, meaning the remote operation won't be affected by them.
+//
+// See providerDevOverrideRuntimeWarnings for runtime warnings specific to local execution
+func (m *Meta) providerDevOverrideRuntimeWarningsRemoteExecution() tfdiags.Diagnostics {
+	if len(m.ProviderDevOverrides) == 0 {
+		return nil
+	}
+	var detailMsg strings.Builder
+	detailMsg.WriteString("The following provider development overrides are set in the CLI configuration:\n")
+	for addr, path := range m.ProviderDevOverrides {
+		detailMsg.WriteString(fmt.Sprintf(" - %s in %s\n", addr.ForDisplay(), path))
+	}
+	detailMsg.WriteString("\nProvider development overrides are only configured locally and the remote operation won't be affected by them")
+
+	return tfdiags.Diagnostics{
+		tfdiags.Sourceless(
+			tfdiags.Warning,
+			"Provider development overrides are in effect",
+			detailMsg.String(),
+		),
+	}
+}
+
 // providerFactories uses the selections made previously by an installer in
 // the local cache directory (m.providerLocalCacheDir) to produce a map
 // from provider addresses to factory functions to create instances of
@@ -378,20 +407,26 @@ func providerFactory(meta *providercache.CachedProvider) providers.Factory {
 
 		// store the client so that the plugin can kill the child process
 		protoVer := client.NegotiatedVersion()
-		switch protoVer {
-		case 5:
-			p := raw.(*tfplugin.GRPCProvider)
-			p.PluginClient = client
-			p.Addr = meta.Provider
-			return p, nil
-		case 6:
-			p := raw.(*tfplugin6.GRPCProvider)
-			p.PluginClient = client
-			p.Addr = meta.Provider
-			return p, nil
-		default:
-			panic("unsupported protocol version")
-		}
+		return finalizeFactoryPlugin(raw, protoVer, meta.Provider, client), nil
+	}
+}
+
+// finalizeFactoryPlugin completes the setup of a plugin dispensed by the rpc
+// client to be returned by the plugin factory.
+func finalizeFactoryPlugin(rawPlugin any, protoVersion int, addr addrs.Provider, client *plugin.Client) providers.Interface {
+	switch protoVersion {
+	case 5:
+		p := rawPlugin.(*tfplugin.GRPCProvider)
+		p.PluginClient = client
+		p.Addr = addr
+		return p
+	case 6:
+		p := rawPlugin.(*tfplugin6.GRPCProvider)
+		p.PluginClient = client
+		p.Addr = addr
+		return p
+	default:
+		panic("unsupported protocol version")
 	}
 }
 
@@ -460,13 +495,9 @@ func unmanagedProviderFactory(provider addrs.Provider, reattach *plugin.Reattach
 			// go-plugin), so client.NegotiatedVersion() always returns 0. We
 			// assume that an unmanaged provider reporting protocol version 0 is
 			// actually using proto v5 for backwards compatibility.
-			p := raw.(*tfplugin.GRPCProvider)
-			p.PluginClient = client
-			return p, nil
+			return finalizeFactoryPlugin(raw, 5, provider, client), nil
 		case 6:
-			p := raw.(*tfplugin6.GRPCProvider)
-			p.PluginClient = client
-			return p, nil
+			return finalizeFactoryPlugin(raw, 6, provider, client), nil
 		default:
 			return nil, fmt.Errorf("unsupported protocol version %d", protoVer)
 		}
