@@ -13,13 +13,13 @@ import (
 
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/depsfile"
-	"github.com/hashicorp/terraform/internal/getproviders"
+	"github.com/hashicorp/terraform/internal/getproviders/providerreqs"
 )
 
 // A Config is a node in the tree of modules within a configuration.
 //
 // The module tree is constructed by following ModuleCall instances recursively
-// through the root module transitively into descendent modules.
+// through the root module transitively into descendant modules.
 //
 // A module tree described in *this* package represents the static tree
 // represented by configuration. During evaluation a static ModuleNode may
@@ -92,7 +92,7 @@ type ModuleRequirements struct {
 	Name         string
 	SourceAddr   addrs.ModuleSource
 	SourceDir    string
-	Requirements getproviders.Requirements
+	Requirements providerreqs.Requirements
 	Children     map[string]*ModuleRequirements
 	Tests        map[string]*TestFileModuleRequirements
 }
@@ -100,7 +100,7 @@ type ModuleRequirements struct {
 // TestFileModuleRequirements maps the runs for a given test file to the module
 // requirements for that run block.
 type TestFileModuleRequirements struct {
-	Requirements getproviders.Requirements
+	Requirements providerreqs.Requirements
 	Runs         map[string]*ModuleRequirements
 }
 
@@ -145,7 +145,7 @@ func (c *Config) DeepEach(cb func(c *Config)) {
 	}
 }
 
-// AllModules returns a slice of all the receiver and all of its descendent
+// AllModules returns a slice of all the receiver and all of its descendant
 // nodes in the module tree, in the same order they would be visited by
 // DeepEach.
 func (c *Config) AllModules() []*Config {
@@ -156,14 +156,14 @@ func (c *Config) AllModules() []*Config {
 	return ret
 }
 
-// Descendent returns the descendent config that has the given path beneath
+// Descendant returns the descendant config that has the given path beneath
 // the receiver, or nil if there is no such module.
 //
 // The path traverses the static module tree, prior to any expansion to handle
 // count and for_each arguments.
 //
 // An empty path will just return the receiver, and is therefore pointless.
-func (c *Config) Descendent(path addrs.Module) *Config {
+func (c *Config) Descendant(path addrs.Module) *Config {
 	current := c
 	for _, name := range path {
 		current = current.Children[name]
@@ -174,13 +174,13 @@ func (c *Config) Descendent(path addrs.Module) *Config {
 	return current
 }
 
-// DescendentForInstance is like Descendent except that it accepts a path
+// DescendantForInstance is like Descendant except that it accepts a path
 // to a particular module instance in the dynamic module graph, returning
 // the node from the static module graph that corresponds to it.
 //
 // All instances created by a particular module call share the same
 // configuration, so the keys within the given path are disregarded.
-func (c *Config) DescendentForInstance(path addrs.ModuleInstance) *Config {
+func (c *Config) DescendantForInstance(path addrs.ModuleInstance) *Config {
 	current := c
 	for _, step := range path {
 		current = current.Children[step.Name]
@@ -191,6 +191,46 @@ func (c *Config) DescendentForInstance(path addrs.ModuleInstance) *Config {
 	return current
 }
 
+// TargetExists returns true if it's possible for the provided target to exist
+// within the configuration.
+//
+// This doesn't consider instance expansion, so we're only making sure the
+// target could exist if the instance expansion expands correctly.
+func (c *Config) TargetExists(target addrs.Targetable) bool {
+	switch target.AddrType() {
+	case addrs.ConfigResourceAddrType:
+		addr := target.(addrs.ConfigResource)
+		module := c.Descendant(addr.Module)
+		if module != nil {
+			return module.Module.ResourceByAddr(addr.Resource) != nil
+		} else {
+			return false
+		}
+	case addrs.AbsResourceInstanceAddrType:
+		addr := target.(addrs.AbsResourceInstance)
+		module := c.DescendantForInstance(addr.Module)
+		if module != nil {
+			return module.Module.ResourceByAddr(addr.Resource.Resource) != nil
+		} else {
+			return false
+		}
+	case addrs.AbsResourceAddrType:
+		addr := target.(addrs.AbsResource)
+		module := c.DescendantForInstance(addr.Module)
+		if module != nil {
+			return module.Module.ResourceByAddr(addr.Resource) != nil
+		} else {
+			return false
+		}
+	case addrs.ModuleAddrType:
+		return c.Descendant(target.(addrs.Module)) != nil
+	case addrs.ModuleInstanceAddrType:
+		return c.DescendantForInstance(target.(addrs.ModuleInstance)) != nil
+	default:
+		panic(fmt.Errorf("unrecognized targetable type: %d", target.AddrType()))
+	}
+}
+
 // EntersNewPackage returns true if this call is to an external module, either
 // directly via a remote source address or indirectly via a registry source
 // address.
@@ -256,14 +296,14 @@ func (c *Config) VerifyDependencySelections(depLocks *depsfile.Locks) []error {
 			lock = depLocks.Provider(providerAddr)
 		}
 		if lock == nil {
-			log.Printf("[TRACE] Config.VerifyDependencySelections: provider %s has no lock file entry to satisfy %q", providerAddr, getproviders.VersionConstraintsString(constraints))
+			log.Printf("[TRACE] Config.VerifyDependencySelections: provider %s has no lock file entry to satisfy %q", providerAddr, providerreqs.VersionConstraintsString(constraints))
 			errs = append(errs, fmt.Errorf("provider %s: required by this configuration but no version is selected", providerAddr))
 			continue
 		}
 
 		selectedVersion := lock.Version()
-		allowedVersions := getproviders.MeetingConstraints(constraints)
-		log.Printf("[TRACE] Config.VerifyDependencySelections: provider %s has %s to satisfy %q", providerAddr, selectedVersion.String(), getproviders.VersionConstraintsString(constraints))
+		allowedVersions := providerreqs.MeetingConstraints(constraints)
+		log.Printf("[TRACE] Config.VerifyDependencySelections: provider %s has %s to satisfy %q", providerAddr, selectedVersion.String(), providerreqs.VersionConstraintsString(constraints))
 		if !allowedVersions.Has(selectedVersion) {
 			// The most likely cause of this is that the author of a module
 			// has changed its constraints, but this could also happen in
@@ -272,8 +312,8 @@ func (c *Config) VerifyDependencySelections(depLocks *depsfile.Locks) []error {
 			// distinguish those cases here in order to avoid the more
 			// specific error message potentially being a red herring in
 			// the edge-cases.
-			currentConstraints := getproviders.VersionConstraintsString(constraints)
-			lockedConstraints := getproviders.VersionConstraintsString(lock.VersionConstraints())
+			currentConstraints := providerreqs.VersionConstraintsString(constraints)
+			lockedConstraints := providerreqs.VersionConstraintsString(lock.VersionConstraints())
 			switch {
 			case currentConstraints != lockedConstraints:
 				errs = append(errs, fmt.Errorf("provider %s: locked version selection %s doesn't match the updated version constraints %q", providerAddr, selectedVersion.String(), currentConstraints))
@@ -299,20 +339,29 @@ func (c *Config) VerifyDependencySelections(depLocks *depsfile.Locks) []error {
 //
 // If the returned diagnostics includes errors then the resulting Requirements
 // may be incomplete.
-func (c *Config) ProviderRequirements() (getproviders.Requirements, hcl.Diagnostics) {
-	reqs := make(getproviders.Requirements)
+func (c *Config) ProviderRequirements() (providerreqs.Requirements, hcl.Diagnostics) {
+	reqs := make(providerreqs.Requirements)
 	diags := c.addProviderRequirements(reqs, true, true)
 
 	return reqs, diags
 }
 
+// ProviderRequirementsConfigOnly searches the full tree of configuration
+// files for all providers. This function does not consider any test files.
+func (c *Config) ProviderRequirementsConfigOnly() (providerreqs.Requirements, hcl.Diagnostics) {
+	reqs := make(providerreqs.Requirements)
+	diags := c.addProviderRequirements(reqs, true, false)
+
+	return reqs, diags
+}
+
 // ProviderRequirementsShallow searches only the direct receiver for explicit
 // and implicit dependencies on providers. Descendant modules are ignored.
 //
 // If the returned diagnostics includes errors then the resulting Requirements
 // may be incomplete.
-func (c *Config) ProviderRequirementsShallow() (getproviders.Requirements, hcl.Diagnostics) {
-	reqs := make(getproviders.Requirements)
+func (c *Config) ProviderRequirementsShallow() (providerreqs.Requirements, hcl.Diagnostics) {
+	reqs := make(providerreqs.Requirements)
 	diags := c.addProviderRequirements(reqs, false, true)
 
 	return reqs, diags
@@ -325,7 +374,7 @@ func (c *Config) ProviderRequirementsShallow() (getproviders.Requirements, hcl.D
 // If the returned diagnostics includes errors then the resulting Requirements
 // may be incomplete.
 func (c *Config) ProviderRequirementsByModule() (*ModuleRequirements, hcl.Diagnostics) {
-	reqs := make(getproviders.Requirements)
+	reqs := make(providerreqs.Requirements)
 	diags := c.addProviderRequirements(reqs, false, false)
 
 	children := make(map[string]*ModuleRequirements)
@@ -339,14 +388,10 @@ func (c *Config) ProviderRequirementsByModule() (*ModuleRequirements, hcl.Diagno
 	tests := make(map[string]*TestFileModuleRequirements)
 	for name, test := range c.Module.Tests {
 		testReqs := &TestFileModuleRequirements{
-			Requirements: make(getproviders.Requirements),
+			Requirements: make(providerreqs.Requirements),
 			Runs:         make(map[string]*ModuleRequirements),
 		}
 
-		for _, provider := range test.Providers {
-			diags = append(diags, c.addProviderRequirementsFromProviderBlock(testReqs.Requirements, provider)...)
-		}
-
 		for _, run := range test.Runs {
 			if run.ConfigUnderTest == nil {
 				continue
@@ -376,7 +421,7 @@ func (c *Config) ProviderRequirementsByModule() (*ModuleRequirements, hcl.Diagno
 // implementation, gradually mutating a shared requirements object to
 // eventually return. If the recurse argument is true, the requirements will
 // include all descendant modules; otherwise, only the specified module.
-func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse, tests bool) hcl.Diagnostics {
+func (c *Config) addProviderRequirements(reqs providerreqs.Requirements, recurse, tests bool) hcl.Diagnostics {
 	var diags hcl.Diagnostics
 
 	// First we'll deal with the requirements directly in _our_ module...
@@ -394,7 +439,7 @@ func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse
 			// don't exactly agree in practice 🙄 so this might produce new errors.
 			// TODO: Use the new parser throughout this package so we can get the
 			// better error messages it produces in more situations.
-			constraints, err := getproviders.ParseVersionConstraints(providerReqs.Requirement.Required.String())
+			constraints, err := providerreqs.ParseVersionConstraints(providerReqs.Requirement.Required.String())
 			if err != nil {
 				diags = diags.Append(&hcl.Diagnostic{
 					Severity: hcl.DiagError,
@@ -421,6 +466,7 @@ func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse
 		}
 		reqs[fqn] = nil
 	}
+
 	for _, rc := range c.Module.DataResources {
 		fqn := rc.Provider
 		if _, exists := reqs[fqn]; exists {
@@ -429,22 +475,18 @@ func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse
 		}
 		reqs[fqn] = nil
 	}
-	for _, i := range c.Module.Import {
-		implied, err := addrs.ParseProviderPart(i.To.Resource.Resource.ImpliedProvider())
-		if err == nil {
-			provider := c.Module.ImpliedProviderForUnqualifiedType(implied)
-			if _, exists := reqs[provider]; exists {
-				// Explicit dependency already present
-				continue
-			}
-			reqs[provider] = nil
+
+	for _, rc := range c.Module.EphemeralResources {
+		fqn := rc.Provider
+		if _, exists := reqs[fqn]; exists {
+			// Explicit dependency already present
+			continue
 		}
-		// We don't return a diagnostic here, because the invalid address will
-		// have been caught elsewhere.
+		reqs[fqn] = nil
 	}
 
-	// Import blocks that are generating config may also have a custom provider
-	// meta argument. Like the provider meta argument used in resource blocks,
+	// Import blocks that are generating config may have a custom provider
+	// meta-argument. Like the provider meta-argument used in resource blocks,
 	// we use this opportunity to load any implicit providers.
 	//
 	// We'll also use this to validate that import blocks and targeted resource
@@ -452,14 +494,14 @@ func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse
 	// this will be because the user has written explicit provider arguments
 	// that don't agree and we'll get them to fix it.
 	for _, i := range c.Module.Import {
-		if len(i.To.Module) > 0 {
+		if len(i.ToResource.Module) > 0 {
 			// All provider information for imports into modules should come
 			// from the module block, so we don't need to load anything for
 			// import targets within modules.
 			continue
 		}
 
-		if target, exists := c.Module.ManagedResources[i.To.String()]; exists {
+		if target, exists := c.Module.ManagedResources[i.ToResource.Resource.String()]; exists {
 			// This means the information about the provider for this import
 			// should come from the resource block itself and not the import
 			// block.
@@ -528,20 +570,13 @@ func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse
 
 	// We may have provider blocks and required_providers set in some testing
 	// files.
-	if tests {
+	if tests && recurse {
 		for _, file := range c.Module.Tests {
-			for _, provider := range file.Providers {
-				moreDiags := c.addProviderRequirementsFromProviderBlock(reqs, provider)
-				diags = append(diags, moreDiags...)
-			}
-
-			if recurse {
-				// Then we'll also look for requirements in testing modules.
-				for _, run := range file.Runs {
-					if run.ConfigUnderTest != nil {
-						moreDiags := run.ConfigUnderTest.addProviderRequirements(reqs, true, false)
-						diags = append(diags, moreDiags...)
-					}
+			// Then we'll also look for requirements in testing modules.
+			for _, run := range file.Runs {
+				if run.ConfigUnderTest != nil {
+					moreDiags := run.ConfigUnderTest.addProviderRequirements(reqs, true, false)
+					diags = append(diags, moreDiags...)
 				}
 			}
 		}
@@ -557,7 +592,7 @@ func (c *Config) addProviderRequirements(reqs getproviders.Requirements, recurse
 	return diags
 }
 
-func (c *Config) addProviderRequirementsFromProviderBlock(reqs getproviders.Requirements, provider *Provider) hcl.Diagnostics {
+func (c *Config) addProviderRequirementsFromProviderBlock(reqs providerreqs.Requirements, provider *Provider) hcl.Diagnostics {
 	var diags hcl.Diagnostics
 
 	fqn := c.Module.ProviderForLocalConfig(addrs.LocalProviderConfig{LocalName: provider.Name})
@@ -573,7 +608,7 @@ func (c *Config) addProviderRequirementsFromProviderBlock(reqs getproviders.Requ
 		// don't exactly agree in practice 🙄 so this might produce new errors.
 		// TODO: Use the new parser throughout this package so we can get the
 		// better error messages it produces in more situations.
-		constraints, err := getproviders.ParseVersionConstraints(provider.Version.Required.String())
+		constraints, err := providerreqs.ParseVersionConstraints(provider.Version.Required.String())
 		if err != nil {
 			diags = diags.Append(&hcl.Diagnostic{
 				Severity: hcl.DiagError,
@@ -819,9 +854,9 @@ func (c *Config) ResolveAbsProviderAddr(addr addrs.ProviderConfig, inModule addr
 		return addr
 
 	case addrs.LocalProviderConfig:
-		// Find the descendent Config that contains the module that this
+		// Find the descendant Config that contains the module that this
 		// local config belongs to.
-		mc := c.Descendent(inModule)
+		mc := c.Descendant(inModule)
 		if mc == nil {
 			panic(fmt.Sprintf("ResolveAbsProviderAddr with non-existent module %s", inModule.String()))
 		}
@@ -855,6 +890,167 @@ func (c *Config) ProviderForConfigAddr(addr addrs.LocalProviderConfig) addrs.Pro
 	return c.ResolveAbsProviderAddr(addr, addrs.RootModule).Provider
 }
 
+// RequiredProviderConfig represents a provider configuration that is required
+// by a module, either explicitly or implicitly.
+//
+// An explicit provider means the LocalName within the addrs.LocalProviderConfig
+// was defined directly within the configuration via a required_providers block
+// instead of implied due to the name of a resource or data block.
+//
+// This helps callers of the EffectiveRequiredProviderConfigs function tailor
+// error messages around implied or explicit provider types.
+type RequiredProviderConfig struct {
+	Local    addrs.LocalProviderConfig
+	Explicit bool
+}
+
+// EffectiveRequiredProviderConfigs returns a set of all of the provider
+// configurations this config's direct module expects to have passed in
+// (explicitly or implicitly) by its caller. This method only makes sense
+// to call on the object representing the root module.
+//
+// This includes both provider configurations declared explicitly using
+// configuration_aliases in the required_providers block _and_ configurations
+// that are implied to be required by declaring something that belongs to
+// an configuration for a provider even when there is no such declaration
+// inside the module itself.
+//
+// Terraform Core treats root modules differently than downstream modules in
+// that it will implicitly create empty provider configurations for any provider
+// config addresses that are implied in the configuration but not explicitly
+// configured. This function assumes those implied empty configurations don't
+// exist and so therefore any provider configuration without an explicit
+// "provider" block is a required provider config. In practice that means that
+// the answer is appropriate for downstream modules but not for root modules,
+// unless a root module is being used in a context where it is treated as if
+// a shared module, such as when directly testing a shared module or when
+// using a shared module as the root of the module tree of a stack component.
+//
+// This function assumes that the configuration is valid. It may produce under-
+// or over-constrained results if called on an invalid configuration.
+func (c *Config) EffectiveRequiredProviderConfigs() addrs.Map[addrs.RootProviderConfig, RequiredProviderConfig] {
+	// The Terraform language has accumulated so many different ways to imply
+	// the need for a provider configuration that answering this is quite a
+	// complicated process that ends up potentially needing to visit the
+	// entire subtree of modules even though we're only actually answering
+	// about the current node's requirements. In the happy explicit case we
+	// can avoid any recursion, but that case is rare in practice.
+
+	if c == nil {
+		return addrs.MakeMap[addrs.RootProviderConfig, RequiredProviderConfig]()
+	}
+
+	// We'll start by visiting all of the "provider" blocks in the module and
+	// figuring out which provider configuration address they each declare. Any
+	// configuration addresses we find here cannot be "required" provider
+	// configs because the module instantiates them itself.
+	selfConfigured := addrs.MakeSet[addrs.RootProviderConfig]()
+	for _, pc := range c.Module.ProviderConfigs {
+		localAddr := pc.Addr()
+		sourceAddr := c.Module.ProviderForLocalConfig(localAddr)
+		selfConfigured.Add(addrs.RootProviderConfig{
+			Provider: sourceAddr,
+			Alias:    localAddr.Alias,
+		})
+	}
+	ret := addrs.MakeMap[addrs.RootProviderConfig, RequiredProviderConfig]()
+
+	// maybePut looks up the default local provider for the given root provider.
+	maybePut := func(addr addrs.RootProviderConfig) {
+		localName := c.Module.LocalNameForProvider(addr.Provider)
+		localAddr := addrs.LocalProviderConfig{
+			LocalName: localName,
+			Alias:     addr.Alias,
+		}
+		if !selfConfigured.Has(addr) && !ret.Has(addr) {
+			ret.Put(addr, RequiredProviderConfig{
+				Local: localAddr,
+
+				// Since we look at the required providers first below, and only
+				// the required providers can set explicit local names, this
+				// will always be false as the map entry will already have been
+				// set if this would be true.
+				Explicit: false,
+			})
+		}
+	}
+
+	// maybePutLocal looks up the default provider for the given local provider
+	// address.
+	maybePutLocal := func(localAddr addrs.LocalProviderConfig, explicit bool) {
+		// Caution: this function is only correct to use for LocalProviderConfig
+		// in the _current_ module c.Module. It will produce incorrect results
+		// if used for addresses from any child module.
+		addr := addrs.RootProviderConfig{
+			Provider: c.Module.ProviderForLocalConfig(localAddr),
+			Alias:    localAddr.Alias,
+		}
+		if !selfConfigured.Has(addr) && !ret.Has(addr) {
+			ret.Put(addr, RequiredProviderConfig{
+				Local:    localAddr,
+				Explicit: explicit,
+			})
+		}
+	}
+
+	if c.Module.ProviderRequirements != nil {
+		for _, req := range c.Module.ProviderRequirements.RequiredProviders {
+			for _, addr := range req.Aliases {
+				// The RequiredProviders block always produces explicit provider
+				// names.
+				maybePutLocal(addr, true)
+			}
+		}
+	}
+	for _, rc := range c.Module.ManagedResources {
+		maybePutLocal(rc.ProviderConfigAddr(), false)
+	}
+	for _, rc := range c.Module.DataResources {
+		maybePutLocal(rc.ProviderConfigAddr(), false)
+	}
+	for _, ic := range c.Module.Import {
+		if ic.ProviderConfigRef != nil {
+			maybePutLocal(addrs.LocalProviderConfig{
+				LocalName: ic.ProviderConfigRef.Name,
+				Alias:     ic.ProviderConfigRef.Alias,
+			}, false)
+		} else {
+			maybePut(addrs.RootProviderConfig{
+				Provider: ic.Provider,
+			})
+		}
+	}
+	for _, mc := range c.Module.ModuleCalls {
+		for _, pp := range mc.Providers {
+			maybePutLocal(pp.InParent.Addr(), false)
+		}
+		// If there aren't any explicitly-passed providers then
+		// the module implicitly requires a default configuration
+		// for each provider the child module mentions, since
+		// that would get implicitly passed into the child by
+		// Terraform Core.
+		// (We don't need to visit the child module at all if
+		// the call has an explicit "providers" argument, because
+		// we require that to be exhaustive when present.)
+		if len(mc.Providers) == 0 {
+			child := c.Children[mc.Name]
+			childReqs := child.EffectiveRequiredProviderConfigs()
+			for _, childReq := range childReqs.Keys() {
+				if childReq.Alias != "" {
+					continue // only default provider configs are eligible for this implicit treatment
+				}
+				// We must reinterpret the child address to appear as
+				// if written in its parent (our current module).
+				maybePut(addrs.RootProviderConfig{
+					Provider: childReq.Provider,
+				})
+			}
+		}
+	}
+
+	return ret
+}
+
 func (c *Config) CheckCoreVersionRequirements() hcl.Diagnostics {
 	var diags hcl.Diagnostics
 
@@ -867,94 +1063,3 @@ func (c *Config) CheckCoreVersionRequirements() hcl.Diagnostics {
 
 	return diags
 }
-
-// TransformForTest prepares the config to execute the given test.
-//
-// This function directly edits the config that is to be tested, and returns a
-// function that will reset the config back to its original state.
-//
-// Tests will call this before they execute, and then call the deferred function
-// to reset the config before the next test.
-func (c *Config) TransformForTest(run *TestRun, file *TestFile) (func(), hcl.Diagnostics) {
-	var diags hcl.Diagnostics
-
-	// Currently, we only need to override the provider settings.
-	//
-	// We can have a set of providers defined within the config, we can also
-	// have a set of providers defined within the test file. Then the run can
-	// also specify a set of overrides that tell Terraform exactly which
-	// providers from the test file to apply into the config.
-	//
-	// The process here is as follows:
-	//   1. Take all the providers in the original config keyed by name.alias,
-	//      we call this `previous`
-	//   2. Copy them all into a new map, we call this `next`.
-	//   3a. If the run has configuration specifying provider overrides, we copy
-	//       only the specified providers from the test file into `next`. While
-	//       doing this we ensure to preserve the name and alias from the
-	//       original config.
-	//   3b. If the run has no override configuration, we copy all the providers
-	//       from the test file into `next`, overriding all providers with name
-	//       collisions from the original config.
-	//   4. We then modify the original configuration so that the providers it
-	//      holds are the combination specified by the original config, the test
-	//      file and the run file.
-	//   5. We then return a function that resets the original config back to
-	//      its original state. This can be called by the surrounding test once
-	//      completed so future run blocks can safely execute.
-
-	// First, initialise `previous` and `next`. `previous` contains a backup of
-	// the providers from the original config. `next` contains the set of
-	// providers that will be used by the test. `next` starts with the set of
-	// providers from the original config.
-	previous := c.Module.ProviderConfigs
-	next := make(map[string]*Provider)
-	for key, value := range previous {
-		next[key] = value
-	}
-
-	if run != nil && len(run.Providers) > 0 {
-		// Then we'll only copy over and overwrite the specific providers asked
-		// for by this run block.
-
-		for _, ref := range run.Providers {
-
-			testProvider, ok := file.Providers[ref.InParent.String()]
-			if !ok {
-				// Then this reference was invalid as we didn't have the
-				// specified provider in the parent. This should have been
-				// caught earlier in validation anyway so is unlikely to happen.
-				diags = append(diags, &hcl.Diagnostic{
-					Severity: hcl.DiagError,
-					Summary:  fmt.Sprintf("Missing provider definition for %s", ref.InParent.String()),
-					Detail:   "This provider block references a provider definition that does not exist.",
-					Subject:  ref.InParent.NameRange.Ptr(),
-				})
-				continue
-			}
-
-			next[ref.InChild.String()] = &Provider{
-				Name:       ref.InChild.Name,
-				NameRange:  ref.InChild.NameRange,
-				Alias:      ref.InChild.Alias,
-				AliasRange: ref.InChild.AliasRange,
-				Version:    testProvider.Version,
-				Config:     testProvider.Config,
-				DeclRange:  testProvider.DeclRange,
-			}
-
-		}
-	} else {
-		// Otherwise, let's copy over and overwrite all providers specified by
-		// the test file itself.
-		for key, provider := range file.Providers {
-			next[key] = provider
-		}
-	}
-
-	c.Module.ProviderConfigs = next
-	return func() {
-		// Reset the original config within the returned function.
-		c.Module.ProviderConfigs = previous
-	}, diags
-}
