@@ -35,6 +35,18 @@ func TestBlockValueMarks(t *testing.T) {
 					Nesting: NestingList,
 				},
 			},
+			"nested_sensitive": {
+				NestedType: &Object{
+					Attributes: map[string]*Attribute{
+						"boop": {
+							Type: cty.String,
+						},
+					},
+					Nesting: NestingList,
+				},
+				Sensitive: true,
+				Optional:  true,
+			},
 		},
 
 		BlockTypes: map[string]*NestedBlock{
@@ -76,6 +88,9 @@ func TestBlockValueMarks(t *testing.T) {
 					"boop": cty.String,
 					"honk": cty.String,
 				}))),
+				"nested_sensitive": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
+					"boop": cty.String,
+				}))),
 				"list": cty.UnknownVal(schema.BlockTypes["list"].ImpliedType()),
 			}),
 			cty.ObjectVal(map[string]cty.Value{
@@ -85,6 +100,9 @@ func TestBlockValueMarks(t *testing.T) {
 					"boop": cty.String,
 					"honk": cty.String,
 				}))),
+				"nested_sensitive": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
+					"boop": cty.String,
+				}))).Mark(marks.Sensitive),
 				"list": cty.UnknownVal(schema.BlockTypes["list"].ImpliedType()),
 			}),
 		},
@@ -96,6 +114,10 @@ func TestBlockValueMarks(t *testing.T) {
 					"boop": cty.String,
 					"honk": cty.String,
 				}))),
+				"nested_sensitive": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
+					"boop": cty.String,
+					"honk": cty.String,
+				}))),
 				"list": cty.ListVal([]cty.Value{
 					cty.ObjectVal(map[string]cty.Value{
 						"sensitive":   cty.UnknownVal(cty.String),
@@ -114,6 +136,9 @@ func TestBlockValueMarks(t *testing.T) {
 					"boop": cty.String,
 					"honk": cty.String,
 				}))),
+				"nested_sensitive": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
+					"boop": cty.String,
+				}))).Mark(marks.Sensitive),
 				"list": cty.ListVal([]cty.Value{
 					cty.ObjectVal(map[string]cty.Value{
 						"sensitive":   cty.UnknownVal(cty.String).Mark(marks.Sensitive),
@@ -144,6 +169,9 @@ func TestBlockValueMarks(t *testing.T) {
 						"honk": cty.UnknownVal(cty.String),
 					}),
 				}),
+				"nested_sensitive": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
+					"boop": cty.String,
+				}))),
 				"list": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
 					"sensitive":   cty.String,
 					"unsensitive": cty.String,
@@ -166,6 +194,9 @@ func TestBlockValueMarks(t *testing.T) {
 						"honk": cty.UnknownVal(cty.String).Mark(marks.Sensitive),
 					}),
 				}),
+				"nested_sensitive": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
+					"boop": cty.String,
+				}))).Mark(marks.Sensitive),
 				"list": cty.NullVal(cty.List(cty.Object(map[string]cty.Type{
 					"sensitive":   cty.String,
 					"unsensitive": cty.String,
@@ -176,9 +207,19 @@ func TestBlockValueMarks(t *testing.T) {
 
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
-			got := tc.given.MarkWithPaths(schema.ValueMarks(tc.given, nil))
-			if !got.RawEquals(tc.expect) {
-				t.Fatalf("\nexpected: %#v\ngot:      %#v\n", tc.expect, got)
+			given, err := schema.CoerceValue(tc.given)
+			if err != nil {
+				t.Fatal(err)
+			}
+			expect, err := schema.CoerceValue(tc.expect)
+			if err != nil {
+				t.Fatal(err)
+			}
+
+			sensitivePaths := schema.SensitivePaths(given, nil)
+			got := marks.MarkPaths(given, marks.Sensitive, sensitivePaths)
+			if !expect.RawEquals(got) {
+				t.Fatalf("\nexpected: %#v\ngot:      %#v\n", expect, got)
 			}
 		})
 	}
