@@ -13,13 +13,17 @@ const (
 	DeleteThenCreate Action = '∓'
 	CreateThenDelete Action = '±'
 	Delete           Action = '-'
+	Forget           Action = '.'
+	CreateThenForget Action = '⨥'
+	Open             Action = '⟃'
+	Renew            Action = '⟳'
+	Close            Action = '⫏'
 )
 
 //go:generate go run golang.org/x/tools/cmd/stringer -type Action
 
-// IsReplace returns true if the action is one of the two actions that
-// represents replacing an existing object with a new object:
-// DeleteThenCreate or CreateThenDelete.
+// IsReplace returns true if the action is one of the actions that
+// represent replacing an existing object with a new object.
 func (a Action) IsReplace() bool {
-	return a == DeleteThenCreate || a == CreateThenDelete
+	return a == DeleteThenCreate || a == CreateThenDelete || a == CreateThenForget
 }
