@@ -4,20 +4,23 @@
 package plans
 
 import (
+	"fmt"
+
 	"github.com/zclconf/go-cty/cty"
 
 	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/lang/marks"
+	"github.com/hashicorp/terraform/internal/providers"
+	"github.com/hashicorp/terraform/internal/schemarepo"
 	"github.com/hashicorp/terraform/internal/states"
+	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
 // Changes describes various actions that Terraform will attempt to take if
 // the corresponding plan is applied.
-//
-// A Changes object can be rendered into a visual diff (by the caller, using
-// code in another package) for display to the user.
 type Changes struct {
 	// Resources tracks planned changes to resource instance objects.
-	Resources []*ResourceInstanceChangeSrc
+	Resources []*ResourceInstanceChange
 
 	// Outputs tracks planned changes output values.
 	//
@@ -27,7 +30,7 @@ type Changes struct {
 	// externally-visible, while other outputs are implementation details and
 	// can be easily re-calculated during the apply phase. Therefore only root
 	// module outputs will survive a round-trip through a plan file.
-	Outputs []*OutputChangeSrc
+	Outputs []*OutputChange
 }
 
 // NewChanges returns a valid Changes object that describes no changes.
@@ -35,30 +38,55 @@ func NewChanges() *Changes {
 	return &Changes{}
 }
 
-func (c *Changes) Empty() bool {
-	for _, res := range c.Resources {
-		if res.Action != NoOp || res.Moved() {
-			return false
+// Encode encodes all the stored resource and output changes into a new *ChangeSrc value
+func (c *Changes) Encode(schemas *schemarepo.Schemas) (*ChangesSrc, error) {
+	// a plan is always built even when there are errors, so make sure to return
+	// a valid changesSrc.
+	changesSrc := NewChangesSrc()
+
+	for _, rc := range c.Resources {
+		p, ok := schemas.Providers[rc.ProviderAddr.Provider]
+		if !ok {
+			return changesSrc, fmt.Errorf("Changes.Encode: missing provider %s for %s", rc.ProviderAddr, rc.Addr)
 		}
 
-		if res.Importing != nil {
-			return false
+		var schema providers.Schema
+		switch rc.Addr.Resource.Resource.Mode {
+		case addrs.ManagedResourceMode:
+			schema = p.ResourceTypes[rc.Addr.Resource.Resource.Type]
+		case addrs.DataResourceMode:
+			schema = p.DataSources[rc.Addr.Resource.Resource.Type]
+		default:
+			panic(fmt.Sprintf("unexpected resource mode %s", rc.Addr.Resource.Resource.Mode))
 		}
+
+		if schema.Block == nil {
+			return changesSrc, fmt.Errorf("Changes.Encode: missing schema for %s", rc.Addr)
+		}
+
+		rcs, err := rc.Encode(schema.Block.ImpliedType())
+		if err != nil {
+			return changesSrc, fmt.Errorf("Changes.Encode: %w", err)
+		}
+
+		changesSrc.Resources = append(changesSrc.Resources, rcs)
 	}
 
-	for _, out := range c.Outputs {
-		if out.Addr.Module.IsRoot() && out.Action != NoOp {
-			return false
+	for _, ocs := range c.Outputs {
+		oc, err := ocs.Encode()
+		if err != nil {
+			return changesSrc, err
 		}
+		changesSrc.Outputs = append(changesSrc.Outputs, oc)
 	}
 
-	return true
+	return changesSrc, nil
 }
 
 // ResourceInstance returns the planned change for the current object of the
 // resource instance of the given address, if any. Returns nil if no change is
 // planned.
-func (c *Changes) ResourceInstance(addr addrs.AbsResourceInstance) *ResourceInstanceChangeSrc {
+func (c *Changes) ResourceInstance(addr addrs.AbsResourceInstance) *ResourceInstanceChange {
 	for _, rc := range c.Resources {
 		if rc.Addr.Equal(addr) && rc.DeposedKey == states.NotDeposed {
 			return rc
@@ -72,8 +100,8 @@ func (c *Changes) ResourceInstance(addr addrs.AbsResourceInstance) *ResourceInst
 // InstancesForAbsResource returns the planned change for the current objects
 // of the resource instances of the given address, if any. Returns nil if no
 // changes are planned.
-func (c *Changes) InstancesForAbsResource(addr addrs.AbsResource) []*ResourceInstanceChangeSrc {
-	var changes []*ResourceInstanceChangeSrc
+func (c *Changes) InstancesForAbsResource(addr addrs.AbsResource) []*ResourceInstanceChange {
+	var changes []*ResourceInstanceChange
 	for _, rc := range c.Resources {
 		resAddr := rc.Addr.ContainingResource()
 		if resAddr.Equal(addr) && rc.DeposedKey == states.NotDeposed {
@@ -87,8 +115,8 @@ func (c *Changes) InstancesForAbsResource(addr addrs.AbsResource) []*ResourceIns
 // InstancesForConfigResource returns the planned change for the current objects
 // of the resource instances of the given address, if any. Returns nil if no
 // changes are planned.
-func (c *Changes) InstancesForConfigResource(addr addrs.ConfigResource) []*ResourceInstanceChangeSrc {
-	var changes []*ResourceInstanceChangeSrc
+func (c *Changes) InstancesForConfigResource(addr addrs.ConfigResource) []*ResourceInstanceChange {
+	var changes []*ResourceInstanceChange
 	for _, rc := range c.Resources {
 		resAddr := rc.Addr.ContainingResource().Config()
 		if resAddr.Equal(addr) && rc.DeposedKey == states.NotDeposed {
@@ -102,7 +130,7 @@ func (c *Changes) InstancesForConfigResource(addr addrs.ConfigResource) []*Resou
 // ResourceInstanceDeposed returns the plan change of a deposed object of
 // the resource instance of the given address, if any. Returns nil if no change
 // is planned.
-func (c *Changes) ResourceInstanceDeposed(addr addrs.AbsResourceInstance, key states.DeposedKey) *ResourceInstanceChangeSrc {
+func (c *Changes) ResourceInstanceDeposed(addr addrs.AbsResourceInstance, key states.DeposedKey) *ResourceInstanceChange {
 	for _, rc := range c.Resources {
 		if rc.Addr.Equal(addr) && rc.DeposedKey == key {
 			return rc
@@ -115,7 +143,7 @@ func (c *Changes) ResourceInstanceDeposed(addr addrs.AbsResourceInstance, key st
 // OutputValue returns the planned change for the output value with the
 //
 //	given address, if any. Returns nil if no change is planned.
-func (c *Changes) OutputValue(addr addrs.AbsOutputValue) *OutputChangeSrc {
+func (c *Changes) OutputValue(addr addrs.AbsOutputValue) *OutputChange {
 	for _, oc := range c.Outputs {
 		if oc.Addr.Equal(addr) {
 			return oc
@@ -126,8 +154,8 @@ func (c *Changes) OutputValue(addr addrs.AbsOutputValue) *OutputChangeSrc {
 }
 
 // RootOutputValues returns planned changes for all outputs of the root module.
-func (c *Changes) RootOutputValues() []*OutputChangeSrc {
-	var res []*OutputChangeSrc
+func (c *Changes) RootOutputValues() []*OutputChange {
+	var res []*OutputChange
 
 	for _, oc := range c.Outputs {
 		// we can't evaluate root module outputs
@@ -145,8 +173,8 @@ func (c *Changes) RootOutputValues() []*OutputChangeSrc {
 // OutputValues returns planned changes for all outputs for all module
 // instances that reside in the parent path.  Returns nil if no changes are
 // planned.
-func (c *Changes) OutputValues(parent addrs.ModuleInstance, module addrs.ModuleCall) []*OutputChangeSrc {
-	var res []*OutputChangeSrc
+func (c *Changes) OutputValues(parent addrs.ModuleInstance, module addrs.ModuleCall) []*OutputChange {
+	var res []*OutputChange
 
 	for _, oc := range c.Outputs {
 		// we can't evaluate root module outputs
@@ -275,6 +303,15 @@ func (rc *ResourceInstanceChange) Encode(ty cty.Type) (*ResourceInstanceChangeSr
 	}, err
 }
 
+func (rc *ResourceInstanceChange) DeepCopy() *ResourceInstanceChange {
+	if rc == nil {
+		return rc
+	}
+
+	ret := *rc
+	return &ret
+}
+
 func (rc *ResourceInstanceChange) Moved() bool {
 	return !rc.Addr.Equal(rc.PrevRunAddr)
 }
@@ -506,8 +543,23 @@ func (oc *OutputChange) Encode() (*OutputChangeSrc, error) {
 // The fields in here are subject to change, so downstream consumers should be
 // prepared for backwards compatibility in case the contents changes.
 type Importing struct {
-	// ID is the original ID of the imported resource.
-	ID string
+	ID cty.Value
+}
+
+// Encode converts the Importing object into a form suitable for serialization
+// to a plan file.
+func (i *Importing) Encode() *ImportingSrc {
+	if i == nil {
+		return nil
+	}
+	if i.ID.IsKnown() {
+		return &ImportingSrc{
+			ID: i.ID.AsString(),
+		}
+	}
+	return &ImportingSrc{
+		Unknown: true,
+	}
 }
 
 // Change describes a single change with a given action.
@@ -525,6 +577,7 @@ type Change struct {
 	//              value after update.
 	//     Replace  As with Update.
 	//     Delete   Before is the value prior to delete, and After is always nil.
+	//     Forget   As with Delete.
 	//
 	// Unknown values may appear anywhere within the Before and After values,
 	// either as the values themselves or as nested elements within known
@@ -554,40 +607,47 @@ type Change struct {
 // to call the corresponding Encode method of that struct rather than working
 // directly with its embedded Change.
 func (c *Change) Encode(ty cty.Type) (*ChangeSrc, error) {
-	// Storing unmarked values so that we can encode unmarked values
-	// and save the PathValueMarks for re-marking the values later
-	var beforeVM, afterVM []cty.PathValueMarks
-	unmarkedBefore := c.Before
-	unmarkedAfter := c.After
-
-	if c.Before.ContainsMarked() {
-		unmarkedBefore, beforeVM = c.Before.UnmarkDeepWithPaths()
+	// We can't serialize value marks directly so we'll need to extract the
+	// sensitive marks and store them in a separate field.
+	//
+	// We don't accept any other marks here. The caller should have dealt
+	// with those somehow and replaced them with unmarked placeholders before
+	// writing the value into the state.
+	unmarkedBefore, marksesBefore := c.Before.UnmarkDeepWithPaths()
+	unmarkedAfter, marksesAfter := c.After.UnmarkDeepWithPaths()
+	sensitiveAttrsBefore, unsupportedMarksesBefore := marks.PathsWithMark(marksesBefore, marks.Sensitive)
+	sensitiveAttrsAfter, unsupportedMarksesAfter := marks.PathsWithMark(marksesAfter, marks.Sensitive)
+	if len(unsupportedMarksesBefore) != 0 {
+		return nil, fmt.Errorf(
+			"prior value %s: can't serialize value marked with %#v (this is a bug in Terraform)",
+			tfdiags.FormatCtyPath(unsupportedMarksesBefore[0].Path),
+			unsupportedMarksesBefore[0].Marks,
+		)
 	}
+	if len(unsupportedMarksesAfter) != 0 {
+		return nil, fmt.Errorf(
+			"new value %s: can't serialize value marked with %#v (this is a bug in Terraform)",
+			tfdiags.FormatCtyPath(unsupportedMarksesAfter[0].Path),
+			unsupportedMarksesAfter[0].Marks,
+		)
+	}
+
 	beforeDV, err := NewDynamicValue(unmarkedBefore, ty)
 	if err != nil {
 		return nil, err
 	}
-
-	if c.After.ContainsMarked() {
-		unmarkedAfter, afterVM = c.After.UnmarkDeepWithPaths()
-	}
 	afterDV, err := NewDynamicValue(unmarkedAfter, ty)
 	if err != nil {
 		return nil, err
 	}
 
-	var importing *ImportingSrc
-	if c.Importing != nil {
-		importing = &ImportingSrc{ID: c.Importing.ID}
-	}
-
 	return &ChangeSrc{
-		Action:          c.Action,
-		Before:          beforeDV,
-		After:           afterDV,
-		BeforeValMarks:  beforeVM,
-		AfterValMarks:   afterVM,
-		Importing:       importing,
-		GeneratedConfig: c.GeneratedConfig,
+		Action:               c.Action,
+		Before:               beforeDV,
+		After:                afterDV,
+		BeforeSensitivePaths: sensitiveAttrsBefore,
+		AfterSensitivePaths:  sensitiveAttrsAfter,
+		Importing:            c.Importing.Encode(),
+		GeneratedConfig:      c.GeneratedConfig,
 	}, nil
 }
