@@ -6,19 +6,177 @@ package plans
 import (
 	"fmt"
 
-	"github.com/hashicorp/terraform/internal/addrs"
-	"github.com/hashicorp/terraform/internal/states"
 	"github.com/zclconf/go-cty/cty"
+
+	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/lang/marks"
+	"github.com/hashicorp/terraform/internal/providers"
+	"github.com/hashicorp/terraform/internal/schemarepo"
+	"github.com/hashicorp/terraform/internal/states"
 )
 
+// ChangesSrc describes various actions that Terraform will attempt to take if
+// the corresponding plan is applied.
+//
+// A Changes object can be rendered into a visual diff (by the caller, using
+// code in another package) for display to the user.
+type ChangesSrc struct {
+	// Resources tracks planned changes to resource instance objects.
+	Resources []*ResourceInstanceChangeSrc
+
+	// Outputs tracks planned changes output values.
+	//
+	// Note that although an in-memory plan contains planned changes for
+	// outputs throughout the configuration, a plan serialized
+	// to disk retains only the root outputs because they are
+	// externally-visible, while other outputs are implementation details and
+	// can be easily re-calculated during the apply phase. Therefore only root
+	// module outputs will survive a round-trip through a plan file.
+	Outputs []*OutputChangeSrc
+}
+
+func NewChangesSrc() *ChangesSrc {
+	return &ChangesSrc{}
+}
+
+func (c *ChangesSrc) Empty() bool {
+	for _, res := range c.Resources {
+		if res.Action != NoOp || res.Moved() {
+			return false
+		}
+
+		if res.Importing != nil {
+			return false
+		}
+	}
+
+	for _, out := range c.Outputs {
+		if out.Addr.Module.IsRoot() && out.Action != NoOp {
+			return false
+		}
+	}
+
+	return true
+}
+
+// ResourceInstance returns the planned change for the current object of the
+// resource instance of the given address, if any. Returns nil if no change is
+// planned.
+func (c *ChangesSrc) ResourceInstance(addr addrs.AbsResourceInstance) *ResourceInstanceChangeSrc {
+	for _, rc := range c.Resources {
+		if rc.Addr.Equal(addr) && rc.DeposedKey == states.NotDeposed {
+			return rc
+		}
+	}
+
+	return nil
+}
+
+// ResourceInstanceDeposed returns the plan change of a deposed object of
+// the resource instance of the given address, if any. Returns nil if no change
+// is planned.
+func (c *ChangesSrc) ResourceInstanceDeposed(addr addrs.AbsResourceInstance, key states.DeposedKey) *ResourceInstanceChangeSrc {
+	for _, rc := range c.Resources {
+		if rc.Addr.Equal(addr) && rc.DeposedKey == key {
+			return rc
+		}
+	}
+
+	return nil
+}
+
+// OutputValue returns the planned change for the output value with the
+//
+//	given address, if any. Returns nil if no change is planned.
+func (c *ChangesSrc) OutputValue(addr addrs.AbsOutputValue) *OutputChangeSrc {
+	for _, oc := range c.Outputs {
+		if oc.Addr.Equal(addr) {
+			return oc
+		}
+	}
+
+	return nil
+}
+
+// Decode decodes all the stored resource and output changes into a new *Changes value.
+func (c *ChangesSrc) Decode(schemas *schemarepo.Schemas) (*Changes, error) {
+	changes := NewChanges()
+
+	for _, rcs := range c.Resources {
+		p, ok := schemas.Providers[rcs.ProviderAddr.Provider]
+		if !ok {
+			return nil, fmt.Errorf("ChangesSrc.Decode: missing provider %s for %s", rcs.ProviderAddr, rcs.Addr)
+		}
+
+		var schema providers.Schema
+		switch rcs.Addr.Resource.Resource.Mode {
+		case addrs.ManagedResourceMode:
+			schema = p.ResourceTypes[rcs.Addr.Resource.Resource.Type]
+		case addrs.DataResourceMode:
+			schema = p.DataSources[rcs.Addr.Resource.Resource.Type]
+		default:
+			panic(fmt.Sprintf("unexpected resource mode %s", rcs.Addr.Resource.Resource.Mode))
+		}
+
+		if schema.Block == nil {
+			return nil, fmt.Errorf("ChangesSrc.Decode: missing schema for %s", rcs.Addr)
+		}
+
+		rc, err := rcs.Decode(schema.Block.ImpliedType())
+		if err != nil {
+			return nil, err
+		}
+
+		rc.Before = marks.MarkPaths(rc.Before, marks.Sensitive, rcs.BeforeSensitivePaths)
+		rc.After = marks.MarkPaths(rc.After, marks.Sensitive, rcs.AfterSensitivePaths)
+
+		changes.Resources = append(changes.Resources, rc)
+	}
+
+	for _, ocs := range c.Outputs {
+		oc, err := ocs.Decode()
+		if err != nil {
+			return nil, err
+		}
+		changes.Outputs = append(changes.Outputs, oc)
+	}
+	return changes, nil
+}
+
+// AppendResourceInstanceChange records the given resource instance change in
+// the set of planned resource changes.
+func (c *ChangesSrc) AppendResourceInstanceChange(change *ResourceInstanceChangeSrc) {
+	if c == nil {
+		panic("AppendResourceInstanceChange on nil ChangesSync")
+	}
+
+	s := change.DeepCopy()
+	c.Resources = append(c.Resources, s)
+}
+
 // ResourceInstanceChangeSrc is a not-yet-decoded ResourceInstanceChange.
 // Pass the associated resource type's schema type to method Decode to
 // obtain a ResourceInstanceChange.
 type ResourceInstanceChangeSrc struct {
 	// Addr is the absolute address of the resource instance that the change
 	// will apply to.
+	//
+	// THIS IS NOT A SUFFICIENT UNIQUE IDENTIFIER! It doesn't consider the
+	// fact that multiple objects for the same resource instance might be
+	// present in the same plan; use the ObjectAddr method instead if you
+	// need a unique address for a particular change.
 	Addr addrs.AbsResourceInstance
 
+	// DeposedKey is the identifier for a deposed object associated with the
+	// given instance, or states.NotDeposed if this change applies to the
+	// current object.
+	//
+	// A Replace change for a resource with create_before_destroy set will
+	// create a new DeposedKey temporarily during replacement. In that case,
+	// DeposedKey in the plan is always states.NotDeposed, representing that
+	// the current object is being replaced with the deposed.
+	DeposedKey states.DeposedKey
+
 	// PrevRunAddr is the absolute address that this resource instance had at
 	// the conclusion of a previous run.
 	//
@@ -32,16 +190,6 @@ type ResourceInstanceChangeSrc struct {
 	// aims to detect and react to the movement of instances between addresses.
 	PrevRunAddr addrs.AbsResourceInstance
 
-	// DeposedKey is the identifier for a deposed object associated with the
-	// given instance, or states.NotDeposed if this change applies to the
-	// current object.
-	//
-	// A Replace change for a resource with create_before_destroy set will
-	// create a new DeposedKey temporarily during replacement. In that case,
-	// DeposedKey in the plan is always states.NotDeposed, representing that
-	// the current object is being replaced with the deposed.
-	DeposedKey states.DeposedKey
-
 	// Provider is the address of the provider configuration that was used
 	// to plan this change, and thus the configuration that must also be
 	// used to apply it.
@@ -74,6 +222,13 @@ type ResourceInstanceChangeSrc struct {
 	Private []byte
 }
 
+func (rcs *ResourceInstanceChangeSrc) ObjectAddr() addrs.AbsResourceInstanceObject {
+	return addrs.AbsResourceInstanceObject{
+		ResourceInstance: rcs.Addr,
+		DeposedKey:       rcs.DeposedKey,
+	}
+}
+
 // Decode unmarshals the raw representation of the instance object being
 // changed. Pass the implied type of the corresponding resource type schema
 // for correct operation.
@@ -193,6 +348,26 @@ func (ocs *OutputChangeSrc) DeepCopy() *OutputChangeSrc {
 type ImportingSrc struct {
 	// ID is the original ID of the imported resource.
 	ID string
+
+	// Unknown is true if the ID was unknown when we tried to import it. This
+	// should only be true if the overall change is embedded within a deferred
+	// action.
+	Unknown bool
+}
+
+// Decode unmarshals the raw representation of the importing action.
+func (is *ImportingSrc) Decode() *Importing {
+	if is == nil {
+		return nil
+	}
+	if is.Unknown {
+		return &Importing{
+			ID: cty.UnknownVal(cty.String),
+		}
+	}
+	return &Importing{
+		ID: cty.StringVal(is.ID),
+	}
 }
 
 // ChangeSrc is a not-yet-decoded Change.
@@ -205,12 +380,13 @@ type ChangeSrc struct {
 	// storage.
 	Before, After DynamicValue
 
-	// BeforeValMarks and AfterValMarks are stored path+mark combinations
-	// that might be discovered when encoding a change. Marks are removed
-	// to enable encoding (marked values cannot be marshalled), and so storing
-	// the path+mark combinations allow us to re-mark the value later
-	// when, for example, displaying the diff to the UI.
-	BeforeValMarks, AfterValMarks []cty.PathValueMarks
+	// BeforeSensitivePaths and AfterSensitivePaths are the paths for any
+	// values in Before or After (respectively) that are considered to be
+	// sensitive. The sensitive marks are removed from the in-memory values
+	// to enable encoding (marked values cannot be marshalled), and so we
+	// store the sensitive paths to allow re-marking later when we decode
+	// the serialized change.
+	BeforeSensitivePaths, AfterSensitivePaths []cty.Path
 
 	// Importing is present if the resource is being imported as part of this
 	// change.
@@ -251,16 +427,11 @@ func (cs *ChangeSrc) Decode(ty cty.Type) (*Change, error) {
 		}
 	}
 
-	var importing *Importing
-	if cs.Importing != nil {
-		importing = &Importing{ID: cs.Importing.ID}
-	}
-
 	return &Change{
 		Action:          cs.Action,
-		Before:          before.MarkWithPaths(cs.BeforeValMarks),
-		After:           after.MarkWithPaths(cs.AfterValMarks),
-		Importing:       importing,
+		Before:          marks.MarkPaths(before, marks.Sensitive, cs.BeforeSensitivePaths),
+		After:           marks.MarkPaths(after, marks.Sensitive, cs.AfterSensitivePaths),
+		Importing:       cs.Importing.Decode(),
 		GeneratedConfig: cs.GeneratedConfig,
 	}, nil
 }
