@@ -6,7 +6,6 @@ package planfile
 import (
 	"fmt"
 	"io"
-	"io/ioutil"
 	"time"
 
 	"github.com/zclconf/go-cty/cty"
@@ -14,11 +13,13 @@ import (
 
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/checks"
+	"github.com/hashicorp/terraform/internal/collections"
 	"github.com/hashicorp/terraform/internal/lang/globalref"
-	"github.com/hashicorp/terraform/internal/lang/marks"
 	"github.com/hashicorp/terraform/internal/plans"
-	"github.com/hashicorp/terraform/internal/plans/internal/planproto"
+	"github.com/hashicorp/terraform/internal/plans/planproto"
+	"github.com/hashicorp/terraform/internal/providers"
 	"github.com/hashicorp/terraform/internal/states"
+	"github.com/hashicorp/terraform/internal/tfdiags"
 	"github.com/hashicorp/terraform/version"
 )
 
@@ -37,7 +38,7 @@ const tfplanFilename = "tfplan"
 // a plan file, which is stored in a special file in the archive called
 // "tfplan".
 func readTfplan(r io.Reader) (*plans.Plan, error) {
-	src, err := ioutil.ReadAll(r)
+	src, err := io.ReadAll(r)
 	if err != nil {
 		return nil, err
 	}
@@ -58,25 +59,22 @@ func readTfplan(r io.Reader) (*plans.Plan, error) {
 
 	plan := &plans.Plan{
 		VariableValues: map[string]plans.DynamicValue{},
-		Changes: &plans.Changes{
+		Changes: &plans.ChangesSrc{
 			Outputs:   []*plans.OutputChangeSrc{},
 			Resources: []*plans.ResourceInstanceChangeSrc{},
 		},
-		DriftedResources: []*plans.ResourceInstanceChangeSrc{},
-		Checks:           &states.CheckResults{},
+		DriftedResources:  []*plans.ResourceInstanceChangeSrc{},
+		DeferredResources: []*plans.DeferredResourceInstanceChangeSrc{},
+		Checks:            &states.CheckResults{},
 	}
 
+	plan.Applyable = rawPlan.Applyable
+	plan.Complete = rawPlan.Complete
 	plan.Errored = rawPlan.Errored
 
-	switch rawPlan.UiMode {
-	case planproto.Mode_NORMAL:
-		plan.UIMode = plans.NormalMode
-	case planproto.Mode_DESTROY:
-		plan.UIMode = plans.DestroyMode
-	case planproto.Mode_REFRESH_ONLY:
-		plan.UIMode = plans.RefreshOnlyMode
-	default:
-		return nil, fmt.Errorf("plan has invalid mode %s", rawPlan.UiMode)
+	plan.UIMode, err = planproto.FromMode(rawPlan.UiMode)
+	if err != nil {
+		return nil, err
 	}
 
 	for _, rawOC := range rawPlan.OutputChanges {
@@ -96,98 +94,14 @@ func readTfplan(r io.Reader) (*plans.Plan, error) {
 		})
 	}
 
-	plan.Checks.ConfigResults = addrs.MakeMap[addrs.ConfigCheckable, *states.CheckResultAggregate]()
-	for _, rawCRs := range rawPlan.CheckResults {
-		aggr := &states.CheckResultAggregate{}
-		switch rawCRs.Status {
-		case planproto.CheckResults_UNKNOWN:
-			aggr.Status = checks.StatusUnknown
-		case planproto.CheckResults_PASS:
-			aggr.Status = checks.StatusPass
-		case planproto.CheckResults_FAIL:
-			aggr.Status = checks.StatusFail
-		case planproto.CheckResults_ERROR:
-			aggr.Status = checks.StatusError
-		default:
-			return nil, fmt.Errorf("aggregate check results for %s have unsupported status %#v", rawCRs.ConfigAddr, rawCRs.Status)
-		}
-
-		var objKind addrs.CheckableKind
-		switch rawCRs.Kind {
-		case planproto.CheckResults_RESOURCE:
-			objKind = addrs.CheckableResource
-		case planproto.CheckResults_OUTPUT_VALUE:
-			objKind = addrs.CheckableOutputValue
-		case planproto.CheckResults_CHECK:
-			objKind = addrs.CheckableCheck
-		case planproto.CheckResults_INPUT_VARIABLE:
-			objKind = addrs.CheckableInputVariable
-		default:
-			return nil, fmt.Errorf("aggregate check results for %s have unsupported object kind %s", rawCRs.ConfigAddr, objKind)
-		}
-
-		// Some trickiness here: we only have an address parser for
-		// addrs.Checkable and not for addrs.ConfigCheckable, but that's okay
-		// because once we have an addrs.Checkable we can always derive an
-		// addrs.ConfigCheckable from it, and a ConfigCheckable should always
-		// be the same syntax as a Checkable with no index information and
-		// thus we can reuse the same parser for both here.
-		configAddrProxy, diags := addrs.ParseCheckableStr(objKind, rawCRs.ConfigAddr)
-		if diags.HasErrors() {
-			return nil, diags.Err()
-		}
-		configAddr := configAddrProxy.ConfigCheckable()
-		if configAddr.String() != configAddrProxy.String() {
-			// This is how we catch if the config address included index
-			// information that would be allowed in a Checkable but not
-			// in a ConfigCheckable.
-			return nil, fmt.Errorf("invalid checkable config address %s", rawCRs.ConfigAddr)
-		}
-
-		aggr.ObjectResults = addrs.MakeMap[addrs.Checkable, *states.CheckResultObject]()
-		for _, rawCR := range rawCRs.Objects {
-			objectAddr, diags := addrs.ParseCheckableStr(objKind, rawCR.ObjectAddr)
-			if diags.HasErrors() {
-				return nil, diags.Err()
-			}
-			if !addrs.Equivalent(objectAddr.ConfigCheckable(), configAddr) {
-				return nil, fmt.Errorf("checkable object %s should not be grouped under %s", objectAddr, configAddr)
-			}
-
-			obj := &states.CheckResultObject{
-				FailureMessages: rawCR.FailureMessages,
-			}
-			switch rawCR.Status {
-			case planproto.CheckResults_UNKNOWN:
-				obj.Status = checks.StatusUnknown
-			case planproto.CheckResults_PASS:
-				obj.Status = checks.StatusPass
-			case planproto.CheckResults_FAIL:
-				obj.Status = checks.StatusFail
-			case planproto.CheckResults_ERROR:
-				obj.Status = checks.StatusError
-			default:
-				return nil, fmt.Errorf("object check results for %s has unsupported status %#v", rawCR.ObjectAddr, rawCR.Status)
-			}
-
-			aggr.ObjectResults.Put(objectAddr, obj)
-		}
-		// If we ended up with no elements in the map then we'll just nil it,
-		// primarily just to make life easier for our round-trip tests.
-		if aggr.ObjectResults.Len() == 0 {
-			aggr.ObjectResults.Elems = nil
-		}
-
-		plan.Checks.ConfigResults.Put(configAddr, aggr)
-	}
-	// If we ended up with no elements in the map then we'll just nil it,
-	// primarily just to make life easier for our round-trip tests.
-	if plan.Checks.ConfigResults.Len() == 0 {
-		plan.Checks.ConfigResults.Elems = nil
+	checkResults, err := CheckResultsFromPlanProto(rawPlan.CheckResults)
+	if err != nil {
+		return nil, fmt.Errorf("failed to decode check results: %s", err)
 	}
+	plan.Checks = checkResults
 
 	for _, rawRC := range rawPlan.ResourceChanges {
-		change, err := resourceChangeFromTfplan(rawRC)
+		change, err := resourceChangeFromTfplan(rawRC, addrs.ParseAbsResourceInstanceStr)
 		if err != nil {
 			// errors from resourceChangeFromTfplan already include context
 			return nil, err
@@ -197,7 +111,7 @@ func readTfplan(r io.Reader) (*plans.Plan, error) {
 	}
 
 	for _, rawRC := range rawPlan.ResourceDrift {
-		change, err := resourceChangeFromTfplan(rawRC)
+		change, err := resourceChangeFromTfplan(rawRC, addrs.ParseAbsResourceInstanceStr)
 		if err != nil {
 			// errors from resourceChangeFromTfplan already include context
 			return nil, err
@@ -206,6 +120,15 @@ func readTfplan(r io.Reader) (*plans.Plan, error) {
 		plan.DriftedResources = append(plan.DriftedResources, change)
 	}
 
+	for _, rawDC := range rawPlan.DeferredChanges {
+		change, err := deferredChangeFromTfplan(rawDC)
+		if err != nil {
+			return nil, err
+		}
+
+		plan.DeferredResources = append(plan.DeferredResources, change)
+	}
+
 	for _, rawRA := range rawPlan.RelevantAttributes {
 		ra, err := resourceAttrFromTfplan(rawRA)
 		if err != nil {
@@ -238,6 +161,22 @@ func readTfplan(r io.Reader) (*plans.Plan, error) {
 		plan.VariableValues[name] = val
 	}
 
+	if len(rawPlan.ApplyTimeVariables) != 0 {
+		plan.ApplyTimeVariables = collections.NewSetCmp[string]()
+		for _, name := range rawPlan.ApplyTimeVariables {
+			plan.ApplyTimeVariables.Add(name)
+		}
+	}
+
+	for _, hash := range rawPlan.ProviderFunctionResults {
+		plan.ProviderFunctionResults = append(plan.ProviderFunctionResults,
+			providers.FunctionHash{
+				Key:    hash.Key,
+				Result: hash.Result,
+			},
+		)
+	}
+
 	if rawBackend := rawPlan.Backend; rawBackend == nil {
 		return nil, fmt.Errorf("plan file has no backend settings; backend settings are required")
 	} else {
@@ -259,7 +198,25 @@ func readTfplan(r io.Reader) (*plans.Plan, error) {
 	return plan, nil
 }
 
-func resourceChangeFromTfplan(rawChange *planproto.ResourceInstanceChange) (*plans.ResourceInstanceChangeSrc, error) {
+// ResourceChangeFromProto decodes an isolated resource instance change from
+// its representation as a protocol buffers message.
+//
+// This is used by the stackplan package, which includes planproto messages
+// in its own wire format while using a different overall container.
+func ResourceChangeFromProto(rawChange *planproto.ResourceInstanceChange) (*plans.ResourceInstanceChangeSrc, error) {
+	return resourceChangeFromTfplan(rawChange, addrs.ParseAbsResourceInstanceStr)
+}
+
+// DeferredResourceChangeFromProto decodes an isolated deferred resource
+// instance change from its representation as a protocol buffers message.
+//
+// This the same as ResourceChangeFromProto but internally allows for splat
+// addresses, which are not allowed outside deferred changes.
+func DeferredResourceChangeFromProto(rawChange *planproto.ResourceInstanceChange) (*plans.ResourceInstanceChangeSrc, error) {
+	return resourceChangeFromTfplan(rawChange, addrs.ParsePartialResourceInstanceStr)
+}
+
+func resourceChangeFromTfplan(rawChange *planproto.ResourceInstanceChange, parseAddr func(str string) (addrs.AbsResourceInstance, tfdiags.Diagnostics)) (*plans.ResourceInstanceChangeSrc, error) {
 	if rawChange == nil {
 		// Should never happen in practice, since protobuf can't represent
 		// a nil value in a list.
@@ -276,13 +233,13 @@ func resourceChangeFromTfplan(rawChange *planproto.ResourceInstanceChange) (*pla
 		return nil, fmt.Errorf("no instance address for resource instance change; perhaps this plan was created by a different version of Terraform?")
 	}
 
-	instAddr, diags := addrs.ParseAbsResourceInstanceStr(rawChange.Addr)
+	instAddr, diags := parseAddr(rawChange.Addr)
 	if diags.HasErrors() {
 		return nil, fmt.Errorf("invalid resource instance address %q: %w", rawChange.Addr, diags.Err())
 	}
 	prevRunAddr := instAddr
 	if rawChange.PrevRunAddr != "" {
-		prevRunAddr, diags = addrs.ParseAbsResourceInstanceStr(rawChange.PrevRunAddr)
+		prevRunAddr, diags = parseAddr(rawChange.PrevRunAddr)
 		if diags.HasErrors() {
 			return nil, fmt.Errorf("invalid resource instance previous run address %q: %w", rawChange.PrevRunAddr, diags.Err())
 		}
@@ -360,6 +317,35 @@ func resourceChangeFromTfplan(rawChange *planproto.ResourceInstanceChange) (*pla
 	return ret, nil
 }
 
+// ActionFromProto translates from the protobuf representation of change actions
+// into the "plans" package's representation, or returns an error if the
+// given action is unrecognized.
+func ActionFromProto(rawAction planproto.Action) (plans.Action, error) {
+	switch rawAction {
+	case planproto.Action_NOOP:
+		return plans.NoOp, nil
+	case planproto.Action_CREATE:
+		return plans.Create, nil
+	case planproto.Action_READ:
+		return plans.Read, nil
+	case planproto.Action_UPDATE:
+		return plans.Update, nil
+	case planproto.Action_DELETE:
+		return plans.Delete, nil
+	case planproto.Action_CREATE_THEN_DELETE:
+		return plans.CreateThenDelete, nil
+	case planproto.Action_DELETE_THEN_CREATE:
+		return plans.DeleteThenCreate, nil
+	case planproto.Action_FORGET:
+		return plans.Forget, nil
+	case planproto.Action_CREATE_THEN_FORGET:
+		return plans.CreateThenForget, nil
+	default:
+		return plans.NoOp, fmt.Errorf("invalid change action %s", rawAction)
+	}
+
+}
+
 func changeFromTfplan(rawChange *planproto.Change) (*plans.ChangeSrc, error) {
 	if rawChange == nil {
 		return nil, fmt.Errorf("change object is absent")
@@ -371,31 +357,35 @@ func changeFromTfplan(rawChange *planproto.Change) (*plans.ChangeSrc, error) {
 	// depending on the change action, and then decode.
 	beforeIdx, afterIdx := -1, -1
 
-	switch rawChange.Action {
-	case planproto.Action_NOOP:
-		ret.Action = plans.NoOp
+	var err error
+	ret.Action, err = ActionFromProto(rawChange.Action)
+	if err != nil {
+		return nil, err
+	}
+
+	switch ret.Action {
+	case plans.NoOp:
 		beforeIdx = 0
 		afterIdx = 0
-	case planproto.Action_CREATE:
-		ret.Action = plans.Create
+	case plans.Create:
 		afterIdx = 0
-	case planproto.Action_READ:
-		ret.Action = plans.Read
+	case plans.Read:
 		beforeIdx = 0
 		afterIdx = 1
-	case planproto.Action_UPDATE:
-		ret.Action = plans.Update
+	case plans.Update:
 		beforeIdx = 0
 		afterIdx = 1
-	case planproto.Action_DELETE:
-		ret.Action = plans.Delete
+	case plans.Delete:
 		beforeIdx = 0
-	case planproto.Action_CREATE_THEN_DELETE:
-		ret.Action = plans.CreateThenDelete
+	case plans.CreateThenDelete:
 		beforeIdx = 0
 		afterIdx = 1
-	case planproto.Action_DELETE_THEN_CREATE:
-		ret.Action = plans.DeleteThenCreate
+	case plans.DeleteThenCreate:
+		beforeIdx = 0
+		afterIdx = 1
+	case plans.Forget:
+		beforeIdx = 0
+	case plans.CreateThenForget:
 		beforeIdx = 0
 		afterIdx = 1
 	default:
@@ -431,25 +421,25 @@ func changeFromTfplan(rawChange *planproto.Change) (*plans.ChangeSrc, error) {
 
 	if rawChange.Importing != nil {
 		ret.Importing = &plans.ImportingSrc{
-			ID: rawChange.Importing.Id,
+			ID:      rawChange.Importing.Id,
+			Unknown: rawChange.Importing.Unknown,
 		}
 	}
 	ret.GeneratedConfig = rawChange.GeneratedConfig
 
-	sensitive := cty.NewValueMarks(marks.Sensitive)
-	beforeValMarks, err := pathValueMarksFromTfplan(rawChange.BeforeSensitivePaths, sensitive)
+	beforeValSensitiveAttrs, err := pathsFromTfplan(rawChange.BeforeSensitivePaths)
 	if err != nil {
 		return nil, fmt.Errorf("failed to decode before sensitive paths: %s", err)
 	}
-	afterValMarks, err := pathValueMarksFromTfplan(rawChange.AfterSensitivePaths, sensitive)
+	afterValSensitiveAttrs, err := pathsFromTfplan(rawChange.AfterSensitivePaths)
 	if err != nil {
 		return nil, fmt.Errorf("failed to decode after sensitive paths: %s", err)
 	}
-	if len(beforeValMarks) > 0 {
-		ret.BeforeValMarks = beforeValMarks
+	if len(beforeValSensitiveAttrs) > 0 {
+		ret.BeforeSensitivePaths = beforeValSensitiveAttrs
 	}
-	if len(afterValMarks) > 0 {
-		ret.AfterValMarks = afterValMarks
+	if len(afterValSensitiveAttrs) > 0 {
+		ret.AfterSensitivePaths = afterValSensitiveAttrs
 	}
 
 	return ret, nil
@@ -463,6 +453,44 @@ func valueFromTfplan(rawV *planproto.DynamicValue) (plans.DynamicValue, error) {
 	return plans.DynamicValue(rawV.Msgpack), nil
 }
 
+func deferredChangeFromTfplan(dc *planproto.DeferredResourceInstanceChange) (*plans.DeferredResourceInstanceChangeSrc, error) {
+	if dc == nil {
+		return nil, fmt.Errorf("deferred change object is absent")
+	}
+
+	change, err := resourceChangeFromTfplan(dc.Change, addrs.ParsePartialResourceInstanceStr)
+	if err != nil {
+		return nil, err
+	}
+
+	reason, err := DeferredReasonFromProto(dc.Deferred.Reason)
+	if err != nil {
+		return nil, err
+	}
+
+	return &plans.DeferredResourceInstanceChangeSrc{
+		DeferredReason: reason,
+		ChangeSrc:      change,
+	}, nil
+}
+
+func DeferredReasonFromProto(reason planproto.DeferredReason) (providers.DeferredReason, error) {
+	switch reason {
+	case planproto.DeferredReason_INSTANCE_COUNT_UNKNOWN:
+		return providers.DeferredReasonInstanceCountUnknown, nil
+	case planproto.DeferredReason_RESOURCE_CONFIG_UNKNOWN:
+		return providers.DeferredReasonResourceConfigUnknown, nil
+	case planproto.DeferredReason_PROVIDER_CONFIG_UNKNOWN:
+		return providers.DeferredReasonProviderConfigUnknown, nil
+	case planproto.DeferredReason_ABSENT_PREREQ:
+		return providers.DeferredReasonAbsentPrereq, nil
+	case planproto.DeferredReason_DEFERRED_PREREQ:
+		return providers.DeferredReasonDeferredPrereq, nil
+	default:
+		return providers.DeferredReasonInvalid, fmt.Errorf("invalid deferred reason %s", reason)
+	}
+}
+
 // writeTfplan serializes the given plan into the protobuf-based format used
 // for the "tfplan" portion of a plan file.
 func writeTfplan(plan *plans.Plan, w io.Writer) error {
@@ -482,19 +510,17 @@ func writeTfplan(plan *plans.Plan, w io.Writer) error {
 		CheckResults:    []*planproto.CheckResults{},
 		ResourceChanges: []*planproto.ResourceInstanceChange{},
 		ResourceDrift:   []*planproto.ResourceInstanceChange{},
+		DeferredChanges: []*planproto.DeferredResourceInstanceChange{},
 	}
 
+	rawPlan.Applyable = plan.Applyable
+	rawPlan.Complete = plan.Complete
 	rawPlan.Errored = plan.Errored
 
-	switch plan.UIMode {
-	case plans.NormalMode:
-		rawPlan.UiMode = planproto.Mode_NORMAL
-	case plans.DestroyMode:
-		rawPlan.UiMode = planproto.Mode_DESTROY
-	case plans.RefreshOnlyMode:
-		rawPlan.UiMode = planproto.Mode_REFRESH_ONLY
-	default:
-		return fmt.Errorf("plan has unsupported mode %s", plan.UIMode)
+	var err error
+	rawPlan.UiMode, err = planproto.NewMode(plan.UIMode)
+	if err != nil {
+		return err
 	}
 
 	for _, oc := range plan.Changes.Outputs {
@@ -522,61 +548,11 @@ func writeTfplan(plan *plans.Plan, w io.Writer) error {
 		})
 	}
 
-	if plan.Checks != nil {
-		for _, configElem := range plan.Checks.ConfigResults.Elems {
-			crs := configElem.Value
-			pcrs := &planproto.CheckResults{
-				ConfigAddr: configElem.Key.String(),
-			}
-			switch crs.Status {
-			case checks.StatusUnknown:
-				pcrs.Status = planproto.CheckResults_UNKNOWN
-			case checks.StatusPass:
-				pcrs.Status = planproto.CheckResults_PASS
-			case checks.StatusFail:
-				pcrs.Status = planproto.CheckResults_FAIL
-			case checks.StatusError:
-				pcrs.Status = planproto.CheckResults_ERROR
-			default:
-				return fmt.Errorf("checkable configuration %s has unsupported aggregate status %s", configElem.Key, crs.Status)
-			}
-			switch kind := configElem.Key.CheckableKind(); kind {
-			case addrs.CheckableResource:
-				pcrs.Kind = planproto.CheckResults_RESOURCE
-			case addrs.CheckableOutputValue:
-				pcrs.Kind = planproto.CheckResults_OUTPUT_VALUE
-			case addrs.CheckableCheck:
-				pcrs.Kind = planproto.CheckResults_CHECK
-			case addrs.CheckableInputVariable:
-				pcrs.Kind = planproto.CheckResults_INPUT_VARIABLE
-			default:
-				return fmt.Errorf("checkable configuration %s has unsupported object type kind %s", configElem.Key, kind)
-			}
-
-			for _, objectElem := range configElem.Value.ObjectResults.Elems {
-				cr := objectElem.Value
-				pcr := &planproto.CheckResults_ObjectResult{
-					ObjectAddr:      objectElem.Key.String(),
-					FailureMessages: objectElem.Value.FailureMessages,
-				}
-				switch cr.Status {
-				case checks.StatusUnknown:
-					pcr.Status = planproto.CheckResults_UNKNOWN
-				case checks.StatusPass:
-					pcr.Status = planproto.CheckResults_PASS
-				case checks.StatusFail:
-					pcr.Status = planproto.CheckResults_FAIL
-				case checks.StatusError:
-					pcr.Status = planproto.CheckResults_ERROR
-				default:
-					return fmt.Errorf("checkable object %s has unsupported status %s", objectElem.Key, crs.Status)
-				}
-				pcrs.Objects = append(pcrs.Objects, pcr)
-			}
-
-			rawPlan.CheckResults = append(rawPlan.CheckResults, pcrs)
-		}
+	checkResults, err := CheckResultsToPlanProto(plan.Checks)
+	if err != nil {
+		return fmt.Errorf("failed to encode check results: %s", err)
 	}
+	rawPlan.CheckResults = checkResults
 
 	for _, rc := range plan.Changes.Resources {
 		rawRC, err := resourceChangeToTfplan(rc)
@@ -594,6 +570,14 @@ func writeTfplan(plan *plans.Plan, w io.Writer) error {
 		rawPlan.ResourceDrift = append(rawPlan.ResourceDrift, rawRC)
 	}
 
+	for _, dc := range plan.DeferredResources {
+		rawDC, err := deferredChangeToTfplan(dc)
+		if err != nil {
+			return err
+		}
+		rawPlan.DeferredChanges = append(rawPlan.DeferredChanges, rawDC)
+	}
+
 	for _, ra := range plan.RelevantAttributes {
 		rawRA, err := resourceAttrToTfplan(ra)
 		if err != nil {
@@ -613,6 +597,21 @@ func writeTfplan(plan *plans.Plan, w io.Writer) error {
 	for name, val := range plan.VariableValues {
 		rawPlan.Variables[name] = valueToTfplan(val)
 	}
+	if plan.ApplyTimeVariables.Len() != 0 {
+		rawPlan.ApplyTimeVariables = make([]string, 0, plan.ApplyTimeVariables.Len())
+		for name := range plan.ApplyTimeVariables.All() {
+			rawPlan.ApplyTimeVariables = append(rawPlan.ApplyTimeVariables, name)
+		}
+	}
+
+	for _, hash := range plan.ProviderFunctionResults {
+		rawPlan.ProviderFunctionResults = append(rawPlan.ProviderFunctionResults,
+			&planproto.ProviderFunctionCallHash{
+				Key:    hash.Key,
+				Result: hash.Result,
+			},
+		)
+	}
 
 	if plan.Backend.Type == "" || plan.Backend.Config == nil {
 		// This suggests a bug in the code that created the plan, since it
@@ -675,6 +674,19 @@ func resourceAttrFromTfplan(ra *planproto.PlanResourceAttr) (globalref.ResourceA
 	return res, nil
 }
 
+// ResourceChangeToProto encodes an isolated resource instance change into
+// its representation as a protocol buffers message.
+//
+// This is used by the stackplan package, which includes planproto messages
+// in its own wire format while using a different overall container.
+func ResourceChangeToProto(change *plans.ResourceInstanceChangeSrc) (*planproto.ResourceInstanceChange, error) {
+	if change == nil {
+		// We assume this represents the absense of a change, then.
+		return nil, nil
+	}
+	return resourceChangeToTfplan(change)
+}
+
 func resourceChangeToTfplan(change *plans.ResourceInstanceChangeSrc) (*planproto.ResourceInstanceChange, error) {
 	ret := &planproto.ResourceInstanceChange{}
 
@@ -753,17 +765,45 @@ func resourceChangeToTfplan(change *plans.ResourceInstanceChangeSrc) (*planproto
 	return ret, nil
 }
 
+// ActionToProto translates from the "plans" package's representation of change
+// actions into the protobuf representation, or returns an error if the
+// given action is unrecognized.
+func ActionToProto(action plans.Action) (planproto.Action, error) {
+	switch action {
+	case plans.NoOp:
+		return planproto.Action_NOOP, nil
+	case plans.Create:
+		return planproto.Action_CREATE, nil
+	case plans.Read:
+		return planproto.Action_READ, nil
+	case plans.Update:
+		return planproto.Action_UPDATE, nil
+	case plans.Delete:
+		return planproto.Action_DELETE, nil
+	case plans.DeleteThenCreate:
+		return planproto.Action_DELETE_THEN_CREATE, nil
+	case plans.CreateThenDelete:
+		return planproto.Action_CREATE_THEN_DELETE, nil
+	case plans.Forget:
+		return planproto.Action_FORGET, nil
+	case plans.CreateThenForget:
+		return planproto.Action_CREATE_THEN_FORGET, nil
+	default:
+		return planproto.Action_NOOP, fmt.Errorf("invalid change action %s", action)
+	}
+}
+
 func changeToTfplan(change *plans.ChangeSrc) (*planproto.Change, error) {
 	ret := &planproto.Change{}
 
 	before := valueToTfplan(change.Before)
 	after := valueToTfplan(change.After)
 
-	beforeSensitivePaths, err := pathValueMarksToTfplan(change.BeforeValMarks)
+	beforeSensitivePaths, err := pathsToTfplan(change.BeforeSensitivePaths)
 	if err != nil {
 		return nil, err
 	}
-	afterSensitivePaths, err := pathValueMarksToTfplan(change.AfterValMarks)
+	afterSensitivePaths, err := pathsToTfplan(change.AfterSensitivePaths)
 	if err != nil {
 		return nil, err
 	}
@@ -772,33 +812,36 @@ func changeToTfplan(change *plans.ChangeSrc) (*planproto.Change, error) {
 
 	if change.Importing != nil {
 		ret.Importing = &planproto.Importing{
-			Id: change.Importing.ID,
+			Id:      change.Importing.ID,
+			Unknown: change.Importing.Unknown,
 		}
 
 	}
 	ret.GeneratedConfig = change.GeneratedConfig
 
-	switch change.Action {
-	case plans.NoOp:
-		ret.Action = planproto.Action_NOOP
+	ret.Action, err = ActionToProto(change.Action)
+	if err != nil {
+		return nil, err
+	}
+
+	switch ret.Action {
+	case planproto.Action_NOOP:
 		ret.Values = []*planproto.DynamicValue{before} // before and after should be identical
-	case plans.Create:
-		ret.Action = planproto.Action_CREATE
+	case planproto.Action_CREATE:
 		ret.Values = []*planproto.DynamicValue{after}
-	case plans.Read:
-		ret.Action = planproto.Action_READ
+	case planproto.Action_READ:
 		ret.Values = []*planproto.DynamicValue{before, after}
-	case plans.Update:
-		ret.Action = planproto.Action_UPDATE
+	case planproto.Action_UPDATE:
 		ret.Values = []*planproto.DynamicValue{before, after}
-	case plans.Delete:
-		ret.Action = planproto.Action_DELETE
+	case planproto.Action_DELETE:
 		ret.Values = []*planproto.DynamicValue{before}
-	case plans.DeleteThenCreate:
-		ret.Action = planproto.Action_DELETE_THEN_CREATE
+	case planproto.Action_DELETE_THEN_CREATE:
 		ret.Values = []*planproto.DynamicValue{before, after}
-	case plans.CreateThenDelete:
-		ret.Action = planproto.Action_CREATE_THEN_DELETE
+	case planproto.Action_CREATE_THEN_DELETE:
+		ret.Values = []*planproto.DynamicValue{before, after}
+	case planproto.Action_FORGET:
+		ret.Values = []*planproto.DynamicValue{before}
+	case planproto.Action_CREATE_THEN_FORGET:
 		ret.Values = []*planproto.DynamicValue{before, after}
 	default:
 		return nil, fmt.Errorf("invalid change action %s", change.Action)
@@ -808,43 +851,51 @@ func changeToTfplan(change *plans.ChangeSrc) (*planproto.Change, error) {
 }
 
 func valueToTfplan(val plans.DynamicValue) *planproto.DynamicValue {
-	if val == nil {
-		// protobuf can't represent nil, so we'll represent it as a
-		// DynamicValue that has no serializations at all.
-		return &planproto.DynamicValue{}
-	}
-	return &planproto.DynamicValue{
-		Msgpack: []byte(val),
-	}
+	return planproto.NewPlanDynamicValue(val)
 }
 
-func pathValueMarksFromTfplan(paths []*planproto.Path, marks cty.ValueMarks) ([]cty.PathValueMarks, error) {
-	ret := make([]cty.PathValueMarks, 0, len(paths))
+func pathsFromTfplan(paths []*planproto.Path) ([]cty.Path, error) {
+	if len(paths) == 0 {
+		return nil, nil
+	}
+	ret := make([]cty.Path, 0, len(paths))
 	for _, p := range paths {
 		path, err := pathFromTfplan(p)
 		if err != nil {
 			return nil, err
 		}
-		ret = append(ret, cty.PathValueMarks{
-			Path:  path,
-			Marks: marks,
-		})
-	}
-	return ret, nil
-}
-
-func pathValueMarksToTfplan(pvm []cty.PathValueMarks) ([]*planproto.Path, error) {
-	ret := make([]*planproto.Path, 0, len(pvm))
-	for _, p := range pvm {
-		path, err := pathToTfplan(p.Path)
-		if err != nil {
-			return nil, err
-		}
 		ret = append(ret, path)
 	}
 	return ret, nil
 }
 
+func pathsToTfplan(paths []cty.Path) ([]*planproto.Path, error) {
+	if len(paths) == 0 {
+		return nil, nil
+	}
+	ret := make([]*planproto.Path, 0, len(paths))
+	for _, p := range paths {
+		path, err := pathToTfplan(p)
+		if err != nil {
+			return nil, err
+		}
+		ret = append(ret, path)
+	}
+	return ret, nil
+}
+
+// PathFromProto decodes a path to a nested attribute into a cty.Path for
+// use in tracking marked values.
+//
+// This is used by the stackstate package, which uses planproto.Path messages
+// while using a different overall container.
+func PathFromProto(path *planproto.Path) (cty.Path, error) {
+	if path == nil {
+		return nil, nil
+	}
+	return pathFromTfplan(path)
+}
+
 func pathFromTfplan(path *planproto.Path) (cty.Path, error) {
 	ret := make([]cty.PathStep, 0, len(path.Steps))
 	for _, step := range path.Steps {
@@ -873,30 +924,220 @@ func pathFromTfplan(path *planproto.Path) (cty.Path, error) {
 }
 
 func pathToTfplan(path cty.Path) (*planproto.Path, error) {
-	steps := make([]*planproto.Path_Step, 0, len(path))
-	for _, step := range path {
-		switch s := step.(type) {
-		case cty.IndexStep:
-			value, err := plans.NewDynamicValue(s.Key, s.Key.Type())
-			if err != nil {
-				return nil, fmt.Errorf("Error encoding path step: %s", err)
-			}
-			steps = append(steps, &planproto.Path_Step{
-				Selector: &planproto.Path_Step_ElementKey{
-					ElementKey: valueToTfplan(value),
-				},
-			})
-		case cty.GetAttrStep:
-			steps = append(steps, &planproto.Path_Step{
-				Selector: &planproto.Path_Step_AttributeName{
-					AttributeName: s.Name,
-				},
-			})
-		default:
-			return nil, fmt.Errorf("Unsupported path step %#v (%t)", step, step)
-		}
+	return planproto.NewPath(path)
+}
+
+func deferredChangeToTfplan(dc *plans.DeferredResourceInstanceChangeSrc) (*planproto.DeferredResourceInstanceChange, error) {
+	change, err := resourceChangeToTfplan(dc.ChangeSrc)
+	if err != nil {
+		return nil, err
 	}
-	return &planproto.Path{
-		Steps: steps,
+
+	reason, err := DeferredReasonToProto(dc.DeferredReason)
+	if err != nil {
+		return nil, err
+	}
+
+	return &planproto.DeferredResourceInstanceChange{
+		Change: change,
+		Deferred: &planproto.Deferred{
+			Reason: reason,
+		},
 	}, nil
 }
+
+func DeferredReasonToProto(reason providers.DeferredReason) (planproto.DeferredReason, error) {
+	switch reason {
+	case providers.DeferredReasonInstanceCountUnknown:
+		return planproto.DeferredReason_INSTANCE_COUNT_UNKNOWN, nil
+	case providers.DeferredReasonResourceConfigUnknown:
+		return planproto.DeferredReason_RESOURCE_CONFIG_UNKNOWN, nil
+	case providers.DeferredReasonProviderConfigUnknown:
+		return planproto.DeferredReason_PROVIDER_CONFIG_UNKNOWN, nil
+	case providers.DeferredReasonAbsentPrereq:
+		return planproto.DeferredReason_ABSENT_PREREQ, nil
+	case providers.DeferredReasonDeferredPrereq:
+		return planproto.DeferredReason_DEFERRED_PREREQ, nil
+	default:
+		return planproto.DeferredReason_INVALID, fmt.Errorf("invalid deferred reason %s", reason)
+	}
+}
+
+// CheckResultsFromPlanProto decodes a slice of check results from their protobuf
+// representation into the "states" package's representation.
+//
+// It's used by the stackplan package, which includes an identical representation
+// of check results within a different overall container.
+func CheckResultsFromPlanProto(proto []*planproto.CheckResults) (*states.CheckResults, error) {
+	configResults := addrs.MakeMap[addrs.ConfigCheckable, *states.CheckResultAggregate]()
+
+	for _, rawCheckResults := range proto {
+		aggr := &states.CheckResultAggregate{}
+		switch rawCheckResults.Status {
+		case planproto.CheckResults_UNKNOWN:
+			aggr.Status = checks.StatusUnknown
+		case planproto.CheckResults_PASS:
+			aggr.Status = checks.StatusPass
+		case planproto.CheckResults_FAIL:
+			aggr.Status = checks.StatusFail
+		case planproto.CheckResults_ERROR:
+			aggr.Status = checks.StatusError
+		default:
+			return nil,
+				fmt.Errorf("aggregate check results for %s have unsupported status %#v",
+					rawCheckResults.ConfigAddr, rawCheckResults.Status)
+		}
+
+		var objKind addrs.CheckableKind
+		switch rawCheckResults.Kind {
+		case planproto.CheckResults_RESOURCE:
+			objKind = addrs.CheckableResource
+		case planproto.CheckResults_OUTPUT_VALUE:
+			objKind = addrs.CheckableOutputValue
+		case planproto.CheckResults_CHECK:
+			objKind = addrs.CheckableCheck
+		case planproto.CheckResults_INPUT_VARIABLE:
+			objKind = addrs.CheckableInputVariable
+		default:
+			return nil, fmt.Errorf("aggregate check results for %s have unsupported object kind %s",
+				rawCheckResults.ConfigAddr, objKind)
+		}
+
+		// Some trickiness here: we only have an address parser for
+		// addrs.Checkable and not for addrs.ConfigCheckable, but that's okay
+		// because once we have an addrs.Checkable we can always derive an
+		// addrs.ConfigCheckable from it, and a ConfigCheckable should always
+		// be the same syntax as a Checkable with no index information and
+		// thus we can reuse the same parser for both here.
+		configAddrProxy, diags := addrs.ParseCheckableStr(objKind, rawCheckResults.ConfigAddr)
+		if diags.HasErrors() {
+			return nil, diags.Err()
+		}
+		configAddr := configAddrProxy.ConfigCheckable()
+		if configAddr.String() != configAddrProxy.String() {
+			// This is how we catch if the config address included index
+			// information that would be allowed in a Checkable but not
+			// in a ConfigCheckable.
+			return nil, fmt.Errorf("invalid checkable config address %s", rawCheckResults.ConfigAddr)
+		}
+
+		aggr.ObjectResults = addrs.MakeMap[addrs.Checkable, *states.CheckResultObject]()
+		for _, rawCheckResult := range rawCheckResults.Objects {
+			objectAddr, diags := addrs.ParseCheckableStr(objKind, rawCheckResult.ObjectAddr)
+			if diags.HasErrors() {
+				return nil, diags.Err()
+			}
+			if !addrs.Equivalent(objectAddr.ConfigCheckable(), configAddr) {
+				return nil, fmt.Errorf("checkable object %s should not be grouped under %s", objectAddr, configAddr)
+			}
+
+			obj := &states.CheckResultObject{
+				FailureMessages: rawCheckResult.FailureMessages,
+			}
+			switch rawCheckResult.Status {
+			case planproto.CheckResults_UNKNOWN:
+				obj.Status = checks.StatusUnknown
+			case planproto.CheckResults_PASS:
+				obj.Status = checks.StatusPass
+			case planproto.CheckResults_FAIL:
+				obj.Status = checks.StatusFail
+			case planproto.CheckResults_ERROR:
+				obj.Status = checks.StatusError
+			default:
+				return nil, fmt.Errorf("object check results for %s has unsupported status %#v",
+					rawCheckResult.ObjectAddr, rawCheckResult.Status)
+			}
+
+			aggr.ObjectResults.Put(objectAddr, obj)
+		}
+
+		// If we ended up with no elements in the map then we'll just nil it,
+		// primarily just to make life easier for our round-trip tests.
+		if aggr.ObjectResults.Len() == 0 {
+			aggr.ObjectResults.Elems = nil
+		}
+
+		configResults.Put(configAddr, aggr)
+	}
+
+	// If we ended up with no elements in the map then we'll just nil it,
+	// primarily just to make life easier for our round-trip tests.
+	if configResults.Len() == 0 {
+		configResults.Elems = nil
+	}
+
+	return &states.CheckResults{
+		ConfigResults: configResults,
+	}, nil
+}
+
+// CheckResultsToPlanProto encodes a slice of check results from the "states"
+// package's representation into their protobuf representation.
+//
+// It's used by the stackplan package, which includes identical representation
+// of check results within a different overall container.
+func CheckResultsToPlanProto(checkResults *states.CheckResults) ([]*planproto.CheckResults, error) {
+	if checkResults != nil {
+		protoResults := make([]*planproto.CheckResults, 0)
+		for _, configElem := range checkResults.ConfigResults.Elems {
+			crs := configElem.Value
+			pcrs := &planproto.CheckResults{
+				ConfigAddr: configElem.Key.String(),
+			}
+			switch crs.Status {
+			case checks.StatusUnknown:
+				pcrs.Status = planproto.CheckResults_UNKNOWN
+			case checks.StatusPass:
+				pcrs.Status = planproto.CheckResults_PASS
+			case checks.StatusFail:
+				pcrs.Status = planproto.CheckResults_FAIL
+			case checks.StatusError:
+				pcrs.Status = planproto.CheckResults_ERROR
+			default:
+				return nil,
+					fmt.Errorf("checkable configuration %s has unsupported aggregate status %s", configElem.Key, crs.Status)
+			}
+			switch kind := configElem.Key.CheckableKind(); kind {
+			case addrs.CheckableResource:
+				pcrs.Kind = planproto.CheckResults_RESOURCE
+			case addrs.CheckableOutputValue:
+				pcrs.Kind = planproto.CheckResults_OUTPUT_VALUE
+			case addrs.CheckableCheck:
+				pcrs.Kind = planproto.CheckResults_CHECK
+			case addrs.CheckableInputVariable:
+				pcrs.Kind = planproto.CheckResults_INPUT_VARIABLE
+			default:
+				return nil,
+					fmt.Errorf("checkable configuration %s has unsupported object type kind %s", configElem.Key, kind)
+			}
+
+			for _, objectElem := range configElem.Value.ObjectResults.Elems {
+				cr := objectElem.Value
+				pcr := &planproto.CheckResults_ObjectResult{
+					ObjectAddr:      objectElem.Key.String(),
+					FailureMessages: objectElem.Value.FailureMessages,
+				}
+				switch cr.Status {
+				case checks.StatusUnknown:
+					pcr.Status = planproto.CheckResults_UNKNOWN
+				case checks.StatusPass:
+					pcr.Status = planproto.CheckResults_PASS
+				case checks.StatusFail:
+					pcr.Status = planproto.CheckResults_FAIL
+				case checks.StatusError:
+					pcr.Status = planproto.CheckResults_ERROR
+				default:
+					return nil,
+						fmt.Errorf("checkable object %s has unsupported status %s", objectElem.Key, crs.Status)
+				}
+				pcrs.Objects = append(pcrs.Objects, pcr)
+			}
+
+			protoResults = append(protoResults, pcrs)
+		}
+
+		return protoResults, nil
+	} else {
+		return nil, nil
+	}
+}
