@@ -12,9 +12,10 @@ import (
 
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/checks"
+	"github.com/hashicorp/terraform/internal/collections"
 	"github.com/hashicorp/terraform/internal/lang/globalref"
-	"github.com/hashicorp/terraform/internal/lang/marks"
 	"github.com/hashicorp/terraform/internal/plans"
+	"github.com/hashicorp/terraform/internal/providers"
 	"github.com/hashicorp/terraform/internal/states"
 )
 
@@ -22,12 +23,17 @@ func TestTFPlanRoundTrip(t *testing.T) {
 	objTy := cty.Object(map[string]cty.Type{
 		"id": cty.String,
 	})
+	applyTimeVariables := collections.NewSetCmp[string]()
+	applyTimeVariables.Add("bar")
 
 	plan := &plans.Plan{
+		Applyable: true,
+		Complete:  true,
 		VariableValues: map[string]plans.DynamicValue{
 			"foo": mustNewDynamicValueStr("foo value"),
 		},
-		Changes: &plans.Changes{
+		ApplyTimeVariables: applyTimeVariables,
+		Changes: &plans.ChangesSrc{
 			Outputs: []*plans.OutputChangeSrc{
 				{
 					Addr: addrs.OutputValue{Name: "bar"}.Absolute(addrs.RootModuleInstance),
@@ -87,11 +93,8 @@ func TestTFPlanRoundTrip(t *testing.T) {
 								cty.StringVal("honk"),
 							}),
 						}), objTy),
-						AfterValMarks: []cty.PathValueMarks{
-							{
-								Path:  cty.GetAttrPath("boop").IndexInt(1),
-								Marks: cty.NewValueMarks(marks.Sensitive),
-							},
+						AfterSensitivePaths: []cty.Path{
+							cty.GetAttrPath("boop").IndexInt(1),
 						},
 					},
 					RequiredReplace: cty.NewPathSet(
@@ -182,11 +185,63 @@ func TestTFPlanRoundTrip(t *testing.T) {
 							cty.StringVal("bonk"),
 						}),
 					}), objTy),
-					AfterValMarks: []cty.PathValueMarks{
-						{
-							Path:  cty.GetAttrPath("boop").IndexInt(1),
-							Marks: cty.NewValueMarks(marks.Sensitive),
+					AfterSensitivePaths: []cty.Path{
+						cty.GetAttrPath("boop").IndexInt(1),
+					},
+				},
+			},
+		},
+		DeferredResources: []*plans.DeferredResourceInstanceChangeSrc{
+			{
+				DeferredReason: providers.DeferredReasonInstanceCountUnknown,
+				ChangeSrc: &plans.ResourceInstanceChangeSrc{
+					Addr: addrs.Resource{
+						Mode: addrs.ManagedResourceMode,
+						Type: "test_thing",
+						Name: "woot",
+					}.Instance(addrs.WildcardKey).Absolute(addrs.RootModuleInstance),
+					ProviderAddr: addrs.AbsProviderConfig{
+						Provider: addrs.NewDefaultProvider("test"),
+						Module:   addrs.RootModule,
+					},
+					ChangeSrc: plans.ChangeSrc{
+						Action: plans.Create,
+						After: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
+							"id": cty.UnknownVal(cty.String),
+							"boop": cty.ListVal([]cty.Value{
+								cty.StringVal("beep"),
+								cty.StringVal("bonk"),
+							}),
+						}), objTy),
+					},
+				},
+			},
+			{
+				DeferredReason: providers.DeferredReasonInstanceCountUnknown,
+				ChangeSrc: &plans.ResourceInstanceChangeSrc{
+					Addr: addrs.Resource{
+						Mode: addrs.ManagedResourceMode,
+						Type: "test_thing",
+						Name: "woot",
+					}.Instance(addrs.WildcardKey).Absolute(addrs.ModuleInstance{
+						addrs.ModuleInstanceStep{
+							Name:        "mod",
+							InstanceKey: addrs.WildcardKey,
 						},
+					}),
+					ProviderAddr: addrs.AbsProviderConfig{
+						Provider: addrs.NewDefaultProvider("test"),
+						Module:   addrs.RootModule,
+					},
+					ChangeSrc: plans.ChangeSrc{
+						Action: plans.Create,
+						After: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
+							"id": cty.UnknownVal(cty.String),
+							"boop": cty.ListVal([]cty.Value{
+								cty.StringVal("beep"),
+								cty.StringVal("bonk"),
+							}),
+						}), objTy),
 					},
 				},
 			},
@@ -327,7 +382,7 @@ func TestTFPlanRoundTripDestroy(t *testing.T) {
 	})
 
 	plan := &plans.Plan{
-		Changes: &plans.Changes{
+		Changes: &plans.ChangesSrc{
 			Outputs: []*plans.OutputChangeSrc{
 				{
 					Addr: addrs.OutputValue{Name: "bar"}.Absolute(addrs.RootModuleInstance),
