@@ -7,14 +7,16 @@ import (
 	"bytes"
 	"fmt"
 	"testing"
+	"time"
 
-	"github.com/golang/mock/gomock"
 	"github.com/google/go-cmp/cmp"
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs/hcl2shim"
 	"github.com/hashicorp/terraform/internal/providers"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 	"github.com/zclconf/go-cty/cty"
+	"go.uber.org/mock/gomock"
+	"google.golang.org/protobuf/types/known/timestamppb"
 
 	mockproto "github.com/hashicorp/terraform/internal/plugin/mock_proto"
 	proto "github.com/hashicorp/terraform/internal/tfplugin5"
@@ -93,6 +95,19 @@ func providerProtoSchema() *proto.GetProviderSchema_Response {
 				},
 			},
 		},
+		EphemeralResourceSchemas: map[string]*proto.Schema{
+			"ephemeral": &proto.Schema{
+				Block: &proto.Schema_Block{
+					Attributes: []*proto.Schema_Attribute{
+						{
+							Name:     "attr",
+							Type:     []byte(`"string"`),
+							Computed: true,
+						},
+					},
+				},
+			},
+		},
 		ServerCapabilities: &proto.ServerCapabilities{
 			GetProviderSchemaOptional: true,
 		},
@@ -367,6 +382,41 @@ func TestGRPCProvider_ReadResource(t *testing.T) {
 	}
 }
 
+func TestGRPCProvider_ReadResource_deferred(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	client.EXPECT().ReadResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.ReadResource_Response{
+		NewState: &proto.DynamicValue{
+			Msgpack: []byte("\x81\xa4attr\xa3bar"),
+		},
+		Deferred: &proto.Deferred{
+			Reason: proto.Deferred_ABSENT_PREREQ,
+		},
+	}, nil)
+
+	resp := p.ReadResource(providers.ReadResourceRequest{
+		TypeName: "resource",
+		PriorState: cty.ObjectVal(map[string]cty.Value{
+			"attr": cty.StringVal("foo"),
+		}),
+	})
+
+	checkDiags(t, resp.Diagnostics)
+
+	expectedDeferred := &providers.Deferred{
+		Reason: providers.DeferredReasonAbsentPrereq,
+	}
+	if !cmp.Equal(expectedDeferred, resp.Deferred, typeComparer, valueComparer, equateEmpty) {
+		t.Fatal(cmp.Diff(expectedDeferred, resp.Deferred, typeComparer, valueComparer, equateEmpty))
+	}
+}
+
 func TestGRPCProvider_ReadResourceJSON(t *testing.T) {
 	client := mockProviderClient(t)
 	p := &GRPCProvider{
@@ -738,6 +788,82 @@ func TestGRPCProvider_ImportResourceStateJSON(t *testing.T) {
 	}
 }
 
+func TestGRPCProvider_MoveResourceState(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	expectedTargetPrivate := []byte(`{"target": "private"}`)
+	expectedTargetState := cty.ObjectVal(map[string]cty.Value{
+		"attr": cty.StringVal("bar"),
+	})
+
+	client.EXPECT().MoveResourceState(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.MoveResourceState_Response{
+		TargetState: &proto.DynamicValue{
+			Msgpack: []byte("\x81\xa4attr\xa3bar"),
+		},
+		TargetPrivate: expectedTargetPrivate,
+	}, nil)
+
+	resp := p.MoveResourceState(providers.MoveResourceStateRequest{
+		SourcePrivate:   []byte(`{"source": "private"}`),
+		SourceStateJSON: []byte(`{"source_attr":"bar"}`),
+		TargetTypeName:  "resource",
+	})
+
+	checkDiags(t, resp.Diagnostics)
+
+	if !cmp.Equal(expectedTargetPrivate, resp.TargetPrivate, typeComparer, valueComparer, equateEmpty) {
+		t.Fatal(cmp.Diff(expectedTargetPrivate, resp.TargetPrivate, typeComparer, valueComparer, equateEmpty))
+	}
+
+	if !cmp.Equal(expectedTargetState, resp.TargetState, typeComparer, valueComparer, equateEmpty) {
+		t.Fatal(cmp.Diff(expectedTargetState, resp.TargetState, typeComparer, valueComparer, equateEmpty))
+	}
+}
+
+func TestGRPCProvider_MoveResourceStateJSON(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	expectedTargetPrivate := []byte(`{"target": "private"}`)
+	expectedTargetState := cty.ObjectVal(map[string]cty.Value{
+		"attr": cty.StringVal("bar"),
+	})
+
+	client.EXPECT().MoveResourceState(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.MoveResourceState_Response{
+		TargetState: &proto.DynamicValue{
+			Json: []byte(`{"attr":"bar"}`),
+		},
+		TargetPrivate: expectedTargetPrivate,
+	}, nil)
+
+	resp := p.MoveResourceState(providers.MoveResourceStateRequest{
+		SourcePrivate:   []byte(`{"source": "private"}`),
+		SourceStateJSON: []byte(`{"source_attr":"bar"}`),
+		TargetTypeName:  "resource",
+	})
+
+	checkDiags(t, resp.Diagnostics)
+
+	if !cmp.Equal(expectedTargetPrivate, resp.TargetPrivate, typeComparer, valueComparer, equateEmpty) {
+		t.Fatal(cmp.Diff(expectedTargetPrivate, resp.TargetPrivate, typeComparer, valueComparer, equateEmpty))
+	}
+
+	if !cmp.Equal(expectedTargetState, resp.TargetState, typeComparer, valueComparer, equateEmpty) {
+		t.Fatal(cmp.Diff(expectedTargetState, resp.TargetState, typeComparer, valueComparer, equateEmpty))
+	}
+}
+
 func TestGRPCProvider_ReadDataSource(t *testing.T) {
 	client := mockProviderClient(t)
 	p := &GRPCProvider{
@@ -803,3 +929,95 @@ func TestGRPCProvider_ReadDataSourceJSON(t *testing.T) {
 		t.Fatal(cmp.Diff(expected, resp.State, typeComparer, valueComparer, equateEmpty))
 	}
 }
+
+func TestGRPCProvider_openEphemeralResource(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	client.EXPECT().OpenEphemeralResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.OpenEphemeralResource_Response{
+		Result: &proto.DynamicValue{
+			Msgpack: []byte("\x81\xa4attr\xa3bar"),
+		},
+		RenewAt: timestamppb.New(time.Now().Add(time.Second)),
+		Private: []byte("private data"),
+	}, nil)
+
+	resp := p.OpenEphemeralResource(providers.OpenEphemeralResourceRequest{
+		TypeName: "ephemeral",
+		Config: cty.ObjectVal(map[string]cty.Value{
+			"attr": cty.NullVal(cty.String),
+		}),
+	})
+
+	checkDiags(t, resp.Diagnostics)
+
+	expected := cty.ObjectVal(map[string]cty.Value{
+		"attr": cty.StringVal("bar"),
+	})
+
+	if !cmp.Equal(expected, resp.Result, typeComparer, valueComparer, equateEmpty) {
+		t.Fatal(cmp.Diff(expected, resp.Result, typeComparer, valueComparer, equateEmpty))
+	}
+
+	if !resp.RenewAt.After(time.Now()) {
+		t.Fatal("invalid RenewAt:", resp.RenewAt)
+	}
+
+	if !bytes.Equal(resp.Private, []byte("private data")) {
+		t.Fatalf("invalid private data: %q", resp.Private)
+	}
+}
+
+func TestGRPCProvider_renewEphemeralResource(t *testing.T) {
+	client := mockproto.NewMockProviderClient(gomock.NewController(t))
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	client.EXPECT().RenewEphemeralResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.RenewEphemeralResource_Response{
+		RenewAt: timestamppb.New(time.Now().Add(time.Second)),
+		Private: []byte("private data"),
+	}, nil)
+
+	resp := p.RenewEphemeralResource(providers.RenewEphemeralResourceRequest{
+		TypeName: "ephemeral",
+		Private:  []byte("private data"),
+	})
+
+	checkDiags(t, resp.Diagnostics)
+
+	if !resp.RenewAt.After(time.Now()) {
+		t.Fatal("invalid RenewAt:", resp.RenewAt)
+	}
+
+	if !bytes.Equal(resp.Private, []byte("private data")) {
+		t.Fatalf("invalid private data: %q", resp.Private)
+	}
+}
+
+func TestGRPCProvider_closeEphemeralResource(t *testing.T) {
+	client := mockproto.NewMockProviderClient(gomock.NewController(t))
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	client.EXPECT().CloseEphemeralResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.CloseEphemeralResource_Response{}, nil)
+
+	resp := p.CloseEphemeralResource(providers.CloseEphemeralResourceRequest{
+		TypeName: "ephemeral",
+		Private:  []byte("private data"),
+	})
+
+	checkDiags(t, resp.Diagnostics)
+}
