@@ -9,6 +9,7 @@ import (
 
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs/hcl2shim"
+	"github.com/hashicorp/terraform/internal/lang/marks"
 )
 
 // ResourceInstanceObjectSrc is a not-fully-decoded version of
@@ -54,7 +55,7 @@ type ResourceInstanceObjectSrc struct {
 
 	// AttrSensitivePaths is an array of paths to mark as sensitive coming out of
 	// state, or to save as sensitive paths when saving state
-	AttrSensitivePaths []cty.PathValueMarks
+	AttrSensitivePaths []cty.Path
 
 	// These fields all correspond to the fields of the same name on
 	// ResourceInstanceObject.
@@ -62,6 +63,9 @@ type ResourceInstanceObjectSrc struct {
 	Status              ObjectStatus
 	Dependencies        []addrs.ConfigResource
 	CreateBeforeDestroy bool
+
+	// decodeValueCache stored the decoded value for repeated decodings.
+	decodeValueCache cty.Value
 }
 
 // Decode unmarshals the raw representation of the object attributes. Pass the
@@ -77,18 +81,21 @@ type ResourceInstanceObjectSrc struct {
 func (os *ResourceInstanceObjectSrc) Decode(ty cty.Type) (*ResourceInstanceObject, error) {
 	var val cty.Value
 	var err error
-	if os.AttrsFlat != nil {
+
+	switch {
+	case os.decodeValueCache != cty.NilVal:
+		val = os.decodeValueCache
+
+	case os.AttrsFlat != nil:
 		// Legacy mode. We'll do our best to unpick this from the flatmap.
 		val, err = hcl2shim.HCL2ValueFromFlatmap(os.AttrsFlat, ty)
 		if err != nil {
 			return nil, err
 		}
-	} else {
+
+	default:
 		val, err = ctyjson.Unmarshal(os.AttrsJSON, ty)
-		// Mark the value with paths if applicable
-		if os.AttrSensitivePaths != nil {
-			val = val.MarkWithPaths(os.AttrSensitivePaths)
-		}
+		val = marks.MarkPaths(val, marks.Sensitive, os.AttrSensitivePaths)
 		if err != nil {
 			return nil, err
 		}
