@@ -84,3 +84,27 @@ func TestResourceInstanceObject_encode(t *testing.T) {
 		}
 	}
 }
+
+func TestResourceInstanceObject_encodeInvalidMarks(t *testing.T) {
+	value := cty.ObjectVal(map[string]cty.Value{
+		// State only supports a subset of marks that we know how to persist
+		// between plan/apply rounds. All values with other marks must be
+		// replaced with unmarked placeholders before attempting to store the
+		// value in the state.
+		"foo": cty.True.Mark("unsupported"),
+	})
+
+	obj := &ResourceInstanceObject{
+		Value:  value,
+		Status: ObjectReady,
+	}
+	_, err := obj.Encode(value.Type(), 0)
+	if err == nil {
+		t.Fatalf("unexpected success; want error")
+	}
+	got := err.Error()
+	want := `.foo: cannot serialize value marked as cty.NewValueMarks("unsupported") for inclusion in a state snapshot (this is a bug in Terraform)`
+	if got != want {
+		t.Errorf("wrong error\ngot:  %s\nwant: %s", got, want)
+	}
+}
