@@ -31,9 +31,14 @@ func (s *State) DeepCopy() *State {
 	for k, m := range s.Modules {
 		modules[k] = m.DeepCopy()
 	}
+	outputValues := make(map[string]*OutputValue, len(s.RootOutputValues))
+	for k, v := range s.RootOutputValues {
+		outputValues[k] = v.DeepCopy()
+	}
 	return &State{
-		Modules:      modules,
-		CheckResults: s.CheckResults.DeepCopy(),
+		Modules:          modules,
+		RootOutputValues: outputValues,
+		CheckResults:     s.CheckResults.DeepCopy(),
 	}
 }
 
@@ -54,21 +59,10 @@ func (ms *Module) DeepCopy() *Module {
 	for k, r := range ms.Resources {
 		resources[k] = r.DeepCopy()
 	}
-	outputValues := make(map[string]*OutputValue, len(ms.OutputValues))
-	for k, v := range ms.OutputValues {
-		outputValues[k] = v.DeepCopy()
-	}
-	localValues := make(map[string]cty.Value, len(ms.LocalValues))
-	for k, v := range ms.LocalValues {
-		// cty.Value is immutable, so we don't need to copy these.
-		localValues[k] = v
-	}
 
 	return &Module{
-		Addr:         ms.Addr, // technically mutable, but immutable by convention
-		Resources:    resources,
-		OutputValues: outputValues,
-		LocalValues:  localValues,
+		Addr:      ms.Addr, // technically mutable, but immutable by convention
+		Resources: resources,
 	}
 }
 
@@ -148,10 +142,10 @@ func (os *ResourceInstanceObjectSrc) DeepCopy() *ResourceInstanceObjectSrc {
 		copy(attrsJSON, os.AttrsJSON)
 	}
 
-	var attrPaths []cty.PathValueMarks
+	var sensitiveAttrPaths []cty.Path
 	if os.AttrSensitivePaths != nil {
-		attrPaths = make([]cty.PathValueMarks, len(os.AttrSensitivePaths))
-		copy(attrPaths, os.AttrSensitivePaths)
+		sensitiveAttrPaths = make([]cty.Path, len(os.AttrSensitivePaths))
+		copy(sensitiveAttrPaths, os.AttrSensitivePaths)
 	}
 
 	var private []byte
@@ -174,9 +168,10 @@ func (os *ResourceInstanceObjectSrc) DeepCopy() *ResourceInstanceObjectSrc {
 		Private:             private,
 		AttrsFlat:           attrsFlat,
 		AttrsJSON:           attrsJSON,
-		AttrSensitivePaths:  attrPaths,
+		AttrSensitivePaths:  sensitiveAttrPaths,
 		Dependencies:        dependencies,
 		CreateBeforeDestroy: os.CreateBeforeDestroy,
+		decodeValueCache:    os.decodeValueCache,
 	}
 }
 
