@@ -15,7 +15,6 @@ import (
 
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/checks"
-	"github.com/hashicorp/terraform/internal/lang/marks"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
@@ -164,15 +163,7 @@ func prepareStateV4(sV4 *stateV4) (*File, tfdiags.Diagnostics) {
 				if pathsDiags.HasErrors() {
 					continue
 				}
-
-				var pvm []cty.PathValueMarks
-				for _, path := range paths {
-					pvm = append(pvm, cty.PathValueMarks{
-						Path:  path,
-						Marks: cty.NewValueMarks(marks.Sensitive),
-					})
-				}
-				obj.AttrSensitivePaths = pvm
+				obj.AttrSensitivePaths = paths
 			}
 
 			{
@@ -257,10 +248,9 @@ func prepareStateV4(sV4 *stateV4) (*File, tfdiags.Diagnostics) {
 	}
 
 	// The root module is special in that we persist its attributes and thus
-	// need to reload them now. (For descendent modules we just re-calculate
+	// need to reload them now. (For descendant modules we just re-calculate
 	// them based on the latest configuration on each run.)
 	{
-		rootModule := state.RootModule()
 		for name, fos := range sV4.RootOutputs {
 			os := &states.OutputValue{
 				Addr: addrs.AbsOutputValue{
@@ -292,7 +282,7 @@ func prepareStateV4(sV4 *stateV4) (*File, tfdiags.Diagnostics) {
 			}
 
 			os.Value = val
-			rootModule.OutputValues[name] = os
+			state.RootOutputValues[name] = os
 		}
 	}
 
@@ -338,7 +328,7 @@ func writeStateV4(file *File, w io.Writer) tfdiags.Diagnostics {
 		Resources:        []resourceStateV4{},
 	}
 
-	for name, os := range file.State.RootModule().OutputValues {
+	for name, os := range file.State.RootOutputValues {
 		src, err := ctyjson.Marshal(os.Value, os.Value.Type())
 		if err != nil {
 			diags = diags.Append(tfdiags.Sourceless(
@@ -489,14 +479,8 @@ func appendInstanceObjectStateV4(rs *states.Resource, is *states.ResourceInstanc
 		}
 	}
 
-	// Extract paths from path value marks
-	var paths []cty.Path
-	for _, vm := range obj.AttrSensitivePaths {
-		paths = append(paths, vm.Path)
-	}
-
 	// Marshal paths to JSON
-	attributeSensitivePaths, pathsDiags := marshalPaths(paths)
+	attributeSensitivePaths, pathsDiags := marshalPaths(obj.AttrSensitivePaths)
 	diags = diags.Append(pathsDiags)
 
 	return append(isV4s, instanceObjectStateV4{
@@ -589,19 +573,6 @@ func encodeCheckResultsV4(in *states.CheckResults) []checkResultsV4 {
 	ret := make([]checkResultsV4, 0, in.ConfigResults.Len())
 
 	for _, configElem := range in.ConfigResults.Elems {
-
-		if configElem.Key.CheckableKind() == addrs.CheckableInputVariable {
-			// For the remainder of the v1.6 series we should not output
-			// checkable input variables into the state file.
-			//
-			// This is to work around the issues in earlier releases that was
-			// fixed by https://github.com/hashicorp/terraform/pull/33813.
-			//
-			// The aim here is to give people more time to upgrade before we
-			// make the latest version incompatible with earlier versions.
-			continue
-		}
-
 		configResultsOut := checkResultsV4{
 			ObjectKind: encodeCheckableObjectKindV4(configElem.Key.CheckableKind()),
 			ConfigAddr: configElem.Key.String(),
@@ -839,6 +810,9 @@ func unmarshalPaths(buf []byte) ([]cty.Path, tfdiags.Diagnostics) {
 		))
 	}
 
+	if len(jsonPaths) == 0 {
+		return nil, diags
+	}
 	paths := make([]cty.Path, 0, len(jsonPaths))
 
 unmarshalOuter:
