@@ -13,6 +13,7 @@ import (
 	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/providers"
+	testing_provider "github.com/hashicorp/terraform/internal/providers/testing"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
@@ -38,8 +39,8 @@ func TestContextChecks(t *testing.T) {
 		applyError   string
 		applyWarning string
 		state        *states.State
-		provider     *MockProvider
-		providerHook func(*MockProvider)
+		provider     *testing_provider.MockProvider
+		providerHook func(*testing_provider.MockProvider)
 	}{
 		"passing": {
 			configs: map[string]string{
@@ -66,7 +67,7 @@ check "passing" {
 					status: checks.StatusPass,
 				},
 			},
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -120,7 +121,7 @@ check "failing" {
 				},
 			},
 			applyWarning: "Check block assertion failed: negative number",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -179,7 +180,7 @@ check "failing" {
 				},
 			},
 			applyWarning: "Check block assertion failed: positive number",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -249,7 +250,7 @@ check "nested_data_block" {
 				},
 			},
 			applyWarning: "Check block assertion failed: negative number",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -273,7 +274,7 @@ check "nested_data_block" {
 					}
 				},
 			},
-			providerHook: func(provider *MockProvider) {
+			providerHook: func(provider *testing_provider.MockProvider) {
 				provider.ReadDataSourceFn = func(request providers.ReadDataSourceRequest) providers.ReadDataSourceResponse {
 					// The data returned by the data sources are changing
 					// between the plan and apply stage. The nested data block
@@ -317,7 +318,7 @@ check "resource_block" {
 					status: checks.StatusPass,
 				},
 			},
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					ResourceTypes: map[string]providers.Schema{
@@ -415,7 +416,7 @@ check "error" {
 				},
 			},
 			applyWarning: "data source read failed: something bad happened and the provider couldn't read the data source",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -468,7 +469,7 @@ check "error" {
 						AttrsJSON: []byte(`{"number": -1}`),
 					},
 					addrs.AbsProviderConfig{
-						Provider: addrs.NewDefaultProvider("test"),
+						Provider: addrs.NewDefaultProvider("checks"),
 						Module:   addrs.RootModule,
 					})
 			}),
@@ -490,7 +491,7 @@ check "error" {
 				},
 			},
 			applyWarning: "data source read failed: something bad happened and the provider couldn't read the data source",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					ResourceTypes: map[string]providers.Schema{
@@ -568,7 +569,7 @@ check "passing" {
 						AttrsJSON: []byte(`{"number": -1}`),
 					},
 					addrs.AbsProviderConfig{
-						Provider: addrs.NewDefaultProvider("test"),
+						Provider: addrs.NewDefaultProvider("checks"),
 						Module:   addrs.RootModule,
 					})
 			}),
@@ -582,7 +583,7 @@ check "passing" {
 					status: checks.StatusPass,
 				},
 			},
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					ResourceTypes: map[string]providers.Schema{
@@ -630,7 +631,7 @@ check "error" {
 `,
 			},
 			planError: "data source read failed: something bad happened and the provider couldn't read the data source",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -673,7 +674,7 @@ check "error" {
 `,
 			},
 			planError: "Reference to scoped resource: The referenced data resource \"checks_object\" \"nested_data_block\" is not available from this context.",
-			provider: &MockProvider{
+			provider: &testing_provider.MockProvider{
 				Meta: "checks",
 				GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
 					DataSources: map[string]providers.Schema{
@@ -735,7 +736,7 @@ check "error" {
 				test.providerHook(test.provider)
 			}
 
-			state, diags := ctx.Apply(plan, configs)
+			state, diags := ctx.Apply(plan, configs, nil)
 			if validateCheckDiagnostics(t, "apply", test.applyWarning, test.applyError, diags) {
 				return
 			}
@@ -792,7 +793,7 @@ check "check_should_not_panic" {
 	}), DefaultPlanOpts)
 	assertNoErrors(t, diags)
 
-	_, diags = ctx.Apply(plan, m)
+	_, diags = ctx.Apply(plan, m, nil)
 	assertNoErrors(t, diags)
 }
 
