@@ -6,14 +6,18 @@ package terraform
 import (
 	"errors"
 	"fmt"
+	"path/filepath"
 	"strings"
 	"testing"
 
+	"github.com/google/go-cmp/cmp"
 	"github.com/zclconf/go-cty/cty"
 
+	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/providers"
+	testing_provider "github.com/hashicorp/terraform/internal/providers/testing"
 	"github.com/hashicorp/terraform/internal/provisioners"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
@@ -21,7 +25,7 @@ import (
 
 func TestContext2Validate_badCount(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{},
@@ -36,7 +40,7 @@ func TestContext2Validate_badCount(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -44,7 +48,7 @@ func TestContext2Validate_badCount(t *testing.T) {
 
 func TestContext2Validate_badResource_reference(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{},
@@ -59,7 +63,7 @@ func TestContext2Validate_badResource_reference(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -67,7 +71,7 @@ func TestContext2Validate_badResource_reference(t *testing.T) {
 
 func TestContext2Validate_badVar(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -85,7 +89,7 @@ func TestContext2Validate_badVar(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -162,7 +166,7 @@ func TestContext2Validate_computedVar(t *testing.T) {
 		return
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -202,7 +206,7 @@ func TestContext2Validate_computedInFunction(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -240,7 +244,7 @@ func TestContext2Validate_countComputed(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -264,7 +268,7 @@ func TestContext2Validate_countNegative(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -290,7 +294,7 @@ func TestContext2Validate_countVariable(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -344,7 +348,7 @@ func TestContext2Validate_moduleBadOutput(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -370,7 +374,7 @@ func TestContext2Validate_moduleGood(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -399,7 +403,7 @@ func TestContext2Validate_moduleBadResource(t *testing.T) {
 		Diagnostics: tfdiags.Diagnostics{}.Append(fmt.Errorf("bad")),
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -426,7 +430,7 @@ func TestContext2Validate_moduleDepsShouldNotCycle(t *testing.T) {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -467,7 +471,7 @@ func TestContext2Validate_moduleProviderVar(t *testing.T) {
 		return
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -508,7 +512,7 @@ func TestContext2Validate_moduleProviderInheritUnused(t *testing.T) {
 		return
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -547,7 +551,7 @@ func TestContext2Validate_orphans(t *testing.T) {
 		}
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -583,7 +587,7 @@ func TestContext2Validate_providerConfig_bad(t *testing.T) {
 		Diagnostics: tfdiags.Diagnostics{}.Append(fmt.Errorf("bad")),
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if len(diags) != 1 {
 		t.Fatalf("wrong number of diagnostics %d; want %d", len(diags), 1)
 	}
@@ -622,7 +626,7 @@ func TestContext2Validate_providerConfig_skippedEmpty(t *testing.T) {
 		Diagnostics: tfdiags.Diagnostics{}.Append(fmt.Errorf("should not be called")),
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -654,7 +658,7 @@ func TestContext2Validate_providerConfig_good(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -689,7 +693,7 @@ func TestContext2Validate_requiredProviderConfig(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -725,7 +729,7 @@ func TestContext2Validate_provisionerConfig_bad(t *testing.T) {
 		Diagnostics: tfdiags.Diagnostics{}.Append(fmt.Errorf("bad")),
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -757,7 +761,7 @@ func TestContext2Validate_badResourceConnection(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	t.Log(diags.Err())
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
@@ -790,7 +794,7 @@ func TestContext2Validate_badProvisionerConnection(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	t.Log(diags.Err())
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
@@ -839,7 +843,7 @@ func TestContext2Validate_provisionerConfig_good(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -905,7 +909,7 @@ func TestContext2Validate_resourceConfig_bad(t *testing.T) {
 		Diagnostics: tfdiags.Diagnostics{}.Append(fmt.Errorf("bad")),
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -931,7 +935,7 @@ func TestContext2Validate_resourceConfig_good(t *testing.T) {
 		},
 	})
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -969,7 +973,7 @@ func TestContext2Validate_tainted(t *testing.T) {
 		}
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -1006,7 +1010,7 @@ func TestContext2Validate_targetedDestroy(t *testing.T) {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -1038,7 +1042,7 @@ func TestContext2Validate_varRefUnknown(t *testing.T) {
 		return providers.ValidateResourceConfigResponse{}
 	}
 
-	c.Validate(m)
+	c.Validate(m, nil)
 
 	// Input variables are always unknown during the validate walk, because
 	// we're checking for validity of all possible input values. Validity
@@ -1074,7 +1078,7 @@ func TestContext2Validate_interpolateVar(t *testing.T) {
 		UIInput: input,
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -1106,7 +1110,7 @@ func TestContext2Validate_interpolateComputedModuleVarDef(t *testing.T) {
 		UIInput: input,
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -1126,7 +1130,7 @@ func TestContext2Validate_interpolateMap(t *testing.T) {
 		UIInput: input,
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -1176,7 +1180,7 @@ resource "aws_instance" "foo" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
@@ -1207,7 +1211,7 @@ output "out" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1243,7 +1247,7 @@ resource "aws_instance" "foo" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1278,7 +1282,7 @@ output "root" {
 
 	ctx := testContext2(t, &ContextOpts{})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
@@ -1301,7 +1305,7 @@ output "out" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1331,7 +1335,7 @@ output "out" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1361,7 +1365,7 @@ output "out" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1390,7 +1394,7 @@ resource "test_instance" "bar" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1422,7 +1426,7 @@ resource "test_instance" "bar" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1446,7 +1450,7 @@ func TestContext2Validate_variableCustomValidationsFail(t *testing.T) {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1480,7 +1484,7 @@ variable "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error\ngot: %s", diags.Err().Error())
 	}
@@ -1541,7 +1545,7 @@ resource "aws_instance" "foo" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -1568,7 +1572,7 @@ resource "aws_instance" "foo" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1598,7 +1602,7 @@ resource "aws_instance" "foo" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1679,7 +1683,7 @@ output "out" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -1706,7 +1710,7 @@ output "out" {
 `,
 	})
 
-	diags := testContext2(t, &ContextOpts{}).Validate(m)
+	diags := testContext2(t, &ContextOpts{}).Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1744,7 +1748,7 @@ output "out" {
 `,
 	})
 
-	diags := testContext2(t, &ContextOpts{}).Validate(m)
+	diags := testContext2(t, &ContextOpts{}).Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatal("succeeded; want errors")
 	}
@@ -1792,7 +1796,7 @@ resource "test_instance" "a" {
 			addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
 		},
 	})
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.Err())
 	}
@@ -1842,7 +1846,7 @@ func TestContext2Validate_sensitiveProvisionerConfig(t *testing.T) {
 		return pr.ValidateProvisionerConfigResponse
 	}
 
-	diags := c.Validate(m)
+	diags := c.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatalf("unexpected error: %s", diags.Err())
 	}
@@ -1852,8 +1856,8 @@ func TestContext2Validate_sensitiveProvisionerConfig(t *testing.T) {
 }
 
 func TestContext2Plan_validateMinMaxDynamicBlock(t *testing.T) {
-	p := new(MockProvider)
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p := new(testing_provider.MockProvider)
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"test_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -1936,7 +1940,7 @@ resource "test_instance" "c" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2003,15 +2007,15 @@ resource "test_object" "t" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
 }
 
 func TestContext2Plan_lookupMismatchedObjectTypes(t *testing.T) {
-	p := new(MockProvider)
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p := new(testing_provider.MockProvider)
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"test_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2060,7 +2064,7 @@ output "out" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2093,7 +2097,7 @@ func TestContext2Validate_nonNullableVariableDefaultValidation(t *testing.T) {
 
 	ctx := testContext2(t, &ContextOpts{})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2101,7 +2105,7 @@ func TestContext2Validate_nonNullableVariableDefaultValidation(t *testing.T) {
 
 func TestContext2Validate_precondition_good(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2136,7 +2140,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2144,7 +2148,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_precondition_badCondition(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2179,7 +2183,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -2190,7 +2194,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_precondition_badErrorMessage(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2225,7 +2229,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -2236,7 +2240,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_postcondition_good(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2266,7 +2270,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2274,7 +2278,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_postcondition_badCondition(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2315,7 +2319,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -2326,7 +2330,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_postcondition_badErrorMessage(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2356,7 +2360,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if !diags.HasErrors() {
 		t.Fatalf("succeeded; want error")
 	}
@@ -2367,7 +2371,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_precondition_count(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2402,7 +2406,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2410,7 +2414,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_postcondition_forEach(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2445,7 +2449,7 @@ resource "aws_instance" "test" {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	if diags.HasErrors() {
 		t.Fatal(diags.ErrWithWarnings())
 	}
@@ -2453,7 +2457,7 @@ resource "aws_instance" "test" {
 
 func TestContext2Validate_deprecatedAttr(t *testing.T) {
 	p := testProvider("aws")
-	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&ProviderSchema{
+	p.GetProviderSchemaResponse = getProviderSchemaResponseFromProviderSchema(&providerSchema{
 		ResourceTypes: map[string]*configschema.Block{
 			"aws_instance": {
 				Attributes: map[string]*configschema.Attribute{
@@ -2479,9 +2483,614 @@ locals {
 		},
 	})
 
-	diags := ctx.Validate(m)
+	diags := ctx.Validate(m, nil)
 	warn := diags.ErrWithWarnings().Error()
 	if !strings.Contains(warn, `The attribute "foo" is deprecated`) {
 		t.Fatalf("expected deprecated warning, got: %q\n", warn)
 	}
 }
+
+func TestContext2Validate_unknownForEach(t *testing.T) {
+	p := testProvider("aws")
+	m := testModuleInline(t, map[string]string{
+		"main.tf": `
+resource "aws_instance" "test" {
+}
+
+locals {
+  follow = {
+    (aws_instance.test.id): "follow"
+  }
+}
+
+resource "aws_instance" "follow" {
+  for_each = local.follow
+}
+ `,
+	})
+	c := testContext2(t, &ContextOpts{
+		Providers: map[addrs.Provider]providers.Factory{
+			addrs.NewDefaultProvider("aws"): testProviderFuncFixed(p),
+		},
+	})
+
+	diags := c.Validate(m, nil)
+	if diags.HasErrors() {
+		t.Fatal(diags.ErrWithWarnings())
+	}
+}
+
+func TestContext2Validate_providerContributedFunctions(t *testing.T) {
+	mockProvider := func() *testing_provider.MockProvider {
+		p := testProvider("test")
+		p.GetProviderSchemaResponse = &providers.GetProviderSchemaResponse{
+			Functions: map[string]providers.FunctionDecl{
+				"count_e": {
+					ReturnType: cty.Number,
+					Parameters: []providers.FunctionParam{
+						{
+							Name: "string",
+							Type: cty.String,
+						},
+					},
+				},
+			},
+		}
+		p.CallFunctionFn = func(req providers.CallFunctionRequest) (resp providers.CallFunctionResponse) {
+			if req.FunctionName != "count_e" {
+				resp.Err = fmt.Errorf("incorrect function name %q", req.FunctionName)
+				return resp
+			}
+			if len(req.Arguments) != 1 {
+				resp.Err = fmt.Errorf("wrong number of arguments %d", len(req.Arguments))
+				return resp
+			}
+			if req.Arguments[0].Type() != cty.String {
+				resp.Err = fmt.Errorf("wrong argument type %#v", req.Arguments[0].Type())
+				return resp
+			}
+			if !req.Arguments[0].IsKnown() {
+				resp.Err = fmt.Errorf("argument is unknown")
+				return resp
+			}
+			if req.Arguments[0].IsNull() {
+				resp.Err = fmt.Errorf("argument is null")
+				return resp
+			}
+
+			str := req.Arguments[0].AsString()
+			count := strings.Count(str, "e")
+			resp.Result = cty.NumberIntVal(int64(count))
+			return resp
+		}
+		return p
+	}
+
+	t.Run("valid", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+locals {
+	result = provider::test::count_e("cheese")
+}
+output "result" {
+	value = local.result
+	precondition {
+		condition     = (local.result == 3)
+		error_message = "Wrong number of Es in my cheese."
+	}
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if diags.HasErrors() {
+			t.Fatal(diags.ErrWithWarnings())
+		}
+
+		if !p.CallFunctionCalled {
+			t.Fatal("CallFunction was not called")
+		}
+	})
+	t.Run("wrong name", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::test::cout_e("cheese")
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		if got, want := diags.Err().Error(), `Unknown provider function: The function "cout_e" is not available from the provider "test"`; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+
+	})
+	t.Run("wrong namespace", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::toast::count_e("cheese")
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		if got, want := diags.Err().Error(), `Unknown provider function: There is no function named "provider::toast::count_e`; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+	})
+	t.Run("wrong argument type", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::test::count_e([])
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		if got, want := diags.Err().Error(), "Invalid function argument: Invalid value for \"string\" parameter: string required."; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+	})
+	t.Run("insufficient arguments", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::test::count_e()
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		if got, want := diags.Err().Error(), "Not enough function arguments: Function \"provider::test::count_e\" expects 1 argument(s). Missing value for \"string\"."; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+	})
+	t.Run("too many arguments", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::test::count_e("cheese", "louise")
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		if got, want := diags.Err().Error(), "Too many function arguments: Function \"provider::test::count_e\" expects only 1 argument(s)."; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+	})
+	t.Run("unexpected null argument", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::test::count_e(null)
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		if got, want := diags.Err().Error(), "Invalid function argument: Invalid value for \"string\" parameter: argument must not be null."; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+	})
+	t.Run("unhandled unknown argument", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		test = {
+			source = "hashicorp/test"
+		}
+	}
+}
+output "result" {
+	value = provider::test::count_e(timestamp())
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		// For this case, validation should succeed without calling the
+		// function yet, because the function doesn't declare that it handles
+		// unknown values and so we must defer validation until a later phase.
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if diags.HasErrors() {
+			t.Fatal(diags.ErrWithWarnings())
+		}
+	})
+	t.Run("provider not declared", func(t *testing.T) {
+		m := testModuleInline(t, map[string]string{
+			"main.tf": `
+terraform {
+	required_providers {
+		# Intentionally no declaration of local name "test" here
+	}
+}
+output "result" {
+	value = provider::test::count_e("cheese")
+}
+`,
+		})
+
+		p := mockProvider()
+		ctx := testContext2(t, &ContextOpts{
+			Providers: map[addrs.Provider]providers.Factory{
+				addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+			},
+		})
+
+		diags := ctx.Validate(m, nil)
+		if p.CallFunctionCalled {
+			t.Error("CallFunction was called, but should not have been")
+		}
+		if !diags.HasErrors() {
+			t.Fatal("unexpected success")
+		}
+		// Module author must declare a provider requirement in order to
+		// import a provider's functions.
+		if got, want := diags.Err().Error(), `Unknown provider function: There is no function named "provider::test::count_e"`; !strings.Contains(got, want) {
+			t.Errorf("wrong error message\nwant substring: %s\ngot: %s", want, got)
+		}
+	})
+}
+
+func TestContextValidate_externalProviders(t *testing.T) {
+
+	m := testModuleInline(t, map[string]string{
+		"main.tf": `
+terraform {
+  required_providers {
+    bar = {
+      source = "hashicorp/bar"
+	}
+  }
+}
+
+provider "bar" {}
+
+resource "bar_instance" "test" {
+  foo = "foo" # should be an int
+}
+`,
+	})
+
+	mustNotConfigure := func(providers.ConfigureProviderRequest) providers.ConfigureProviderResponse {
+		return providers.ConfigureProviderResponse{
+			Diagnostics: tfdiags.Diagnostics{
+				tfdiags.Sourceless(
+					tfdiags.Error,
+					"Pre-configured provider was reconfigured by the modules runtime",
+					"An externally-configured provider should not have its ConfigureProvider function called during planning.",
+				),
+			},
+		}
+	}
+
+	providerAddr := addrs.NewDefaultProvider("bar")
+	providerConfigAddr := addrs.RootProviderConfig{
+		Provider: providerAddr,
+	}
+
+	provider := &testing_provider.MockProvider{
+		GetProviderSchemaResponse: &providers.GetProviderSchemaResponse{
+			Provider: providers.Schema{
+				Block: &configschema.Block{
+					Attributes: map[string]*configschema.Attribute{
+						// We have a required attribute that is not set, we're
+						// expecting this to not matter as we shouldn't validate
+						// the provider configuration as we're using an external
+						// provider.
+						"required": {
+							Type:     cty.String,
+							Required: true,
+						},
+					},
+				},
+			},
+			ResourceTypes: map[string]providers.Schema{
+				"bar_instance": {
+					Block: &configschema.Block{
+						Attributes: map[string]*configschema.Attribute{
+							// We should still validate this attribute as being
+							// incorrect, even though we have an external
+							// provider.
+							"foo": {
+								Type:     cty.Number,
+								Required: true,
+							},
+						},
+					},
+				},
+			},
+		},
+		ConfigureProviderFn: mustNotConfigure,
+	}
+
+	ctx, diags := NewContext(&ContextOpts{
+		PreloadedProviderSchemas: map[addrs.Provider]providers.ProviderSchema{
+			providerAddr: *provider.GetProviderSchemaResponse,
+		},
+	})
+	assertNoDiagnostics(t, diags)
+
+	// Many of the MockProvider methods check for this, so we'll set it to be
+	// true externally.
+	provider.ConfigureProviderCalled = true
+
+	diags = ctx.Validate(m, &ValidateOpts{
+		ExternalProviders: map[addrs.RootProviderConfig]providers.Interface{
+			providerConfigAddr: provider,
+		},
+	})
+
+	// We should have exactly one diagnostic, stating there was an error in the
+	// resource. But nothing complaining about the provider itself.
+
+	if len(diags) != 1 {
+		t.Fatalf("expected exactly one diagnostic, got %d", len(diags))
+	}
+
+	if diff := cmp.Diff(diags[0].Description().Summary, "Incorrect attribute value type"); len(diff) > 0 {
+		t.Errorf("unexpected diagnostic summary: %s", diff)
+	}
+	if diff := cmp.Diff(diags[0].Description().Detail, "Inappropriate value for attribute \"foo\": a number is required."); len(diff) > 0 {
+		t.Errorf("unexpected diagnostic detail: %s", diff)
+	}
+}
+
+func TestContext2Validate_providerSchemaError(t *testing.T) {
+	// validate module and output depends_on
+	m := testModuleInline(t, map[string]string{
+		"main.tf": `
+terraform {
+  required_providers {
+    test = {
+      source = "hashicorp/test"
+    }
+  }
+}
+
+output "foo" {
+	value = provider::test::func("foo")
+}
+`,
+	})
+
+	p := testProvider("test")
+	p.GetProviderSchemaResponse = &providers.GetProviderSchemaResponse{
+		ResourceTypes: map[string]providers.Schema{
+			"test_instance": {
+				Block: &configschema.Block{
+					Attributes: map[string]*configschema.Attribute{
+						"id": {Type: cty.String, Computed: true},
+					},
+				},
+			},
+		},
+		Diagnostics: tfdiags.Diagnostics(nil).Append(errors.New("schema problem!")),
+	}
+
+	ctx := testContext2(t, &ContextOpts{
+		Providers: map[addrs.Provider]providers.Factory{
+			addrs.NewDefaultProvider("test"): testProviderFuncFixed(p),
+		},
+	})
+	diags := ctx.Validate(m, nil)
+	if !diags.HasErrors() {
+		t.Fatal("expected error")
+	}
+
+	// while the function in the config doesn't exist, we should not have gotten
+	// that far and stopped at the schema error first
+	for _, d := range diags {
+		if detail := d.Description().Detail; !strings.Contains(detail, "schema problem!") {
+			t.Errorf("unexpected error: %s", detail)
+		}
+	}
+}
+
+func TestContext2Validate_ephemeralOutput_root(t *testing.T) {
+	m := testModuleInline(t, map[string]string{
+		"main.tf": `
+variable "foo" {
+  ephemeral = true
+  default   = "foo"
+}
+output "test" {
+  ephemeral = true
+  value     = var.foo
+}
+`,
+	})
+
+	ctx := testContext2(t, &ContextOpts{})
+	diags := ctx.Validate(m, &ValidateOpts{})
+	var wantDiags tfdiags.Diagnostics
+	wantDiags = wantDiags.Append(
+		&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Ephemeral output not allowed",
+			Detail:   "Ephemeral outputs are not allowed in context of a root module",
+			Subject: &hcl.Range{
+				Filename: filepath.Join(m.Module.SourceDir, "main.tf"),
+				Start:    hcl.Pos{Line: 6, Column: 1, Byte: 59},
+				End:      hcl.Pos{Line: 6, Column: 14, Byte: 72},
+			},
+		},
+	)
+	assertDiagnosticsMatch(t, diags, wantDiags)
+}
+
+func TestContext2Validate_ephemeralOutput_child(t *testing.T) {
+	m := testModuleInline(t, map[string]string{
+		"child/main.tf": `
+variable "child-eph" {
+  ephemeral = true
+}
+output "out" {
+  ephemeral = true
+  value     = var.child-eph
+}`,
+		"main.tf": `
+variable "eph" {
+  ephemeral = true
+  default   = "foo"
+}
+
+module "child" {
+  source    = "./child"
+  child-eph = var.eph
+}
+`,
+	})
+
+	ctx := testContext2(t, &ContextOpts{})
+	diags := ctx.Validate(m, &ValidateOpts{})
+	assertNoDiagnostics(t, diags)
+}
