@@ -7,11 +7,17 @@ import (
 	"log"
 	"time"
 
+	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/checks"
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/instances"
+	"github.com/hashicorp/terraform/internal/moduletest/mocking"
+	"github.com/hashicorp/terraform/internal/namedvals"
 	"github.com/hashicorp/terraform/internal/plans"
+	"github.com/hashicorp/terraform/internal/plans/deferring"
+	"github.com/hashicorp/terraform/internal/providers"
 	"github.com/hashicorp/terraform/internal/refactoring"
+	"github.com/hashicorp/terraform/internal/resources/ephemeral"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
@@ -28,6 +34,26 @@ type graphWalkOpts struct {
 	Changes    *plans.Changes
 	Config     *configs.Config
 
+	// ExternalProviderConfigs is used for walks that make use of configured
+	// providers (e.g. plan and apply) to satisfy situations where the root
+	// module itself declares that it expects to have providers passed in
+	// from outside.
+	//
+	// This should not be populated for walks that use only unconfigured
+	// providers, such as validate. Populating it for those walks might cause
+	// strange things to happen, because our graph walking machinery doesn't
+	// always take into account what walk type it's dealing with.
+	ExternalProviderConfigs map[addrs.RootProviderConfig]providers.Interface
+
+	// DeferralAlowed indicates that the current runtime supports deferred actions.
+	DeferralAllowed bool
+
+	// ExternalDependencyDeferred indicates that something that this entire
+	// configuration depends on (outside the view of this modules runtime)
+	// has deferred changes, and therefore we must treat _all_ actions
+	// as deferred to produce the correct overall dependency ordering.
+	ExternalDependencyDeferred bool
+
 	// PlanTimeCheckResults should be populated during the apply phase with
 	// the snapshot of check results that was generated during the plan step.
 	//
@@ -41,13 +67,23 @@ type graphWalkOpts struct {
 	// the apply phase.
 	PlanTimeTimestamp time.Time
 
+	// Overrides contains the set of overrides we should apply during this
+	// operation.
+	Overrides *mocking.Overrides
+
 	MoveResults refactoring.MoveResults
+
+	ProviderFuncResults *providers.FunctionResults
+
+	// Forget if set to true will cause the plan to forget all resources. This is
+	// only allowd in the context of a destroy plan.
+	Forget bool
 }
 
 func (c *Context) walk(graph *Graph, operation walkOperation, opts *graphWalkOpts) (*ContextGraphWalker, tfdiags.Diagnostics) {
 	log.Printf("[DEBUG] Starting graph walk: %s", operation.String())
 
-	walker := c.graphWalker(operation, opts)
+	walker := c.graphWalker(graph, operation, opts)
 
 	// Watch for a stop so we can call the provider Stop() API.
 	watchStop, watchWait := c.watchStop(walker)
@@ -62,7 +98,7 @@ func (c *Context) walk(graph *Graph, operation walkOperation, opts *graphWalkOpt
 	return walker, diags
 }
 
-func (c *Context) graphWalker(operation walkOperation, opts *graphWalkOpts) *ContextGraphWalker {
+func (c *Context) graphWalker(graph *Graph, operation walkOperation, opts *graphWalkOpts) *ContextGraphWalker {
 	var state *states.SyncState
 	var refreshState *states.SyncState
 	var prevRunState *states.SyncState
@@ -137,18 +173,30 @@ func (c *Context) graphWalker(operation walkOperation, opts *graphWalkOpts) *Con
 		}
 	}
 
+	deferred := deferring.NewDeferred(opts.DeferralAllowed)
+	if opts.ExternalDependencyDeferred {
+		deferred.SetExternalDependencyDeferred()
+	}
+
 	return &ContextGraphWalker{
-		Context:          c,
-		State:            state,
-		Config:           opts.Config,
-		RefreshState:     refreshState,
-		PrevRunState:     prevRunState,
-		Changes:          changes.SyncWrapper(),
-		Checks:           checkState,
-		InstanceExpander: instances.NewExpander(),
-		MoveResults:      opts.MoveResults,
-		Operation:        operation,
-		StopContext:      c.runContext,
-		PlanTimestamp:    opts.PlanTimeTimestamp,
+		Context:                 c,
+		State:                   state,
+		Config:                  opts.Config,
+		RefreshState:            refreshState,
+		Overrides:               opts.Overrides,
+		PrevRunState:            prevRunState,
+		Changes:                 changes.SyncWrapper(),
+		NamedValues:             namedvals.NewState(),
+		EphemeralResources:      ephemeral.NewResources(),
+		Deferrals:               deferred,
+		Checks:                  checkState,
+		InstanceExpander:        instances.NewExpander(opts.Overrides),
+		ExternalProviderConfigs: opts.ExternalProviderConfigs,
+		MoveResults:             opts.MoveResults,
+		Operation:               operation,
+		StopContext:             c.runContext,
+		PlanTimestamp:           opts.PlanTimeTimestamp,
+		providerFuncResults:     opts.ProviderFuncResults,
+		Forget:                  opts.Forget,
 	}
 }
