@@ -4,7 +4,7 @@
 package terraform
 
 import (
-	"sync"
+	"fmt"
 	"testing"
 
 	"github.com/davecgh/go-spew/spew"
@@ -13,8 +13,12 @@ import (
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/configs/configschema"
+	"github.com/hashicorp/terraform/internal/instances"
+	"github.com/hashicorp/terraform/internal/lang"
 	"github.com/hashicorp/terraform/internal/lang/marks"
+	"github.com/hashicorp/terraform/internal/namedvals"
 	"github.com/hashicorp/terraform/internal/plans"
+	"github.com/hashicorp/terraform/internal/plans/deferring"
 	"github.com/hashicorp/terraform/internal/providers"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
@@ -25,11 +29,14 @@ func TestEvaluatorGetTerraformAttr(t *testing.T) {
 		Meta: &ContextMeta{
 			Env: "foo",
 		},
+		NamedValues: namedvals.NewState(),
 	}
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 
 	t.Run("workspace", func(t *testing.T) {
 		want := cty.StringVal("foo")
@@ -55,11 +62,14 @@ func TestEvaluatorGetPathAttr(t *testing.T) {
 				SourceDir: "bar/baz",
 			},
 		},
+		NamedValues: namedvals.NewState(),
 	}
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 
 	t.Run("module", func(t *testing.T) {
 		want := cty.StringVal("bar/baz")
@@ -123,9 +133,11 @@ func TestEvaluatorGetOutputValue(t *testing.T) {
 	}
 
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 
 	want := cty.StringVal("first").Mark(marks.Sensitive)
 	got, diags := scope.Data.GetOutput(addrs.OutputValue{
@@ -155,6 +167,14 @@ func TestEvaluatorGetOutputValue(t *testing.T) {
 // This particularly tests that a sensitive attribute in config
 // results in a value that has a "sensitive" cty Mark
 func TestEvaluatorGetInputVariable(t *testing.T) {
+	namedValues := namedvals.NewState()
+	namedValues.SetInputVariableValue(
+		addrs.RootModuleInstance.InputVariable("some_var"), cty.StringVal("bar"),
+	)
+	namedValues.SetInputVariableValue(
+		addrs.RootModuleInstance.InputVariable("some_other_var"), cty.StringVal("boop").Mark(marks.Sensitive),
+	)
+
 	evaluator := &Evaluator{
 		Meta: &ContextMeta{
 			Env: "foo",
@@ -180,19 +200,15 @@ func TestEvaluatorGetInputVariable(t *testing.T) {
 				},
 			},
 		},
-		VariableValues: map[string]map[string]cty.Value{
-			"": {
-				"some_var":       cty.StringVal("bar"),
-				"some_other_var": cty.StringVal("boop").Mark(marks.Sensitive),
-			},
-		},
-		VariableValuesLock: &sync.Mutex{},
+		NamedValues: namedValues,
 	}
 
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 
 	want := cty.StringVal("bar").Mark(marks.Sensitive)
 	got, diags := scope.Data.GetInputVariable(addrs.InputVariable{
@@ -230,6 +246,14 @@ func TestEvaluatorGetResource(t *testing.T) {
 			&states.ResourceInstanceObjectSrc{
 				Status:    states.ObjectReady,
 				AttrsJSON: []byte(`{"id":"foo", "nesting_list": [{"sensitive_value":"abc"}], "nesting_map": {"foo":{"foo":"x"}}, "nesting_set": [{"baz":"abc"}], "nesting_single": {"boop":"abc"}, "nesting_nesting": {"nesting_list":[{"sensitive_value":"abc"}]}, "value":"hello"}`),
+				AttrSensitivePaths: []cty.Path{
+					cty.GetAttrPath("nesting_list").IndexInt(0).GetAttr("sensitive_value"),
+					cty.GetAttrPath("nesting_map").IndexString("foo").GetAttr("foo"),
+					cty.GetAttrPath("nesting_nesting").GetAttr("nesting_list").IndexInt(0).GetAttr("sensitive_value"),
+					cty.GetAttrPath("nesting_set"),
+					cty.GetAttrPath("nesting_single").GetAttr("boop"),
+					cty.GetAttrPath("value"),
+				},
 			},
 			addrs.AbsProviderConfig{
 				Provider: addrs.NewDefaultProvider("test"),
@@ -264,7 +288,9 @@ func TestEvaluatorGetResource(t *testing.T) {
 				},
 			},
 		},
-		State: stateSync,
+		State:       stateSync,
+		NamedValues: namedvals.NewState(),
+		Deferrals:   deferring.NewDeferred(false),
 		Plugins: schemaOnlyProvidersForTesting(map[addrs.Provider]providers.ProviderSchema{
 			addrs.NewDefaultProvider("test"): {
 				ResourceTypes: map[string]providers.Schema{
@@ -340,9 +366,11 @@ func TestEvaluatorGetResource(t *testing.T) {
 	}
 
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 
 	want := cty.ObjectVal(map[string]cty.Value{
 		"id": cty.StringVal("foo"),
@@ -427,10 +455,10 @@ func TestEvaluatorGetResource_changes(t *testing.T) {
 			After: cty.ObjectVal(map[string]cty.Value{
 				"id":              cty.StringVal("foo"),
 				"to_mark_val":     cty.StringVal("pizza").Mark(marks.Sensitive),
-				"sensitive_value": cty.StringVal("abc"),
+				"sensitive_value": cty.StringVal("abc").Mark(marks.Sensitive),
 				"sensitive_collection": cty.MapVal(map[string]cty.Value{
 					"boop": cty.StringVal("beep"),
-				}),
+				}).Mark(marks.Sensitive),
 			}),
 		},
 	}
@@ -475,10 +503,8 @@ func TestEvaluatorGetResource_changes(t *testing.T) {
 		Type: "test_resource",
 		Name: "foo",
 	}
-	schema, _ := schemas.ResourceTypeConfig(addrs.NewDefaultProvider("test"), addr.Mode, addr.Type)
-	// This encoding separates out the After's marks into its AfterValMarks
-	csrc, _ := change.Encode(schema.ImpliedType())
-	changesSync.AppendResourceInstanceChange(csrc)
+
+	changesSync.AppendResourceInstanceChange(change)
 
 	evaluator := &Evaluator{
 		Meta: &ContextMeta{
@@ -501,14 +527,18 @@ func TestEvaluatorGetResource_changes(t *testing.T) {
 				},
 			},
 		},
-		State:   stateSync,
-		Plugins: schemaOnlyProvidersForTesting(schemas.Providers),
+		State:       stateSync,
+		NamedValues: namedvals.NewState(),
+		Deferrals:   deferring.NewDeferred(false),
+		Plugins:     schemaOnlyProvidersForTesting(schemas.Providers),
 	}
 
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 
 	want := cty.ObjectVal(map[string]cty.Value{
 		"id":              cty.StringVal("foo"),
@@ -531,19 +561,18 @@ func TestEvaluatorGetResource_changes(t *testing.T) {
 }
 
 func TestEvaluatorGetModule(t *testing.T) {
-	// Create a new evaluator with an existing state
-	stateSync := states.BuildState(func(ss *states.SyncState) {
-		ss.SetOutputValue(
-			addrs.OutputValue{Name: "out"}.Absolute(addrs.ModuleInstance{addrs.ModuleInstanceStep{Name: "mod"}}),
-			cty.StringVal("bar"),
-			true,
-		)
-	}).SyncWrapper()
-	evaluator := evaluatorForModule(stateSync, plans.NewChanges().SyncWrapper())
+	evaluator := evaluatorForModule(states.NewState().SyncWrapper(), plans.NewChanges().SyncWrapper())
+	evaluator.Instances.SetModuleSingle(addrs.RootModuleInstance, addrs.ModuleCall{Name: "mod"})
+	evaluator.NamedValues.SetOutputValue(
+		addrs.OutputValue{Name: "out"}.Absolute(addrs.ModuleInstance{addrs.ModuleInstanceStep{Name: "mod"}}),
+		cty.StringVal("bar").Mark(marks.Sensitive),
+	)
 	data := &evaluationStateData{
-		Evaluator: evaluator,
+		evaluationData: &evaluationData{
+			Evaluator: evaluator,
+		},
 	}
-	scope := evaluator.Scope(data, nil, nil)
+	scope := evaluator.Scope(data, nil, nil, lang.ExternalFuncs{})
 	want := cty.ObjectVal(map[string]cty.Value{"out": cty.StringVal("bar").Mark(marks.Sensitive)})
 	got, diags := scope.Data.GetModule(addrs.ModuleCall{
 		Name: "mod",
@@ -553,53 +582,7 @@ func TestEvaluatorGetModule(t *testing.T) {
 		t.Errorf("unexpected diagnostics %s", spew.Sdump(diags))
 	}
 	if !got.RawEquals(want) {
-		t.Errorf("wrong result %#v; want %#v", got, want)
-	}
-
-	// Changes should override the state value
-	changesSync := plans.NewChanges().SyncWrapper()
-	change := &plans.OutputChange{
-		Addr:      addrs.OutputValue{Name: "out"}.Absolute(addrs.ModuleInstance{addrs.ModuleInstanceStep{Name: "mod"}}),
-		Sensitive: true,
-		Change: plans.Change{
-			After: cty.StringVal("baz"),
-		},
-	}
-	cs, _ := change.Encode()
-	changesSync.AppendOutputChange(cs)
-	evaluator = evaluatorForModule(stateSync, changesSync)
-	data = &evaluationStateData{
-		Evaluator: evaluator,
-	}
-	scope = evaluator.Scope(data, nil, nil)
-	want = cty.ObjectVal(map[string]cty.Value{"out": cty.StringVal("baz").Mark(marks.Sensitive)})
-	got, diags = scope.Data.GetModule(addrs.ModuleCall{
-		Name: "mod",
-	}, tfdiags.SourceRange{})
-
-	if len(diags) != 0 {
-		t.Errorf("unexpected diagnostics %s", spew.Sdump(diags))
-	}
-	if !got.RawEquals(want) {
-		t.Errorf("wrong result %#v; want %#v", got, want)
-	}
-
-	// Test changes with empty state
-	evaluator = evaluatorForModule(states.NewState().SyncWrapper(), changesSync)
-	data = &evaluationStateData{
-		Evaluator: evaluator,
-	}
-	scope = evaluator.Scope(data, nil, nil)
-	want = cty.ObjectVal(map[string]cty.Value{"out": cty.StringVal("baz").Mark(marks.Sensitive)})
-	got, diags = scope.Data.GetModule(addrs.ModuleCall{
-		Name: "mod",
-	}, tfdiags.SourceRange{})
-
-	if len(diags) != 0 {
-		t.Errorf("unexpected diagnostics %s", spew.Sdump(diags))
-	}
-	if !got.RawEquals(want) {
-		t.Errorf("wrong result %#v; want %#v", got, want)
+		t.Errorf("wrong result\ngot:  %#v\nwant: %#v", got, want)
 	}
 }
 
@@ -630,99 +613,108 @@ func evaluatorForModule(stateSync *states.SyncState, changesSync *plans.ChangesS
 				},
 			},
 		},
-		State:   stateSync,
-		Changes: changesSync,
+		State:       stateSync,
+		Changes:     changesSync,
+		Instances:   instances.NewExpander(nil),
+		NamedValues: namedvals.NewState(),
 	}
 }
 
-func TestGetRunBlocks(t *testing.T) {
-	evaluator := &Evaluator{
-		AlternateStates: map[string]*evaluationStateData{
-			"zero": {
-				Evaluator: &Evaluator{
-					State: states.BuildState(func(state *states.SyncState) {
-						state.SetOutputValue(addrs.AbsOutputValue{
-							Module: addrs.RootModuleInstance,
-							OutputValue: addrs.OutputValue{
-								Name: "value",
-							},
-						}, cty.StringVal("Hello, world!"), false)
-					}).SyncWrapper(),
-					Config: &configs.Config{
-						Module: &configs.Module{
-							Outputs: map[string]*configs.Output{
-								"value": {
-									Name: "value",
-								},
-							},
-						},
-					},
-				},
-			},
-			"one": {
-				Evaluator: &Evaluator{
-					State: states.BuildState(func(state *states.SyncState) {
-						state.SetOutputValue(addrs.AbsOutputValue{
-							Module: addrs.RootModuleInstance,
-							OutputValue: addrs.OutputValue{
-								Name: "value",
-							},
-						}, cty.StringVal("Hello, universe!"), false)
-					}).SyncWrapper(),
-					Config: &configs.Config{
-						Module: &configs.Module{
-							Outputs: map[string]*configs.Output{
-								"value": {
-									Name: "value",
-								},
-							},
-						},
-					},
-				},
-			},
-		},
-	}
+// fakeEvaluationData is an implementation of [lang.Data] that answers most
+// questions just by returning data directly from the maps stored inside it.
+type fakeEvaluationData struct {
+	checkBlocks    map[addrs.Check]cty.Value
+	countAttrs     map[addrs.CountAttr]cty.Value
+	forEachAttrs   map[addrs.ForEachAttr]cty.Value
+	inputVariables map[addrs.InputVariable]cty.Value
+	localValues    map[addrs.LocalValue]cty.Value
+	modules        map[addrs.ModuleCall]cty.Value
+	outputValues   map[addrs.OutputValue]cty.Value
+	pathAttrs      map[addrs.PathAttr]cty.Value
+	resources      map[addrs.Resource]cty.Value
+	runBlocks      map[addrs.Run]cty.Value
+	terraformAttrs map[addrs.TerraformAttr]cty.Value
 
-	data := &evaluationStateData{
-		Evaluator:       evaluator,
-		ModulePath:      nil,
-		InstanceKeyData: EvalDataForNoInstanceKey,
+	// staticValidateRefs optionally implements [lang.Data.StaticValidateReferences],
+	// but can be left as nil to just skip static validation altogether.
+	staticValidateRefs func(refs []*addrs.Reference, self addrs.Referenceable, source addrs.Referenceable) tfdiags.Diagnostics
+}
+
+var _ lang.Data = (*fakeEvaluationData)(nil)
+
+func fakeEvaluationDataLookup[Addr interface {
+	comparable
+	addrs.Referenceable
+}](addr Addr, _ tfdiags.SourceRange, table map[Addr]cty.Value) (cty.Value, tfdiags.Diagnostics) {
+	ret, ok := table[addr]
+	if !ok {
+		var diags tfdiags.Diagnostics
+		diags = diags.Append(fmt.Errorf("fakeEvaluationData does not know about %s", addr))
+		return cty.DynamicVal, diags
 	}
+	return ret, nil
+}
+
+// GetCheckBlock implements lang.Data.
+func (d *fakeEvaluationData) GetCheckBlock(addr addrs.Check, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.checkBlocks)
+}
+
+// GetCountAttr implements lang.Data.
+func (d *fakeEvaluationData) GetCountAttr(addr addrs.CountAttr, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.countAttrs)
+}
+
+// GetForEachAttr implements lang.Data.
+func (d *fakeEvaluationData) GetForEachAttr(addr addrs.ForEachAttr, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.forEachAttrs)
+}
 
-	scope := evaluator.Scope(data, nil, nil)
-	got, diags := scope.Data.GetRunBlock(addrs.Run{Name: "zero"}, tfdiags.SourceRange{})
-	want := cty.ObjectVal(map[string]cty.Value{
-		"value": cty.StringVal("Hello, world!"),
-	})
+// GetInputVariable implements lang.Data.
+func (d *fakeEvaluationData) GetInputVariable(addr addrs.InputVariable, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.inputVariables)
+}
 
-	if diags.HasErrors() {
-		t.Fatalf("unexpected diagnostics: %s", diags.Err())
-	}
+// GetLocalValue implements lang.Data.
+func (d *fakeEvaluationData) GetLocalValue(addr addrs.LocalValue, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.localValues)
+}
 
-	if !got.RawEquals(want) {
-		t.Errorf("\ngot: %#v\nwant: %#v", got, want)
-	}
+// GetModule implements lang.Data.
+func (d *fakeEvaluationData) GetModule(addr addrs.ModuleCall, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.modules)
+}
 
-	got, diags = scope.Data.GetRunBlock(addrs.Run{Name: "one"}, tfdiags.SourceRange{})
-	want = cty.ObjectVal(map[string]cty.Value{
-		"value": cty.StringVal("Hello, universe!"),
-	})
+// GetOutput implements lang.Data.
+func (d *fakeEvaluationData) GetOutput(addr addrs.OutputValue, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.outputValues)
+}
 
-	if diags.HasErrors() {
-		t.Fatalf("unexpected diagnostics: %s", diags.Err())
-	}
+// GetPathAttr implements lang.Data.
+func (d *fakeEvaluationData) GetPathAttr(addr addrs.PathAttr, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.pathAttrs)
+}
 
-	if !got.RawEquals(want) {
-		t.Errorf("\ngot: %#v\nwant: %#v", got, want)
-	}
+// GetResource implements lang.Data.
+func (d *fakeEvaluationData) GetResource(addr addrs.Resource, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.resources)
+}
 
-	_, diags = scope.Data.GetRunBlock(addrs.Run{Name: "two"}, tfdiags.SourceRange{})
+// GetRunBlock implements lang.Data.
+func (d *fakeEvaluationData) GetRunBlock(addr addrs.Run, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.runBlocks)
+}
 
-	if !diags.HasErrors() {
-		t.Fatalf("expected some diags but got none")
-	}
+// GetTerraformAttr implements lang.Data.
+func (d *fakeEvaluationData) GetTerraformAttr(addr addrs.TerraformAttr, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
+	return fakeEvaluationDataLookup(addr, rng, d.terraformAttrs)
+}
 
-	if diags[0].Description().Summary != "Reference to unavailable run block" {
-		t.Errorf("retrieved unexpected diagnostic: %s", diags[0].Description().Summary)
+// StaticValidateReferences implements lang.Data.
+func (d *fakeEvaluationData) StaticValidateReferences(refs []*addrs.Reference, self addrs.Referenceable, source addrs.Referenceable) tfdiags.Diagnostics {
+	if d.staticValidateRefs == nil {
+		// By default we just skip static validation
+		return nil
 	}
+	return d.staticValidateRefs(refs, self, source)
 }
