@@ -21,7 +21,7 @@ type MockHook struct {
 
 	PreApplyCalled       bool
 	PreApplyAddr         addrs.AbsResourceInstance
-	PreApplyGen          states.Generation
+	PreApplyGen          addrs.DeposedKey
 	PreApplyAction       plans.Action
 	PreApplyPriorState   cty.Value
 	PreApplyPlannedState cty.Value
@@ -30,16 +30,16 @@ type MockHook struct {
 
 	PostApplyCalled      bool
 	PostApplyAddr        addrs.AbsResourceInstance
-	PostApplyGen         states.Generation
+	PostApplyGen         addrs.DeposedKey
 	PostApplyNewState    cty.Value
 	PostApplyError       error
 	PostApplyReturn      HookAction
 	PostApplyReturnError error
-	PostApplyFn          func(addrs.AbsResourceInstance, states.Generation, cty.Value, error) (HookAction, error)
+	PostApplyFn          func(addrs.AbsResourceInstance, addrs.DeposedKey, cty.Value, error) (HookAction, error)
 
 	PreDiffCalled        bool
 	PreDiffAddr          addrs.AbsResourceInstance
-	PreDiffGen           states.Generation
+	PreDiffGen           addrs.DeposedKey
 	PreDiffPriorState    cty.Value
 	PreDiffProposedState cty.Value
 	PreDiffReturn        HookAction
@@ -47,7 +47,7 @@ type MockHook struct {
 
 	PostDiffCalled       bool
 	PostDiffAddr         addrs.AbsResourceInstance
-	PostDiffGen          states.Generation
+	PostDiffGen          addrs.DeposedKey
 	PostDiffAction       plans.Action
 	PostDiffPriorState   cty.Value
 	PostDiffPlannedState cty.Value
@@ -86,14 +86,14 @@ type MockHook struct {
 
 	PreRefreshCalled     bool
 	PreRefreshAddr       addrs.AbsResourceInstance
-	PreRefreshGen        states.Generation
+	PreRefreshGen        addrs.DeposedKey
 	PreRefreshPriorState cty.Value
 	PreRefreshReturn     HookAction
 	PreRefreshError      error
 
 	PostRefreshCalled     bool
 	PostRefreshAddr       addrs.AbsResourceInstance
-	PostRefreshGen        states.Generation
+	PostRefreshGen        addrs.DeposedKey
 	PostRefreshPriorState cty.Value
 	PostRefreshNewState   cty.Value
 	PostRefreshReturn     HookAction
@@ -131,6 +131,17 @@ type MockHook struct {
 	PostApplyImportReturn HookAction
 	PostApplyImportError  error
 
+	PreEphemeralOpCalled      bool
+	PreEphemeralOpAddr        addrs.AbsResourceInstance
+	PreEphemeralOpReturn      HookAction
+	PreEphemeralOpReturnError error
+
+	PostEphemeralOpCalled      bool
+	PostEphemeralOpAddr        addrs.AbsResourceInstance
+	PostEphemeralOpError       error
+	PostEphemeralOpReturn      HookAction
+	PostEphemeralOpReturnError error
+
 	StoppingCalled bool
 
 	PostStateUpdateCalled bool
@@ -141,181 +152,200 @@ type MockHook struct {
 
 var _ Hook = (*MockHook)(nil)
 
-func (h *MockHook) PreApply(addr addrs.AbsResourceInstance, gen states.Generation, action plans.Action, priorState, plannedNewState cty.Value) (HookAction, error) {
+func (h *MockHook) PreApply(id HookResourceIdentity, dk addrs.DeposedKey, action plans.Action, priorState, plannedNewState cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PreApplyCalled = true
-	h.PreApplyAddr = addr
-	h.PreApplyGen = gen
+	h.PreApplyAddr = id.Addr
+	h.PreApplyGen = dk
 	h.PreApplyAction = action
 	h.PreApplyPriorState = priorState
 	h.PreApplyPlannedState = plannedNewState
 	return h.PreApplyReturn, h.PreApplyError
 }
 
-func (h *MockHook) PostApply(addr addrs.AbsResourceInstance, gen states.Generation, newState cty.Value, err error) (HookAction, error) {
+func (h *MockHook) PostApply(id HookResourceIdentity, dk addrs.DeposedKey, newState cty.Value, err error) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostApplyCalled = true
-	h.PostApplyAddr = addr
-	h.PostApplyGen = gen
+	h.PostApplyAddr = id.Addr
+	h.PostApplyGen = dk
 	h.PostApplyNewState = newState
 	h.PostApplyError = err
 
 	if h.PostApplyFn != nil {
-		return h.PostApplyFn(addr, gen, newState, err)
+		return h.PostApplyFn(id.Addr, dk, newState, err)
 	}
 
 	return h.PostApplyReturn, h.PostApplyReturnError
 }
 
-func (h *MockHook) PreDiff(addr addrs.AbsResourceInstance, gen states.Generation, priorState, proposedNewState cty.Value) (HookAction, error) {
+func (h *MockHook) PreDiff(id HookResourceIdentity, dk addrs.DeposedKey, priorState, proposedNewState cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PreDiffCalled = true
-	h.PreDiffAddr = addr
-	h.PreDiffGen = gen
+	h.PreDiffAddr = id.Addr
+	h.PreDiffGen = dk
 	h.PreDiffPriorState = priorState
 	h.PreDiffProposedState = proposedNewState
 	return h.PreDiffReturn, h.PreDiffError
 }
 
-func (h *MockHook) PostDiff(addr addrs.AbsResourceInstance, gen states.Generation, action plans.Action, priorState, plannedNewState cty.Value) (HookAction, error) {
+func (h *MockHook) PostDiff(id HookResourceIdentity, dk addrs.DeposedKey, action plans.Action, priorState, plannedNewState cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostDiffCalled = true
-	h.PostDiffAddr = addr
-	h.PostDiffGen = gen
+	h.PostDiffAddr = id.Addr
+	h.PostDiffGen = dk
 	h.PostDiffAction = action
 	h.PostDiffPriorState = priorState
 	h.PostDiffPlannedState = plannedNewState
 	return h.PostDiffReturn, h.PostDiffError
 }
 
-func (h *MockHook) PreProvisionInstance(addr addrs.AbsResourceInstance, state cty.Value) (HookAction, error) {
+func (h *MockHook) PreProvisionInstance(id HookResourceIdentity, state cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PreProvisionInstanceCalled = true
-	h.PreProvisionInstanceAddr = addr
+	h.PreProvisionInstanceAddr = id.Addr
 	h.PreProvisionInstanceState = state
 	return h.PreProvisionInstanceReturn, h.PreProvisionInstanceError
 }
 
-func (h *MockHook) PostProvisionInstance(addr addrs.AbsResourceInstance, state cty.Value) (HookAction, error) {
+func (h *MockHook) PostProvisionInstance(id HookResourceIdentity, state cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostProvisionInstanceCalled = true
-	h.PostProvisionInstanceAddr = addr
+	h.PostProvisionInstanceAddr = id.Addr
 	h.PostProvisionInstanceState = state
 	return h.PostProvisionInstanceReturn, h.PostProvisionInstanceError
 }
 
-func (h *MockHook) PreProvisionInstanceStep(addr addrs.AbsResourceInstance, typeName string) (HookAction, error) {
+func (h *MockHook) PreProvisionInstanceStep(id HookResourceIdentity, typeName string) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PreProvisionInstanceStepCalled = true
-	h.PreProvisionInstanceStepAddr = addr
+	h.PreProvisionInstanceStepAddr = id.Addr
 	h.PreProvisionInstanceStepProvisionerType = typeName
 	return h.PreProvisionInstanceStepReturn, h.PreProvisionInstanceStepError
 }
 
-func (h *MockHook) PostProvisionInstanceStep(addr addrs.AbsResourceInstance, typeName string, err error) (HookAction, error) {
+func (h *MockHook) PostProvisionInstanceStep(id HookResourceIdentity, typeName string, err error) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostProvisionInstanceStepCalled = true
-	h.PostProvisionInstanceStepAddr = addr
+	h.PostProvisionInstanceStepAddr = id.Addr
 	h.PostProvisionInstanceStepProvisionerType = typeName
 	h.PostProvisionInstanceStepErrorArg = err
 	return h.PostProvisionInstanceStepReturn, h.PostProvisionInstanceStepError
 }
 
-func (h *MockHook) ProvisionOutput(addr addrs.AbsResourceInstance, typeName string, line string) {
+func (h *MockHook) ProvisionOutput(id HookResourceIdentity, typeName string, line string) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.ProvisionOutputCalled = true
-	h.ProvisionOutputAddr = addr
+	h.ProvisionOutputAddr = id.Addr
 	h.ProvisionOutputProvisionerType = typeName
 	h.ProvisionOutputMessage = line
 }
 
-func (h *MockHook) PreRefresh(addr addrs.AbsResourceInstance, gen states.Generation, priorState cty.Value) (HookAction, error) {
+func (h *MockHook) PreRefresh(id HookResourceIdentity, dk addrs.DeposedKey, priorState cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PreRefreshCalled = true
-	h.PreRefreshAddr = addr
-	h.PreRefreshGen = gen
+	h.PreRefreshAddr = id.Addr
+	h.PreRefreshGen = dk
 	h.PreRefreshPriorState = priorState
 	return h.PreRefreshReturn, h.PreRefreshError
 }
 
-func (h *MockHook) PostRefresh(addr addrs.AbsResourceInstance, gen states.Generation, priorState cty.Value, newState cty.Value) (HookAction, error) {
+func (h *MockHook) PostRefresh(id HookResourceIdentity, dk addrs.DeposedKey, priorState cty.Value, newState cty.Value) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostRefreshCalled = true
-	h.PostRefreshAddr = addr
+	h.PostRefreshAddr = id.Addr
 	h.PostRefreshPriorState = priorState
 	h.PostRefreshNewState = newState
 	return h.PostRefreshReturn, h.PostRefreshError
 }
 
-func (h *MockHook) PreImportState(addr addrs.AbsResourceInstance, importID string) (HookAction, error) {
+func (h *MockHook) PreImportState(id HookResourceIdentity, importID string) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PreImportStateCalled = true
-	h.PreImportStateAddr = addr
+	h.PreImportStateAddr = id.Addr
 	h.PreImportStateID = importID
 	return h.PreImportStateReturn, h.PreImportStateError
 }
 
-func (h *MockHook) PostImportState(addr addrs.AbsResourceInstance, imported []providers.ImportedResource) (HookAction, error) {
+func (h *MockHook) PostImportState(id HookResourceIdentity, imported []providers.ImportedResource) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostImportStateCalled = true
-	h.PostImportStateAddr = addr
+	h.PostImportStateAddr = id.Addr
 	h.PostImportStateNewStates = imported
 	return h.PostImportStateReturn, h.PostImportStateError
 }
 
-func (h *MockHook) PrePlanImport(addr addrs.AbsResourceInstance, importID string) (HookAction, error) {
+func (h *MockHook) PrePlanImport(id HookResourceIdentity, importID string) (HookAction, error) {
 	h.PrePlanImportCalled = true
-	h.PrePlanImportAddr = addr
+	h.PrePlanImportAddr = id.Addr
 	return h.PrePlanImportReturn, h.PrePlanImportError
 }
 
-func (h *MockHook) PostPlanImport(addr addrs.AbsResourceInstance, imported []providers.ImportedResource) (HookAction, error) {
+func (h *MockHook) PostPlanImport(id HookResourceIdentity, imported []providers.ImportedResource) (HookAction, error) {
 	h.PostPlanImportCalled = true
-	h.PostPlanImportAddr = addr
+	h.PostPlanImportAddr = id.Addr
 	return h.PostPlanImportReturn, h.PostPlanImportError
 }
 
-func (h *MockHook) PreApplyImport(addr addrs.AbsResourceInstance, importing plans.ImportingSrc) (HookAction, error) {
+func (h *MockHook) PreApplyImport(id HookResourceIdentity, importing plans.ImportingSrc) (HookAction, error) {
 	h.PreApplyImportCalled = true
-	h.PreApplyImportAddr = addr
+	h.PreApplyImportAddr = id.Addr
 	return h.PreApplyImportReturn, h.PreApplyImportError
 }
 
-func (h *MockHook) PostApplyImport(addr addrs.AbsResourceInstance, importing plans.ImportingSrc) (HookAction, error) {
+func (h *MockHook) PostApplyImport(id HookResourceIdentity, importing plans.ImportingSrc) (HookAction, error) {
 	h.Lock()
 	defer h.Unlock()
 
 	h.PostApplyImportCalled = true
-	h.PostApplyImportAddr = addr
+	h.PostApplyImportAddr = id.Addr
 	return h.PostApplyImportReturn, h.PostApplyImportError
 }
 
+func (h *MockHook) PreEphemeralOp(id HookResourceIdentity, action plans.Action) (HookAction, error) {
+	h.Lock()
+	defer h.Unlock()
+
+	h.PreEphemeralOpCalled = true
+	h.PreEphemeralOpAddr = id.Addr
+	return h.PreEphemeralOpReturn, h.PreEphemeralOpReturnError
+}
+
+func (h *MockHook) PostEphemeralOp(id HookResourceIdentity, action plans.Action, opErr error) (HookAction, error) {
+	h.Lock()
+	defer h.Unlock()
+
+	h.PostEphemeralOpCalled = true
+	h.PostEphemeralOpAddr = id.Addr
+	h.PostEphemeralOpError = opErr
+	return h.PostEphemeralOpReturn, h.PostEphemeralOpReturnError
+}
+
 func (h *MockHook) Stopping() {
 	h.Lock()
 	defer h.Unlock()
