@@ -14,7 +14,7 @@ import (
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/dag"
 	"github.com/hashicorp/terraform/internal/instances"
-	"github.com/hashicorp/terraform/internal/lang"
+	"github.com/hashicorp/terraform/internal/lang/langrefs"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
@@ -50,7 +50,7 @@ func (n *nodeExpandModuleVariable) temporaryValue() bool {
 	return true
 }
 
-func (n *nodeExpandModuleVariable) DynamicExpand(ctx EvalContext) (*Graph, error) {
+func (n *nodeExpandModuleVariable) DynamicExpand(ctx EvalContext) (*Graph, tfdiags.Diagnostics) {
 	var g Graph
 
 	// If this variable has preconditions, we need to report these checks now.
@@ -65,9 +65,10 @@ func (n *nodeExpandModuleVariable) DynamicExpand(ctx EvalContext) (*Graph, error
 	}
 
 	expander := ctx.InstanceExpander()
-	for _, module := range expander.ExpandModule(n.Module) {
+	forEachModuleInstance(expander, n.Module, false, func(module addrs.ModuleInstance) {
 		addr := n.Addr.Absolute(module)
 		if checkableAddrs != nil {
+			log.Printf("[TRACE] nodeExpandModuleVariable: found checkable object %s", addr)
 			checkableAddrs.Add(addr)
 		}
 
@@ -79,7 +80,17 @@ func (n *nodeExpandModuleVariable) DynamicExpand(ctx EvalContext) (*Graph, error
 			DestroyApply:   n.DestroyApply,
 		}
 		g.Add(o)
-	}
+	}, func(pem addrs.PartialExpandedModule) {
+		addr := addrs.ObjectInPartialExpandedModule(pem, n.Addr)
+		o := &nodeModuleVariableInPartialModule{
+			Addr:           addr,
+			Config:         n.Config,
+			Expr:           n.Expr,
+			ModuleInstance: pem,
+			DestroyApply:   n.DestroyApply,
+		}
+		g.Add(o)
+	})
 	addRootNodeToGraph(&g)
 
 	if checkableAddrs != nil {
@@ -122,7 +133,7 @@ func (n *nodeExpandModuleVariable) References() []*addrs.Reference {
 	// where our associated variable was declared, which is correct because
 	// our value expression is assigned within a "module" block in the parent
 	// module.
-	refs, _ := lang.ReferencesInExpr(addrs.ParseRef, n.Expr)
+	refs, _ := langrefs.ReferencesInExpr(addrs.ParseRef, n.Expr)
 	return refs
 }
 
@@ -136,6 +147,25 @@ func (n *nodeExpandModuleVariable) ReferenceableAddrs() []addrs.Referenceable {
 	return []addrs.Referenceable{n.Addr}
 }
 
+// variableValidationRules implements [graphNodeValidatableVariable].
+func (n *nodeExpandModuleVariable) variableValidationRules() (addrs.ConfigInputVariable, []*configs.CheckRule, hcl.Range) {
+	var defnRange hcl.Range
+	if n.Expr != nil { // should always be set in real calls, but not always in tests
+		defnRange = n.Expr.Range()
+	}
+	if n.DestroyApply {
+		// We don't perform any variable validation during the apply phase
+		// of a destroy, because validation rules typically aren't prepared
+		// for dealing with things already having been destroyed.
+		return n.Addr.InModule(n.Module), nil, defnRange
+	}
+	var rules []*configs.CheckRule
+	if n.Config != nil { // always in normal code, but sometimes not in unit tests
+		rules = n.Config.Validations
+	}
+	return n.Addr.InModule(n.Module), rules, defnRange
+}
+
 // nodeModuleVariable represents a module variable input during
 // the apply step.
 type nodeModuleVariable struct {
@@ -146,6 +176,11 @@ type nodeModuleVariable struct {
 	// ModuleCallArguments, ex. so count.index and each.key can resolve
 	ModuleInstance addrs.ModuleInstance
 
+	// ModuleCallConfig is the module call that the expression in field Expr
+	// came from, which helps decide what [instances.RepetitionData] we should
+	// use when evaluating Expr.
+	ModuleCallConfig *configs.ModuleCall
+
 	// DestroyApply must be set to true when applying a destroy operation and
 	// false otherwise.
 	DestroyApply bool
@@ -201,16 +236,11 @@ func (n *nodeModuleVariable) Execute(ctx EvalContext, op walkOperation) (diags t
 
 	// Set values for arguments of a child module call, for later retrieval
 	// during expression evaluation.
-	_, call := n.Addr.Module.CallInstance()
-	ctx.SetModuleCallArgument(call, n.Addr.Variable, val)
+	ctx.NamedValues().SetInputVariableValue(n.Addr, val)
+
+	// Custom validation rules are handled by a separate graph node of type
+	// nodeVariableValidation, added by variableValidationTransformer.
 
-	// Skip evalVariableValidations during destroy operations. We still want
-	// to evaluate the variable in case it is used to initialise providers
-	// or something downstream but we don't need to report on the success
-	// or failure of any validations for destroy operations.
-	if !n.DestroyApply {
-		diags = diags.Append(evalVariableValidations(n.Addr, n.Config, n.Expr, ctx))
-	}
 	return diags
 }
 
@@ -228,12 +258,6 @@ func (n *nodeModuleVariable) DotNode(name string, opts *dag.DotOpts) *dag.DotNod
 // evalModuleVariable produces the value for a particular variable as will
 // be used by a child module instance.
 //
-// The result is written into a map, with its key set to the local name of the
-// variable, disregarding the module instance address. A map is returned instead
-// of a single value as a result of trying to be convenient for use with
-// EvalContext.SetModuleCallArguments, which expects a map to merge in with any
-// existing arguments.
-//
 // validateOnly indicates that this evaluation is only for config
 // validation, and we will not have any expansion module instance
 // repetition data.
@@ -248,11 +272,10 @@ func (n *nodeModuleVariable) evalModuleVariable(ctx EvalContext, validateOnly bo
 		case validateOnly:
 			// the instance expander does not track unknown expansion values, so we
 			// have to assume all RepetitionData is unknown.
-			moduleInstanceRepetitionData = instances.RepetitionData{
-				CountIndex: cty.UnknownVal(cty.Number),
-				EachKey:    cty.UnknownVal(cty.String),
-				EachValue:  cty.DynamicVal,
-			}
+			// TODO: Ideally we should vary the placeholder we use here based
+			// on how the module call repetition was configured, but we don't
+			// have enough information here to decide that.
+			moduleInstanceRepetitionData = instances.TotallyUnknownRepetitionData
 
 		default:
 			// Get the repetition data for this module instance,
@@ -288,3 +311,59 @@ func (n *nodeModuleVariable) evalModuleVariable(ctx EvalContext, validateOnly bo
 
 	return finalVal, diags.ErrWithWarnings()
 }
+
+// nodeModuleVariableInPartialModule represents an infinite set of possible
+// input variable instances beneath a partially-expanded module instance prefix.
+//
+// Its job is to find a suitable placeholder value that approximates the
+// values of all of those possible instances. Ideally that's a concrete
+// known value if all instances would have the same value, an unknown value
+// of a specific type if the definition produces a known type, or a
+// totally-unknown value of unknown type in the worst case.
+type nodeModuleVariableInPartialModule struct {
+	Addr   addrs.InPartialExpandedModule[addrs.InputVariable]
+	Config *configs.Variable // Config is the var in the config
+	Expr   hcl.Expression    // Expr is the value expression given in the call
+	// ModuleInstance in order to create the appropriate context for evaluating
+	// ModuleCallArguments, ex. so count.index and each.key can resolve
+	ModuleInstance addrs.PartialExpandedModule
+
+	// DestroyApply must be set to true when applying a destroy operation and
+	// false otherwise.
+	DestroyApply bool
+}
+
+func (n *nodeModuleVariableInPartialModule) Path() addrs.PartialExpandedModule {
+	return n.Addr.Module
+}
+
+func (n *nodeModuleVariableInPartialModule) Execute(ctx EvalContext, op walkOperation) tfdiags.Diagnostics {
+	// Our job here is to make sure that the input variable definition is
+	// valid for all instances of this input variable across all of the possible
+	// module instances under our partially-expanded prefix, and to record
+	// a placeholder value that captures as precisely as possible what all
+	// of those results have in common. In the worst case where they have
+	// absolutely nothing in common cty.DynamicVal is the ultimate fallback,
+	// but we should try to do better when possible to give operators earlier
+	// feedback about any problems they would definitely encounter on a
+	// subsequent plan where the input variables get evaluated concretely.
+
+	namedVals := ctx.NamedValues()
+
+	// TODO: Ideally we should vary the placeholder we use here based
+	// on how the module call repetition was configured, but we don't
+	// have enough information here to decide that.
+	moduleInstanceRepetitionData := instances.TotallyUnknownRepetitionData
+
+	// NOTE WELL: Input variables are a little strange in that they announce
+	// themselves as belonging to the caller of the module they are declared
+	// in, because that's where their definition expressions get evaluated.
+	// Therefore this [EvalContext] is in the scope of the parent module,
+	// while n.Addr describes an object in the child module (where the
+	// variable declaration appeared).
+	scope := ctx.EvaluationScope(nil, nil, moduleInstanceRepetitionData)
+	val, diags := scope.EvalExpr(n.Expr, cty.DynamicPseudoType)
+
+	namedVals.SetInputVariablePlaceholder(n.Addr, val)
+	return diags
+}
