@@ -13,9 +13,12 @@ import (
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/dag"
-	"github.com/hashicorp/terraform/internal/lang"
+	"github.com/hashicorp/terraform/internal/lang/langrefs"
 	"github.com/hashicorp/terraform/internal/lang/marks"
+	"github.com/hashicorp/terraform/internal/moduletest/mocking"
+	"github.com/hashicorp/terraform/internal/namedvals"
 	"github.com/hashicorp/terraform/internal/plans"
+	"github.com/hashicorp/terraform/internal/plans/deferring"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
@@ -36,15 +39,23 @@ type nodeExpandOutput struct {
 	// we need to take between plan and apply. See method DynamicExpand for
 	// details.
 	Planning bool
+
+	// Overrides is the set of overrides applied by the testing framework. We
+	// may need to override the value for this output and if we do the value
+	// comes from here.
+	Overrides *mocking.Overrides
+
+	Dependencies []addrs.ConfigResource
 }
 
 var (
-	_ GraphNodeReferenceable     = (*nodeExpandOutput)(nil)
-	_ GraphNodeReferencer        = (*nodeExpandOutput)(nil)
-	_ GraphNodeReferenceOutside  = (*nodeExpandOutput)(nil)
-	_ GraphNodeDynamicExpandable = (*nodeExpandOutput)(nil)
-	_ graphNodeTemporaryValue    = (*nodeExpandOutput)(nil)
-	_ graphNodeExpandsInstances  = (*nodeExpandOutput)(nil)
+	_ GraphNodeReferenceable      = (*nodeExpandOutput)(nil)
+	_ GraphNodeReferencer         = (*nodeExpandOutput)(nil)
+	_ GraphNodeReferenceOutside   = (*nodeExpandOutput)(nil)
+	_ GraphNodeDynamicExpandable  = (*nodeExpandOutput)(nil)
+	_ graphNodeTemporaryValue     = (*nodeExpandOutput)(nil)
+	_ GraphNodeAttachDependencies = (*nodeExpandOutput)(nil)
+	_ graphNodeExpandsInstances   = (*nodeExpandOutput)(nil)
 )
 
 func (n *nodeExpandOutput) expandsInstances() {}
@@ -54,7 +65,12 @@ func (n *nodeExpandOutput) temporaryValue() bool {
 	return !n.Module.IsRoot()
 }
 
-func (n *nodeExpandOutput) DynamicExpand(ctx EvalContext) (*Graph, error) {
+// GraphNodeAttachDependencies
+func (n *nodeExpandOutput) AttachDependencies(resources []addrs.ConfigResource) {
+	n.Dependencies = resources
+}
+
+func (n *nodeExpandOutput) DynamicExpand(ctx EvalContext) (*Graph, tfdiags.Diagnostics) {
 	expander := ctx.InstanceExpander()
 	changes := ctx.Changes()
 
@@ -78,50 +94,69 @@ func (n *nodeExpandOutput) DynamicExpand(ctx EvalContext) (*Graph, error) {
 	}
 
 	var g Graph
-	for _, module := range expander.ExpandModule(n.Module) {
-		absAddr := n.Addr.Absolute(module)
-		if checkableAddrs != nil {
-			checkableAddrs.Add(absAddr)
-		}
-
-		// Find any recorded change for this output
-		var change *plans.OutputChangeSrc
-		var outputChanges []*plans.OutputChangeSrc
-		if module.IsRoot() {
-			outputChanges = changes.GetRootOutputChanges()
-		} else {
-			parent, call := module.Call()
-			outputChanges = changes.GetOutputChanges(parent, call)
-		}
-		for _, c := range outputChanges {
-			if c.Addr.String() == absAddr.String() {
-				change = c
-				break
-			}
-		}
-
-		var node dag.Vertex
-		switch {
-		case module.IsRoot() && n.Destroying:
-			node = &NodeDestroyableOutput{
-				Addr:     absAddr,
-				Planning: n.Planning,
+	forEachModuleInstance(
+		expander, n.Module, true,
+		func(module addrs.ModuleInstance) {
+			absAddr := n.Addr.Absolute(module)
+			if checkableAddrs != nil {
+				checkableAddrs.Add(absAddr)
 			}
 
-		default:
-			node = &NodeApplyableOutput{
-				Addr:         absAddr,
-				Config:       n.Config,
-				Change:       change,
-				RefreshOnly:  n.RefreshOnly,
-				DestroyApply: n.Destroying,
-				Planning:     n.Planning,
+			// Find any recorded change for this output
+			var change *plans.OutputChange
+			var outputChanges []*plans.OutputChange
+			if module.IsRoot() {
+				outputChanges = changes.GetRootOutputChanges()
+			} else {
+				parent, call := module.Call()
+				outputChanges = changes.GetOutputChanges(parent, call)
+			}
+			for _, c := range outputChanges {
+				if c.Addr.String() == absAddr.String() {
+					change = c
+					break
+				}
 			}
-		}
 
-		log.Printf("[TRACE] Expanding output: adding %s as %T", absAddr.String(), node)
-		g.Add(node)
-	}
+			var node dag.Vertex
+			switch {
+			case module.IsRoot() && n.Destroying:
+				node = &NodeDestroyableOutput{
+					Addr:     absAddr,
+					Planning: n.Planning,
+				}
+
+			default:
+				node = &NodeApplyableOutput{
+					Addr:         absAddr,
+					Config:       n.Config,
+					Change:       change,
+					RefreshOnly:  n.RefreshOnly,
+					DestroyApply: n.Destroying,
+					Planning:     n.Planning,
+					Override:     n.getOverrideValue(absAddr.Module),
+					Dependencies: n.Dependencies,
+				}
+			}
+
+			log.Printf("[TRACE] Expanding output: adding %s as %T", absAddr.String(), node)
+			g.Add(node)
+		},
+		func(pem addrs.PartialExpandedModule) {
+			absAddr := addrs.ObjectInPartialExpandedModule(pem, n.Addr)
+			node := &nodeOutputInPartialModule{
+				Addr:        absAddr,
+				Config:      n.Config,
+				RefreshOnly: n.RefreshOnly,
+			}
+			// We don't need to handle the module.IsRoot() && n.Destroying case
+			// seen in the fully-expanded case above, because the root module
+			// instance is always "fully expanded" (it's always a singleton)
+			// and so we can't get here for output values in the root module.
+			log.Printf("[TRACE] Expanding output: adding placeholder for all %s as %T", absAddr.String(), node)
+			g.Add(node)
+		},
+	)
 	addRootNodeToGraph(&g)
 
 	if checkableAddrs != nil {
@@ -188,13 +223,48 @@ func (n *nodeExpandOutput) References() []*addrs.Reference {
 	return referencesForOutput(n.Config)
 }
 
+func (n *nodeExpandOutput) getOverrideValue(inst addrs.ModuleInstance) cty.Value {
+	// First check if we have any overrides at all, this is a shorthand for
+	// "are we running terraform test".
+	if n.Overrides.Empty() {
+		// cty.NilVal means no override
+		return cty.NilVal
+	}
+
+	// We have overrides, let's see if we have one for this module instance.
+	if override, ok := n.Overrides.GetModuleOverride(inst); ok {
+
+		output := n.Addr.Name
+		values := override.Values
+
+		// The values.Type() should be an object type, but it might have
+		// been set to nil by a test or something. We can handle it in the
+		// same way as the attribute just not being specified. It's
+		// functionally the same for us and not something we need to raise
+		// alarms about.
+		if values.Type().IsObjectType() && values.Type().HasAttribute(output) {
+			return values.GetAttr(output)
+		}
+
+		// If we don't have a value provided for an output, then we'll
+		// just set it to be null.
+		//
+		// TODO(liamcervante): Can we generate a value here? Probably
+		//   not as we don't know the type.
+		return cty.NullVal(cty.DynamicPseudoType)
+	}
+
+	// cty.NilVal indicates no override.
+	return cty.NilVal
+}
+
 // NodeApplyableOutput represents an output that is "applyable":
 // it is ready to be applied.
 type NodeApplyableOutput struct {
 	Addr   addrs.AbsOutputValue
 	Config *configs.Output // Config is the output in the config
 	// If this is being evaluated during apply, we may have a change recorded already
-	Change *plans.OutputChangeSrc
+	Change *plans.OutputChange
 
 	// Refresh-only mode means that any failing output preconditions are
 	// reported as warnings rather than errors
@@ -205,6 +275,14 @@ type NodeApplyableOutput struct {
 	DestroyApply bool
 
 	Planning bool
+
+	// Override provides the value to use for this output, if any. This can be
+	// set by testing framework when a module is overridden.
+	Override cty.Value
+
+	// Dependencies is the full set of resources that are referenced by this
+	// output.
+	Dependencies []addrs.ConfigResource
 }
 
 var (
@@ -278,16 +356,16 @@ func (n *NodeApplyableOutput) ReferenceableAddrs() []addrs.Referenceable {
 func referencesForOutput(c *configs.Output) []*addrs.Reference {
 	var refs []*addrs.Reference
 
-	impRefs, _ := lang.ReferencesInExpr(addrs.ParseRef, c.Expr)
-	expRefs, _ := lang.References(addrs.ParseRef, c.DependsOn)
+	impRefs, _ := langrefs.ReferencesInExpr(addrs.ParseRef, c.Expr)
+	expRefs, _ := langrefs.References(addrs.ParseRef, c.DependsOn)
 
 	refs = append(refs, impRefs...)
 	refs = append(refs, expRefs...)
 
 	for _, check := range c.Preconditions {
-		condRefs, _ := lang.ReferencesInExpr(addrs.ParseRef, check.Condition)
+		condRefs, _ := langrefs.ReferencesInExpr(addrs.ParseRef, check.Condition)
 		refs = append(refs, condRefs...)
-		errRefs, _ := lang.ReferencesInExpr(addrs.ParseRef, check.ErrorMessage)
+		errRefs, _ := langrefs.ReferencesInExpr(addrs.ParseRef, check.ErrorMessage)
 		refs = append(refs, errRefs...)
 	}
 
@@ -313,16 +391,24 @@ func (n *NodeApplyableOutput) Execute(ctx EvalContext, op walkOperation) (diags
 	// we we have a change recorded, we don't need to re-evaluate if the value
 	// was known
 	if changeRecorded {
-		change, err := n.Change.Decode()
-		diags = diags.Append(err)
-		if err == nil {
-			val = change.After
-		}
+		val = n.Change.After
+	}
+
+	if n.Addr.Module.IsRoot() && n.Config.Ephemeral {
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Ephemeral output not allowed",
+			Detail:   "Ephemeral outputs are not allowed in context of a root module",
+			Subject:  n.Config.DeclRange.Ptr(),
+		})
+		return
 	}
 
 	// Checks are not evaluated during a destroy. The checks may fail, may not
 	// be valid, or may not have been registered at all.
-	if !n.DestroyApply {
+	// We also don't evaluate checks for overridden outputs. This is because
+	// any references within the checks will likely not have been created.
+	if !n.DestroyApply && n.Override == cty.NilVal {
 		checkRuleSeverity := tfdiags.Error
 		if n.RefreshOnly {
 			checkRuleSeverity = tfdiags.Warning
@@ -342,32 +428,38 @@ func (n *NodeApplyableOutput) Execute(ctx EvalContext, op walkOperation) (diags
 	// If there was no change recorded, or the recorded change was not wholly
 	// known, then we need to re-evaluate the output
 	if !changeRecorded || !val.IsWhollyKnown() {
-		// This has to run before we have a state lock, since evaluation also
-		// reads the state
-		var evalDiags tfdiags.Diagnostics
-		val, evalDiags = ctx.EvaluateExpr(n.Config.Expr, cty.DynamicPseudoType, nil)
-		diags = diags.Append(evalDiags)
 
-		// We'll handle errors below, after we have loaded the module.
-		// Outputs don't have a separate mode for validation, so validate
-		// depends_on expressions here too
-		diags = diags.Append(validateDependsOn(ctx, n.Config.DependsOn))
+		// First, we check if we have an overridden value. If we do, then we
+		// use that and we don't try and evaluate the underlying expression.
+		val = n.Override
+		if val == cty.NilVal {
+			// This has to run before we have a state lock, since evaluation also
+			// reads the state
+			var evalDiags tfdiags.Diagnostics
+			val, evalDiags = ctx.EvaluateExpr(n.Config.Expr, cty.DynamicPseudoType, nil)
+			diags = diags.Append(evalDiags)
 
-		// For root module outputs in particular, an output value must be
-		// statically declared as sensitive in order to dynamically return
-		// a sensitive result, to help avoid accidental exposure in the state
-		// of a sensitive value that the user doesn't want to include there.
-		if n.Addr.Module.IsRoot() {
-			if !n.Config.Sensitive && marks.Contains(val, marks.Sensitive) {
-				diags = diags.Append(&hcl.Diagnostic{
-					Severity: hcl.DiagError,
-					Summary:  "Output refers to sensitive values",
-					Detail: `To reduce the risk of accidentally exporting sensitive data that was intended to be only internal, Terraform requires that any root module output containing sensitive data be explicitly marked as sensitive, to confirm your intent.
+			// We'll handle errors below, after we have loaded the module.
+			// Outputs don't have a separate mode for validation, so validate
+			// depends_on expressions here too
+			diags = diags.Append(validateDependsOn(ctx, n.Config.DependsOn))
+
+			// For root module outputs in particular, an output value must be
+			// statically declared as sensitive in order to dynamically return
+			// a sensitive result, to help avoid accidental exposure in the state
+			// of a sensitive value that the user doesn't want to include there.
+			if n.Addr.Module.IsRoot() {
+				if !n.Config.Sensitive && marks.Contains(val, marks.Sensitive) {
+					diags = diags.Append(&hcl.Diagnostic{
+						Severity: hcl.DiagError,
+						Summary:  "Output refers to sensitive values",
+						Detail: `To reduce the risk of accidentally exporting sensitive data that was intended to be only internal, Terraform requires that any root module output containing sensitive data be explicitly marked as sensitive, to confirm your intent.
 
 If you do intend to export this data, annotate the output value as sensitive by adding the following argument:
     sensitive = true`,
-					Subject: n.Config.DeclRange.Ptr(),
-				})
+						Subject: n.Config.DeclRange.Ptr(),
+					})
+				}
 			}
 		}
 	}
@@ -379,7 +471,7 @@ If you do intend to export this data, annotate the output value as sensitive by
 			// if we're continuing, make sure the output is included, and
 			// marked as unknown. If the evaluator was able to find a type
 			// for the value in spite of the error then we'll use it.
-			n.setValue(state, changes, cty.UnknownVal(val.Type()))
+			n.setValue(ctx.NamedValues(), state, changes, ctx.Deferrals(), cty.UnknownVal(val.Type()))
 
 			// Keep existing warnings, while converting errors to warnings.
 			// This is not meant to be the normal path, so there no need to
@@ -399,13 +491,39 @@ If you do intend to export this data, annotate the output value as sensitive by
 		}
 		return diags
 	}
-	n.setValue(state, changes, val)
+
+	// The checks below this point are intentionally not opted out by
+	// "flagWarnOutputErrors", because they relate to features that were added
+	// more recently than the historical change to treat invalid output values
+	// as errors rather than warnings.
+	if n.Config.Ephemeral && !marks.Has(val, marks.Ephemeral) {
+		// An ephemeral output value must always be ephemeral
+		// This is to prevent accidental persistence upstream
+		// from here.
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Value not allowed in ephemeral output",
+			Detail:   "This output value is declared as returning an ephemeral value, so it can only be set to an ephemeral value.",
+			Subject:  n.Config.Expr.Range().Ptr(),
+		})
+		return diags
+	} else if !n.Config.Ephemeral && marks.Contains(val, marks.Ephemeral) {
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Ephemeral value not allowed",
+			Detail:   "This output value is not declared as returning an ephemeral value, so it cannot be set to a result derived from an ephemeral value.",
+			Subject:  n.Config.Expr.Range().Ptr(),
+		})
+		return diags
+	}
+
+	n.setValue(ctx.NamedValues(), state, changes, ctx.Deferrals(), val)
 
 	// If we were able to evaluate a new value, we can update that in the
 	// refreshed state as well.
 	if state = ctx.RefreshState(); state != nil && val.IsWhollyKnown() {
 		// we only need to update the state, do not pass in the changes again
-		n.setValue(state, nil, val)
+		n.setValue(nil, state, nil, ctx.Deferrals(), val)
 	}
 
 	return diags
@@ -422,6 +540,61 @@ func (n *NodeApplyableOutput) DotNode(name string, opts *dag.DotOpts) *dag.DotNo
 	}
 }
 
+// nodeOutputInPartialModule represents an infinite set of possible output value
+// instances beneath a partially-expanded module instance prefix.
+//
+// Its job is to find a suitable placeholder value that approximates the
+// values of all of those possible instances. Ideally that's a concrete
+// known value if all instances would have the same value, an unknown value
+// of a specific type if the definition produces a known type, or a
+// totally-unknown value of unknown type in the worst case.
+type nodeOutputInPartialModule struct {
+	Addr   addrs.InPartialExpandedModule[addrs.OutputValue]
+	Config *configs.Output
+
+	// Refresh-only mode means that any failing output preconditions are
+	// reported as warnings rather than errors
+	RefreshOnly bool
+}
+
+// Path implements [GraphNodePartialExpandedModule], meaning that the
+// Execute method receives an [EvalContext] that's set up for partial-expanded
+// evaluation instead of full evaluation.
+func (n *nodeOutputInPartialModule) Path() addrs.PartialExpandedModule {
+	return n.Addr.Module
+}
+
+func (n *nodeOutputInPartialModule) Execute(ctx EvalContext, op walkOperation) tfdiags.Diagnostics {
+	// Our job here is to make sure that the output value definition is
+	// valid for all instances of this output value across all of the possible
+	// module instances under our partially-expanded prefix, and to record
+	// a placeholder value that captures as precisely as possible what all
+	// of those results have in common. In the worst case where they have
+	// absolutely nothing in common cty.DynamicVal is the ultimate fallback,
+	// but we should try to do better when possible to give operators earlier
+	// feedback about any problems they would definitely encounter on a
+	// subsequent plan where the output values get evaluated concretely.
+
+	namedVals := ctx.NamedValues()
+
+	// this "ctx" is preconfigured to evaluate in terms of other placeholder
+	// values generated in the same unexpanded module prefix, rather than
+	// from the active state/plan, so this result is likely to be derived
+	// from unknown value placeholders itself.
+	val, diags := ctx.EvaluateExpr(n.Config.Expr, cty.DynamicPseudoType, nil)
+	if val == cty.NilVal {
+		val = cty.DynamicVal
+	}
+
+	// We'll also check that the depends_on argument is valid, since that's
+	// a static concern anyway and so cannot vary between instances of the
+	// same module.
+	diags = diags.Append(validateDependsOn(ctx, n.Config.DependsOn))
+
+	namedVals.SetOutputValuePlaceholder(n.Addr, val)
+	return diags
+}
+
 // NodeDestroyableOutput represents an output that is "destroyable":
 // its application will remove the output from the state.
 type NodeDestroyableOutput struct {
@@ -461,15 +634,18 @@ func (n *NodeDestroyableOutput) Execute(ctx EvalContext, op walkOperation) tfdia
 	before := cty.NullVal(cty.DynamicPseudoType)
 	mod := state.Module(n.Addr.Module)
 	if n.Addr.Module.IsRoot() && mod != nil {
-		if o, ok := mod.OutputValues[n.Addr.OutputValue.Name]; ok {
+		s := state.Lock()
+		rootOutputs := s.RootOutputValues
+		if o, ok := rootOutputs[n.Addr.OutputValue.Name]; ok {
 			sensitiveBefore = o.Sensitive
 			before = o.Value
 		} else {
 			// If the output was not in state, a delete change would
 			// be meaningless, so exit early.
+			state.Unlock()
 			return nil
-
 		}
+		state.Unlock()
 	}
 
 	changes := ctx.Changes()
@@ -484,18 +660,11 @@ func (n *NodeDestroyableOutput) Execute(ctx EvalContext, op walkOperation) tfdia
 			},
 		}
 
-		cs, err := change.Encode()
-		if err != nil {
-			// Should never happen, since we just constructed this right above
-			panic(fmt.Sprintf("planned change for %s could not be encoded: %s", n.Addr, err))
-		}
-		log.Printf("[TRACE] NodeDestroyableOutput: Saving %s change for %s in changeset", change.Action, n.Addr)
-
 		changes.RemoveOutputChange(n.Addr) // remove any existing planned change, if present
-		changes.AppendOutputChange(cs)     // add the new planned change
+		changes.AppendOutputChange(change) // add the new planned change
 	}
-
 	state.RemoveOutputValue(n.Addr)
+
 	return nil
 }
 
@@ -510,7 +679,7 @@ func (n *NodeDestroyableOutput) DotNode(name string, opts *dag.DotOpts) *dag.Dot
 	}
 }
 
-func (n *NodeApplyableOutput) setValue(state *states.SyncState, changes *plans.ChangesSync, val cty.Value) {
+func (n *NodeApplyableOutput) setValue(namedVals *namedvals.State, state *states.SyncState, changes *plans.ChangesSync, deferred *deferring.Deferred, val cty.Value) {
 	if changes != nil && n.Planning {
 		// if this is a root module, try to get a before value from the state for
 		// the diff
@@ -522,7 +691,9 @@ func (n *NodeApplyableOutput) setValue(state *states.SyncState, changes *plans.C
 
 		mod := state.Module(n.Addr.Module)
 		if n.Addr.Module.IsRoot() && mod != nil {
-			for name, o := range mod.OutputValues {
+			s := state.Lock()
+			rootOutputs := s.RootOutputValues
+			for name, o := range rootOutputs {
 				if name == n.Addr.OutputValue.Name {
 					before = o.Value
 					sensitiveBefore = o.Sensitive
@@ -530,6 +701,7 @@ func (n *NodeApplyableOutput) setValue(state *states.SyncState, changes *plans.C
 					break
 				}
 			}
+			state.Unlock()
 		}
 
 		// We will not show the value if either the before or after are marked
@@ -561,23 +733,21 @@ func (n *NodeApplyableOutput) setValue(state *states.SyncState, changes *plans.C
 			action = plans.NoOp
 		}
 
-		change := &plans.OutputChange{
-			Addr:      n.Addr,
-			Sensitive: sensitiveChange,
-			Change: plans.Change{
-				Action: action,
-				Before: before,
-				After:  val,
-			},
-		}
+		// Non-ephemeral output values get their changes recorded in the plan
+		if !n.Config.Ephemeral {
+			change := &plans.OutputChange{
+				Addr:      n.Addr,
+				Sensitive: sensitiveChange,
+				Change: plans.Change{
+					Action: action,
+					Before: before,
+					After:  val,
+				},
+			}
 
-		cs, err := change.Encode()
-		if err != nil {
-			// Should never happen, since we just constructed this right above
-			panic(fmt.Sprintf("planned change for %s could not be encoded: %s", n.Addr, err))
+			log.Printf("[TRACE] setValue: Saving %s change for %s in changeset", change.Action, n.Addr)
+			changes.AppendOutputChange(change) // add the new planned change
 		}
-		log.Printf("[TRACE] setValue: Saving %s change for %s in changeset", change.Action, n.Addr)
-		changes.AppendOutputChange(cs) // add the new planned change
 	}
 
 	if changes != nil && !n.Planning {
@@ -595,14 +765,38 @@ func (n *NodeApplyableOutput) setValue(state *states.SyncState, changes *plans.C
 		return
 	}
 
-	log.Printf("[TRACE] setValue: Saving value for %s in state", n.Addr)
-
-	// non-root outputs need to keep sensitive marks for evaluation, but are
-	// not serialized.
-	if n.Addr.Module.IsRoot() {
-		val, _ = val.UnmarkDeep()
-		val = cty.UnknownAsNull(val)
+	// caller leaves namedVals nil if they've already called this function
+	// with a different state, since we only have one namedVals regardless
+	// of how many states are involved in an operation.
+	if namedVals != nil {
+		saveVal := val
+		if n.Config.Ephemeral {
+			// Downstream uses of this output value must propagate the
+			// ephemerality.
+			saveVal = saveVal.Mark(marks.Ephemeral)
+		}
+		namedVals.SetOutputValue(n.Addr, saveVal)
 	}
 
+	// Non-ephemeral output values get saved in the state too
+	if !n.Config.Ephemeral {
+		// The state itself doesn't represent unknown values, so we null them
+		// out here and then we'll save the real unknown value in the planned
+		// changeset, if we have one on this graph walk.
+		log.Printf("[TRACE] setValue: Saving value for %s in state", n.Addr)
+		// non-root outputs need to keep sensitive marks for evaluation, but are
+		// not serialized.
+		if n.Addr.Module.IsRoot() {
+			val, _ = val.UnmarkDeep()
+			if deferred.DependenciesDeferred(n.Dependencies) {
+				// If the output is from deferred resources then we return a
+				// simple null value representing that the value is really
+				// unknown as the dependencies were not properly computed.
+				val = cty.NullVal(val.Type())
+			} else {
+				val = cty.UnknownAsNull(val)
+			}
+		}
+	}
 	state.SetOutputValue(n.Addr, val, n.Config.Sensitive)
 }
