@@ -15,7 +15,8 @@ import (
 	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/didyoumean"
 	"github.com/hashicorp/terraform/internal/instances"
-	"github.com/hashicorp/terraform/internal/lang"
+	"github.com/hashicorp/terraform/internal/lang/ephemeral"
+	"github.com/hashicorp/terraform/internal/lang/langrefs"
 	"github.com/hashicorp/terraform/internal/providers"
 	"github.com/hashicorp/terraform/internal/provisioners"
 	"github.com/hashicorp/terraform/internal/tfdiags"
@@ -45,6 +46,9 @@ func (n *NodeValidatableResource) Path() addrs.ModuleInstance {
 
 // GraphNodeEvalable
 func (n *NodeValidatableResource) Execute(ctx EvalContext, op walkOperation) (diags tfdiags.Diagnostics) {
+	// this is done first since there may not be config if we are generating it
+	diags = diags.Append(n.validateImportTargets(ctx))
+
 	if n.Config == nil {
 		return diags
 	}
@@ -296,7 +300,7 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 
 		// Basic type-checking of the count argument. More complete validation
 		// of this will happen when we DynamicExpand during the plan walk.
-		countDiags := validateCount(ctx, n.Config.Count)
+		_, countDiags := evaluateCountExpressionValue(n.Config.Count, ctx)
 		diags = diags.Append(countDiags)
 
 	case n.Config.ForEach != nil:
@@ -306,46 +310,12 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 		}
 
 		// Evaluate the for_each expression here so we can expose the diagnostics
-		forEachDiags := validateForEach(ctx, n.Config.ForEach)
+		forEachDiags := newForEachEvaluator(n.Config.ForEach, ctx, false).ValidateResourceValue()
 		diags = diags.Append(forEachDiags)
 	}
 
 	diags = diags.Append(validateDependsOn(ctx, n.Config.DependsOn))
 
-	// Validate the provider_meta block for the provider this resource
-	// belongs to, if there is one.
-	//
-	// Note: this will return an error for every resource a provider
-	// uses in a module, if the provider_meta for that module is
-	// incorrect. The only way to solve this that we've found is to
-	// insert a new ProviderMeta graph node in the graph, and make all
-	// that provider's resources in the module depend on the node. That's
-	// an awful heavy hammer to swing for this feature, which should be
-	// used only in limited cases with heavy coordination with the
-	// Terraform team, so we're going to defer that solution for a future
-	// enhancement to this functionality.
-	/*
-		if n.ProviderMetas != nil {
-			if m, ok := n.ProviderMetas[n.ProviderAddr.ProviderConfig.Type]; ok && m != nil {
-				// if the provider doesn't support this feature, throw an error
-				if (*n.ProviderSchema).ProviderMeta == nil {
-					diags = diags.Append(&hcl.Diagnostic{
-						Severity: hcl.DiagError,
-						Summary:  fmt.Sprintf("Provider %s doesn't support provider_meta", cfg.ProviderConfigAddr()),
-						Detail:   fmt.Sprintf("The resource %s belongs to a provider that doesn't support provider_meta blocks", n.Addr),
-						Subject:  &m.ProviderRange,
-					})
-				} else {
-					_, _, metaDiags := ctx.EvaluateBlock(m.Config, (*n.ProviderSchema).ProviderMeta, nil, EvalDataForNoInstanceKey)
-					diags = diags.Append(metaDiags)
-				}
-			}
-		}
-	*/
-	// BUG(paddy): we're not validating provider_meta blocks on EvalValidate right now
-	// because the ProviderAddr for the resource isn't available on the EvalValidate
-	// struct.
-
 	// Provider entry point varies depending on resource mode, because
 	// managed resources and data resources are two distinct concepts
 	// in the provider abstraction.
@@ -380,6 +350,9 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 		if valDiags.HasErrors() {
 			return diags
 		}
+		diags = diags.Append(
+			validateResourceForbiddenEphemeralValues(ctx, configVal, schema).InConfigBody(n.Config.Config, n.Addr.String()),
+		)
 
 		if n.Config.Managed != nil { // can be nil only in tests with poorly-configured mocks
 			for _, traversal := range n.Config.Managed.IgnoreChanges {
@@ -393,7 +366,7 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 				// use that to check whether the Attribute is Computed and
 				// non-Optional.
 				if !diags.HasErrors() {
-					path := traversalToPath(traversal)
+					path, _ := traversalToPath(traversal)
 
 					attrSchema := schema.AttributeByPath(path)
 
@@ -417,8 +390,9 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 		// Use unmarked value for validate request
 		unmarkedConfigVal, _ := configVal.UnmarkDeep()
 		req := providers.ValidateResourceConfigRequest{
-			TypeName: n.Config.Type,
-			Config:   unmarkedConfigVal,
+			TypeName:           n.Config.Type,
+			Config:             unmarkedConfigVal,
+			ClientCapabilities: ctx.ClientCapabilities(),
 		}
 
 		resp := provider.ValidateResourceConfig(req)
@@ -454,6 +428,9 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 		if valDiags.HasErrors() {
 			return diags
 		}
+		diags = diags.Append(
+			validateResourceForbiddenEphemeralValues(ctx, configVal, schema).InConfigBody(n.Config.Config, n.Addr.String()),
+		)
 
 		// Use unmarked value for validate request
 		unmarkedConfigVal, _ := configVal.UnmarkDeep()
@@ -464,6 +441,32 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 
 		resp := provider.ValidateDataResourceConfig(req)
 		diags = diags.Append(resp.Diagnostics.InConfigBody(n.Config.Config, n.Addr.String()))
+	case addrs.EphemeralResourceMode:
+		schema, _ := providerSchema.SchemaForResourceType(n.Config.Mode, n.Config.Type)
+		if schema == nil {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid ephemeral resource",
+				Detail:   fmt.Sprintf("The provider %s does not support ephemeral resource %q.", n.Provider().ForDisplay(), n.Config.Type),
+				Subject:  &n.Config.TypeRange,
+			})
+			return diags
+		}
+
+		configVal, _, valDiags := ctx.EvaluateBlock(n.Config.Config, schema, nil, keyData)
+		diags = diags.Append(valDiags)
+		if valDiags.HasErrors() {
+			return diags
+		}
+		// Use unmarked value for validate request
+		unmarkedConfigVal, _ := configVal.UnmarkDeep()
+		req := providers.ValidateEphemeralResourceConfigRequest{
+			TypeName: n.Config.Type,
+			Config:   unmarkedConfigVal,
+		}
+
+		resp := provider.ValidateEphemeralResourceConfig(req)
+		diags = diags.Append(resp.Diagnostics.InConfigBody(n.Config.Config, n.Addr.String()))
 	}
 
 	return diags
@@ -472,7 +475,7 @@ func (n *NodeValidatableResource) validateResource(ctx EvalContext) tfdiags.Diag
 func (n *NodeValidatableResource) evaluateExpr(ctx EvalContext, expr hcl.Expression, wantTy cty.Type, self addrs.Referenceable, keyData instances.RepetitionData) (cty.Value, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
-	refs, refDiags := lang.ReferencesInExpr(addrs.ParseRef, expr)
+	refs, refDiags := langrefs.ReferencesInExpr(addrs.ParseRef, expr)
 	diags = diags.Append(refDiags)
 
 	scope := ctx.EvaluationScope(self, nil, keyData)
@@ -544,33 +547,171 @@ func (n *NodeValidatableResource) validateCheckRules(ctx EvalContext, config *co
 	return diags
 }
 
-func validateCount(ctx EvalContext, expr hcl.Expression) (diags tfdiags.Diagnostics) {
-	val, countDiags := evaluateCountExpressionValue(expr, ctx)
-	// If the value isn't known then that's the best we can do for now, but
-	// we'll check more thoroughly during the plan walk
-	if !val.IsKnown() {
+// validateImportTargets checks that the import block expressions are valid.
+func (n *NodeValidatableResource) validateImportTargets(ctx EvalContext) tfdiags.Diagnostics {
+	var diags tfdiags.Diagnostics
+
+	if len(n.importTargets) == 0 {
 		return diags
 	}
 
-	if countDiags.HasErrors() {
-		diags = diags.Append(countDiags)
+	diags = diags.Append(n.validateConfigGen(ctx))
+
+	// Import blocks are only valid within the root module, and must be
+	// evaluated within that context
+	ctx = evalContextForModuleInstance(ctx, addrs.RootModuleInstance)
+
+	for _, imp := range n.importTargets {
+		if imp.Config == nil {
+			// if we have a legacy addr, it will be supplied on the command line so
+			// there is nothing to check now and we need to wait for plan.
+			continue
+		}
+
+		diags = diags.Append(validateImportSelfRef(n.Addr.Resource, imp.Config.ID))
+		if diags.HasErrors() {
+			return diags
+		}
+
+		if imp.Config.ForEach != nil {
+			forEachData, _, forEachDiags := newForEachEvaluator(imp.Config.ForEach, ctx, true).ImportValues()
+			diags = diags.Append(forEachDiags)
+			if forEachDiags.HasErrors() {
+				return diags
+			}
+
+			for _, keyData := range forEachData {
+				to, evalDiags := evalImportToExpression(imp.Config.To, keyData)
+				diags = diags.Append(evalDiags)
+				if diags.HasErrors() {
+					return diags
+				}
+				diags = diags.Append(validateImportTargetExpansion(n.Config, to, imp.Config.To))
+				if diags.HasErrors() {
+					return diags
+				}
+				_, evalDiags = evaluateImportIdExpression(imp.Config.ID, ctx, keyData, true)
+				diags = diags.Append(evalDiags)
+				if diags.HasErrors() {
+					return diags
+				}
+			}
+		} else {
+			traversal, hds := hcl.AbsTraversalForExpr(imp.Config.To)
+			diags = diags.Append(hds)
+			to, tds := addrs.ParseAbsResourceInstance(traversal)
+			diags = diags.Append(tds)
+			if diags.HasErrors() {
+				return diags
+			}
+
+			diags = diags.Append(validateImportTargetExpansion(n.Config, to, imp.Config.To))
+			if diags.HasErrors() {
+				return diags
+			}
+
+			_, evalDiags := evaluateImportIdExpression(imp.Config.ID, ctx, EvalDataForNoInstanceKey, true)
+			diags = diags.Append(evalDiags)
+			if diags.HasErrors() {
+				return diags
+			}
+		}
 	}
 
 	return diags
 }
 
-func validateForEach(ctx EvalContext, expr hcl.Expression) (diags tfdiags.Diagnostics) {
-	val, forEachDiags := evaluateForEachExpressionValue(expr, ctx, true)
-	// If the value isn't known then that's the best we can do for now, but
-	// we'll check more thoroughly during the plan walk
-	if !val.IsKnown() {
+// validateImportTargetExpansion ensures that the To address key and resource expansion mode both agree.
+func validateImportTargetExpansion(cfg *configs.Resource, to addrs.AbsResourceInstance, toExpr hcl.Expression) tfdiags.Diagnostics {
+	var diags tfdiags.Diagnostics
+
+	forEach := cfg != nil && cfg.ForEach != nil
+	count := cfg != nil && cfg.Count != nil
+
+	switch to.Resource.Key.(type) {
+	case addrs.StringKey:
+		if !forEach {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid import 'to' expression",
+				Detail:   "The target resource does not use for_each.",
+				Subject:  toExpr.Range().Ptr(),
+			})
+		}
+	case addrs.IntKey:
+		if !count {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid import 'to' expression",
+				Detail:   "The target resource does not use count.",
+				Subject:  toExpr.Range().Ptr(),
+			})
+		}
+	default:
+		if forEach {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid import 'to' expression",
+				Detail:   "The target resource is using for_each.",
+				Subject:  toExpr.Range().Ptr(),
+			})
+		}
+
+		if count {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid import 'to' expression",
+				Detail:   "The target resource is using count.",
+				Subject:  toExpr.Range().Ptr(),
+			})
+		}
+	}
+
+	return diags
+}
+
+// validate imports with no config for possible config generation
+func (n *NodeValidatableResource) validateConfigGen(ctx EvalContext) tfdiags.Diagnostics {
+	var diags tfdiags.Diagnostics
+
+	if n.Config != nil {
 		return diags
 	}
 
-	if forEachDiags.HasErrors() {
-		diags = diags.Append(forEachDiags)
-	}
+	// We won't have the config generation output path during validate, so only
+	// check if generation is at all possible.
 
+	for _, imp := range n.importTargets {
+		if !n.Addr.Module.IsRoot() {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Configuration for import target does not exist",
+				Detail:   fmt.Sprintf("Resource %s not found. Only resources within the root module are eligible for config generation.", n.Addr),
+				Subject:  imp.Config.To.Range().Ptr(),
+			})
+			continue
+		}
+
+		var toDiags tfdiags.Diagnostics
+		traversal, hd := hcl.AbsTraversalForExpr(imp.Config.To)
+		toDiags = toDiags.Append(hd)
+		to, td := addrs.ParseAbsResourceInstance(traversal)
+		toDiags = toDiags.Append(td)
+
+		if toDiags.HasErrors() {
+			// these will be caught elsewhere with better context
+			continue
+		}
+
+		if to.Resource.Key != addrs.NoKey || imp.Config.ForEach != nil {
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Configuration for import target does not exist",
+				Detail:   "The given import block is not compatible with config generation. The -generate-config-out option cannot be used with import blocks which use for_each, or resources which use for_each or count.",
+				Subject:  imp.Config.To.Range().Ptr(),
+			})
+		}
+	}
 	return diags
 }
 
@@ -600,3 +741,49 @@ func validateDependsOn(ctx EvalContext, dependsOn []hcl.Traversal) (diags tfdiag
 	}
 	return diags
 }
+
+// validateResourceForbiddenEphemeralValues returns an error diagnostic for each
+// value anywhere inside the given value that is marked as ephemeral, for
+// situations where ephemeral values are not permitted.
+//
+// All returned diagnostics are contextual diagnostics that must be finalized
+// by calling [tfdiags.Diagnostics.InConfigBody] before returning them to
+// any caller that expects fully-resolved diagnostics.
+func validateResourceForbiddenEphemeralValues(ctx EvalContext, value cty.Value, schema *configschema.Block) (diags tfdiags.Diagnostics) {
+	for _, path := range ephemeral.EphemeralValuePaths(value) {
+		attr := schema.AttributeByPath(path)
+
+		// If this attribute doesn't exist (usually through a dynamic type) or
+		// itself isn't write-only, it may be nested within a more complex type
+		// which is write-only. We need to walk upwards through the path
+		// segments and see of something wrapping this path is write-only.
+		for (attr == nil || !attr.WriteOnly) && len(path) > 1 {
+			path = path[:len(path)-1]
+			attr = schema.AttributeByPath((path))
+		}
+
+		// We know the config decoded, so the "attribute" exists in the
+		// schema somehow. If the ephemeral mark ended up being hoisted into
+		// a container however, especially if that container is a block,
+		// it's not actually an assignable attribute so we need to make a
+		// generic sounding error with a little more context because the
+		// AttributeValue diagnostic won't point to anything except the
+		// resource block.
+		if attr == nil {
+			diags = diags.Append(tfdiags.AttributeValue(
+				tfdiags.Error,
+				"Invalid use of ephemeral value",
+				fmt.Sprintf("Ephemeral values are not valid for %q, because it is not an assignable attribute.", strings.TrimPrefix(tfdiags.FormatCtyPath(path), ".")),
+				path,
+			))
+		} else if !attr.WriteOnly {
+			diags = diags.Append(tfdiags.AttributeValue(
+				tfdiags.Error,
+				"Invalid use of ephemeral value",
+				fmt.Sprintf("Ephemeral values are not valid for %q, because it is not a write-only attribute and must be persisted to state.", strings.TrimPrefix(tfdiags.FormatCtyPath(path), ".")),
+				path,
+			))
+		}
+	}
+	return diags
+}
